@@ -283,9 +283,8 @@ void F77_FUNC(gdptco, GDPTCO) (const int32_t *dirfile, const char *field_code,
 void F77_FUNC(gdptca, GDPTCA) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const int32_t *data_type, const void *data_in);
 
-void F77_FUNC(gdptst, GDPTST) (int32_t *n_read, const int32_t *dirfile,
-    const char *field_code, const int32_t *field_code_l, const int32_t *len,
-    const char *data_out);
+void F77_FUNC(gdptst, GDPTST) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const int32_t *len, const char *data_out);
 
 void F77_FUNC(gdnmfd, GDNMFD) (int32_t *nfields, const int32_t *dirfile,
     const char *parent, const int32_t *parent_l);
@@ -660,6 +659,10 @@ void F77_FUNC(gdinca, GDINCA) (const int32_t *dirfile, const char *file,
     const int32_t *prefix_l, const char *suffix, const int32_t *suffix_l,
     const int32_t *flags);
 
+void F77_FUNC(gdincn, GDINCN) (const int32_t *dirfile, const char *file,
+    const int32_t *file_l, const int32_t *fragment_index, const char *nsin,
+    const int32_t *ns_l, const int32_t *flags);
+
 void F77_FUNC(gdmova, GDMOVA) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const int32_t *new_fragment);
 
@@ -679,6 +682,10 @@ void F77_FUNC(gdhidn, GDHIDN) (int32_t *result, const int32_t *dirfile,
 void F77_FUNC(gdfraf, GDFRAF) (char *prefix, int32_t *prefix_l, char *suffix,
     int32_t *suffix_l, const int32_t *dirfile, const int32_t *index);
 
+void F77_FUNC(gdfrns, GDFRNS) (char *nsout, int32_t *nsout_l,
+    const int32_t *dirfile, const int32_t *index, const char *nsin,
+    const int32_t *nsin_l);
+
 void F77_FUNC(gdaafx, GDAAFX) (const int32_t *dirfile, const int32_t *index,
     const char *prefix, const int32_t *prefix_l, const char *suffix,
     const int32_t *suffix_l);
@@ -789,6 +796,21 @@ void F77_FUNC(gdascl, GDASCL) (const int32_t *dirfile, const char *field_code,
     const GD_DCOMPLEXP(b3), const char *b3_scalar, const int32_t *b3_scalar_l,
     const int32_t *b3_scalar_ind, const int32_t *fragment_index);
 
+void F77_FUNC(gdascl, GDASCL) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const int32_t *n_fields, const char *in_field1,
+    const int32_t *in_field1_l, const GD_DCOMPLEXP(m1), const char *m1_scalar,
+    const int32_t *m1_scalar_l, const int32_t *m1_scalar_ind,
+    const GD_DCOMPLEXP(b1), const char *b1_scalar, const int32_t *b1_scalar_l,
+    const int32_t *b1_scalar_ind, const char *in_field2,
+    const int32_t *in_field2_l, const GD_DCOMPLEXP(m2), const char *m2_scalar,
+    const int32_t *m2_scalar_l, const int32_t *m2_scalar_ind,
+    const GD_DCOMPLEXP(b2), const char *b2_scalar, const int32_t *b2_scalar_l,
+    const int32_t *b2_scalar_ind, const char *in_field3,
+    const int32_t *in_field3_l, const GD_DCOMPLEXP(m3), const char *m3_scalar,
+    const int32_t *m3_scalar_l, const int32_t *m3_scalar_ind,
+    const GD_DCOMPLEXP(b3), const char *b3_scalar, const int32_t *b3_scalar_l,
+    const int32_t *b3_scalar_ind, const int32_t *fragment_index);
+
 void F77_FUNC(gdasph, GDASPH) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const char *in_field,
     const int32_t *in_field_l, const int32_t *shift,
@@ -817,6 +839,12 @@ void F77_FUNC(gdasrc, GDASCR) (const int32_t *dirfile, const char *field_code,
     const char *dividend_scalar, const int32_t *dividend_scalar_l,
     const int32_t *dividend_scalar_ind, const int32_t *fragment_index);
 
+void F77_FUNC(gdascr, GDASCR) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field,
+    const int32_t *in_field_l, const double *dividend,
+    const char *dividend_scalar, const int32_t *dividend_scalar_l,
+    const int32_t *dividend_scalar_ind, const int32_t *fragment_index);
+
 void F77_FUNC(gdlsrw, GDLSRW) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const int32_t *data_type, const int32_t *spf,
     const char *spf_scalar, const int32_t *spf_scalar_l,
@@ -859,6 +887,21 @@ void F77_FUNC(gdlscl, GDLSCL) (const int32_t *dirfile, const char *field_code,
     const GD_DCOMPLEXP(b3), const char *b3_scalar, const int32_t *b3_scalar_l,
     const int32_t *b3_scalar_ind);
 
+void F77_FUNC(gdlscl, GDLSCL) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const int32_t *n_fields, const char *in_field1,
+    const int32_t *in_field1_l, const GD_DCOMPLEXP(m1), const char *m1_scalar,
+    const int32_t *m1_scalar_l, const int32_t *m1_scalar_ind,
+    const GD_DCOMPLEXP(b1), const char *b1_scalar, const int32_t *b1_scalar_l,
+    const int32_t *b1_scalar_ind, const char *in_field2,
+    const int32_t *in_field2_l, const GD_DCOMPLEXP(m2), const char *m2_scalar,
+    const int32_t *m2_scalar_l, const int32_t *m2_scalar_ind,
+    const GD_DCOMPLEXP(b2), const char *b2_scalar, const int32_t *b2_scalar_l,
+    const int32_t *b2_scalar_ind, const char *in_field3,
+    const int32_t *in_field3_l, const GD_DCOMPLEXP(m3), const char *m3_scalar,
+    const int32_t *m3_scalar_l, const int32_t *m3_scalar_ind,
+    const GD_DCOMPLEXP(b3), const char *b3_scalar, const int32_t *b3_scalar_l,
+    const int32_t *b3_scalar_ind);
+
 void F77_FUNC(gdlsph, GDLSPH) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const char *in_field,
     const int32_t *in_field_l, const int32_t *shift,
@@ -886,11 +929,96 @@ void F77_FUNC(gdlsrc, GDLSCR) (const int32_t *dirfile, const char *field_code,
     const char *dividend_scalar, const int32_t *dividend_scalar_l,
     const int32_t *dividend_scalar_ind);
 
+void F77_FUNC(gdlscr, GDLSCR) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field,
+    const int32_t *in_field_l, const double *dividend,
+    const char *dividend_scalar, const int32_t *dividend_scalar_l,
+    const int32_t *dividend_scalar_ind);
+
 void F77_FUNC(gdencs,GDENCS) (int32_t *ret, const int32_t *encoding);
 
+void F77_FUNC(gdsarx, GDSARX) (int32_t *max, const int32_t *dirfile);
+
+void F77_FUNC(gdmsax, GDMSAX) (int32_t *max, const int32_t *dirfile,
+    const char *parent, const int32_t *parent_l);
+
+void F77_FUNC(gdgesa, GDGESA) (int32_t *array_len, int32_t *fragment_index,
+    const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l);
+
+void F77_FUNC(gdgtsa, GDGTSA) (char *value, int32_t *value_l,
+    const int32_t *dirfile, const char *field_code, const int32_t *field_code_l,
+    const int32_t *index);
+
 void F77_FUNC(gdarln, GDARLN) (int32_t *len, const int32_t *dirfile,
     const char *field_code, const int32_t *field_code_l);
 
+void F77_FUNC(gdptsa, GDPTSA) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const int32_t *index, const char *value,
+    const int32_t *value_l);
+
+void F77_FUNC(gdadsa, GDADSA) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, int32_t *array_len,
+    const int32_t *fragment_index);
+
+void F77_FUNC(gdmdsa, GDMDSA) (const int32_t *dirfile, const char *parent,
+    const int32_t *parent_l, const char *field_code,
+    const int32_t *field_code_l, const int32_t *array_len);
+
+void F77_FUNC(gdalsa, GDALSA) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const int32_t *array_len);
+
+void F77_FUNC(gdgeid, GDGEID) (char *in_field1, int32_t *in_field1_l,
+    char *in_field2, int32_t *in_field2_l, int32_t *fragment_index,
+    const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l);
+
+void F77_FUNC(gdgesd, GDGESD) (char *in_field1, int32_t *in_field1_l,
+    char *in_field2, int32_t *in_field2_l, int32_t *fragment_index,
+    const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l);
+
+void F77_FUNC(gdadid, GDADID) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l, const int32_t *fragment_index);
+
+void F77_FUNC(gdadsd, GDADSD) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l, const int32_t *fragment_index);
+
+void F77_FUNC(gdmdid, GDMDID) (const int32_t *dirfile, const char *parent,
+    const int32_t *parent_l, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l);
+
+void F77_FUNC(gdmdsd, GDMDSD) (const int32_t *dirfile, const char *parent,
+    const int32_t *parent_l, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l);
+
+void F77_FUNC(gdalid, GDALID) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l);
+
+void F77_FUNC(gdalsd, GDALSD) (const int32_t *dirfile, const char *field_code,
+    const int32_t *field_code_l, const char *in_field1,
+    const int32_t *in_field1_l, const char *in_field2,
+    const int32_t *in_field2_l);
+
+void F77_FUNC(gdgstd, GDGSTD) (char *value, int32_t *value_l,
+    const int32_t *dirfile, const char *field_code, const int32_t *field_code_l,
+    const int32_t *first_frame, const int32_t *first_sample);
+
+void F77_FUNC(gdgstp, GDGSTP) (int32_t *n_read, char *data,
+    const int32_t *dirfile, const char *field_code, const int32_t *field_code_l,
+    const int32_t *first_frame, const int32_t *first_sample,
+    const int32_t *num_frames, const int32_t *num_samples);
+
 void F77_FUNC(gdaspn, GDASPN) (const int32_t *dirfile, const char *field_code,
     const int32_t *field_code_l, const int32_t *poly_ord, const char *in_field,
     const int32_t *in_field_l, const double *a0, const char *a0_scalar,
@@ -958,6 +1086,24 @@ void F77_FUNC(gdlssb, GDLSSB) (const int32_t *dirfile, const char *field_code,
     const int32_t *numbits, const char *numbits_scalar,
     const int32_t *numbits_scalar_l, const int32_t *numbits_scalar_ind);
 
+void F77_FUNC(gdxstp, GDXSTP) (char *value, int32_t *value_l, const char *data,
+    const int32_t *index);
+
+void F77_FUNC(gddstp, GDDSTP) (char *data);
+
+void F77_FUNC(gdmatx, GDMATX) (int32_t *max, const int32_t *dirfile,
+    const char *regex, const int32_t *regex_l, const int32_t *fragment,
+    const int32_t *type, const int32_t *flags);
+
+void F77_FUNC(gdmatn, GDMATN) (char *name, int32_t *name_l,
+    const int32_t *dirfile, const char *regex, const int32_t *regex_l,
+    const int32_t *fragment, const int32_t *type, const int32_t *flags,
+    const int32_t *field_num);
+
+void F77_FUNC(gdnmat, GDNMAT) (int32_t *nentries, const int32_t *dirfile,
+    const char *regex, const int32_t *regex_l, const int32_t *fragment,
+    const int32_t *type, const int32_t *flags);
+
 #ifdef __cplusplus
 }
 #endif
