@@ -183,7 +183,7 @@ C     This is good to single precision
       INTEGER flen
       PARAMETER (flen = 11)
       INTEGER nfields
-      PARAMETER (nfields = 17)
+      PARAMETER (nfields = 20)
       INTEGER slen
       PARAMETER (slen = 26)
       INTEGER plen
@@ -197,7 +197,7 @@ C     doesn't.
       PARAMETER (backslash = '\\')
 
       CHARACTER*26 strings(3)
-      CHARACTER*11 fields(nfields + 10)
+      CHARACTER*11 fields(nfields + 9)
       CHARACTER*11 fn
       CHARACTER*26 str
       CHARACTER*4096 path
@@ -224,23 +224,26 @@ C     doesn't.
       datdat(k) = k
    10 CONTINUE
  
-      fields( 1) = 'INDEX  '
-      fields( 2) = 'alias  '
-      fields( 3) = 'bit    '
-      fields( 4) = 'carray '
-      fields( 5) = 'const  '
-      fields( 6) = 'data   '
-      fields( 7) = 'div    '
-      fields( 8) = 'lincom '
-      fields( 9) = 'linterp'
+      fields( 1) = 'bit    '
+      fields( 2) = 'div    '
+      fields( 3) = 'data   '
+      fields( 4) = 'mult   '
+      fields( 5) = 'sbit   '
+      fields( 6) = 'INDEX  '
+      fields( 7) = 'alias  '
+      fields( 8) = 'const  '
+      fields( 9) = 'indir  '
       fields(10) = 'mplex  '
-      fields(11) = 'mult   '
-      fields(12) = 'phase  '
-      fields(13) = 'polynom'
-      fields(14) = 'recip  '
-      fields(15) = 'sbit   '
-      fields(16) = 'string '
-      fields(17) = 'window '
+      fields(11) = 'phase  '
+      fields(12) = 'recip  '
+      fields(13) = 'carray '
+      fields(14) = 'lincom '
+      fields(15) = 'sarray '
+      fields(16) = 'sindir '
+      fields(17) = 'string '
+      fields(18) = 'window '
+      fields(19) = 'linterp'
+      fields(20) = 'polynom'
 
 C     Write the test dirfile
       OPEN(1, FILE=frmat, STATUS='NEW')
@@ -266,6 +269,10 @@ C     Write the test dirfile
       WRITE(1, *) 'window WINDOW linterp mult LT 4.1'
       WRITE(1, *) '/ALIAS alias data'
       WRITE(1, *) 'string STRING "Zaphod Beeblebrox"'
+      WRITE(1, *) 'sarray SARRAY one two three four five six seven'
+      WRITE(1, *) 'data/msarray SARRAY eight nine ten eleven twelve'
+      WRITE(1, *) 'indir INDIR data carray'
+      WRITE(1, *) 'sindir SINDIR data sarray'
       CLOSE(1, STATUS='KEEP')
 
       OPEN(1, FILE=frm2, STATUS='NEW')
@@ -308,7 +315,7 @@ C     21: GDFDNX check
 C     22: GDMFNX check
       CALL GDMFNX(i, d, 'data', 4)
       CALL CHKEOK(ne, 22, d)
-      CALL CHKINT(ne, 22, i, 6)
+      CALL CHKINT(ne, 22, i, 7)
 
 C     23: GDNFLD check
       CALL GDNFLD(n, d)
@@ -319,7 +326,7 @@ C     24: This is a check of (one of many instances of) _GDF_FString
       l = 2
       CALL GDFLDN(fn, l, d, 1)
       CALL CHKEOK(ne, 24, d)
-      CALL CHKINT(ne, 24, l, 5)
+      CALL CHKINT(ne, 24, l, 3)
 
 C     25: GDFLDN check
       DO 250 i = 1, n
@@ -333,12 +340,13 @@ C     25: GDFLDN check
 C     26: GDNMFD check
       CALL GDNMFD(n, d, 'data', 4)
       CALL CHKEOK(ne, 26, d)
-      CALL CHKINT(ne, 26, n, 3)
+      CALL CHKINT(ne, 26, n, 4)
 
 C     27: GDMFDN check
       fields(1) = 'mstr'
       fields(2) = 'mconst'
       fields(3) = 'mlut'
+      fields(4) = 'msarray'
       DO 270 i = 1, n
       l = flen
       CALL GDMFDN(fn, l, d, 'data', 4, i)
@@ -755,9 +763,9 @@ C     67: GDNFDT check
       CALL CHKINT(ne, 67, n, 3)
 
 C     68: GDFDNT check
-      fields(1) = 'lincom'
-      fields(2) = 'new2'
-      fields(3) = 'new3'
+      fields(1) = 'new2'
+      fields(2) = 'new3'
+      fields(3) = 'lincom'
       DO 680 i = 1, n
       l = flen
       CALL GDFDNT(fn, l, d, GD_LCE, i)
@@ -769,33 +777,35 @@ C     68: GDFDNT check
 C     69: GDNVEC check
       CALL GDNVEC(n, d)
       CALL CHKEOK(ne, 69, d)
-      CALL CHKINT(ne, 69, n, 24)
+      CALL CHKINT(ne, 69, n, 25)
 
 C     70: GDVECN check
-      fields( 1) = 'INDEX  '
-      fields( 2) = 'alias  '
-      fields( 3) = 'bit    '
-      fields( 4) = 'data   '
-      fields( 5) = 'div    '
-      fields( 6) = 'lincom '
-      fields( 7) = 'linterp'
-      fields( 8) = 'mplex  '
-      fields( 9) = 'mult   '
-      fields(10) = 'new1   '
-      fields(11) = 'new10  '
-      fields(12) = 'new2   '
-      fields(13) = 'new3   '
-      fields(14) = 'new4   '
-      fields(15) = 'new5   '
-      fields(16) = 'new6   '
-      fields(17) = 'new7   '
-      fields(18) = 'new8   '
-      fields(19) = 'new9   '
+      fields( 1) = 'bit    '
+      fields( 2) = 'div    '
+      fields( 3) = 'data   '
+      fields( 4) = 'mult   '
+      fields( 5) = 'new1   '
+      fields( 6) = 'new2   '
+      fields( 7) = 'new3   '
+      fields( 8) = 'new4   '
+      fields( 9) = 'new5   '
+      fields(10) = 'new6   '
+      fields(11) = 'new7   '
+      fields(12) = 'new8   '
+      fields(13) = 'new9   '
+      fields(14) = 'sbit   '
+      fields(15) = 'INDEX  '
+      fields(16) = 'alias  '
+      fields(17) = 'indir  '
+      fields(18) = 'mplex  '
+      fields(19) = 'new10  '
       fields(20) = 'phase  '
-      fields(21) = 'polynom'
-      fields(22) = 'recip  '
-      fields(23) = 'sbit   '
-      fields(24) = 'window '
+      fields(21) = 'recip  '
+      fields(22) = 'lincom '
+      fields(23) = 'window '
+      fields(24) = 'linterp'
+      fields(25) = 'polynom'
+
       DO 700 i = 1, n
       l = flen
       CALL GDVECN(fn, l, d, i)
@@ -1029,9 +1039,8 @@ C     86: GDPTCO check
       CALL CHKDBL(ne, 86, 1d0 * fl, 10.0d0)
 
 C     94: GDPTST check
-      CALL GDPTST(n, d, 'string', 6, 11, "Arthur Dent")
+      CALL GDPTST(d, 'string', 6, 11, "Arthur Dent")
       CALL CHKOK2(ne, 94, 1, d)
-      CALL CHKINT(ne, 94, n, 11)
 
       CALL GDGTST(n, d, 'string', 6, slen, str)
       CALL CHKOK2(ne, 94, 2, d)
@@ -1418,7 +1427,7 @@ C     130: GDENFL check
 C     131: GDVLDT check
       CALL GDVLDT(n, d, 'new7', 4)
       CALL CHKERR(ne, 131, d, GD_EBC)
-      CALL CHKINT(ne, 131, n, -1)
+      CALL CHKINT(ne, 131, n, GD_EBC)
 
 C     132: GDFNUM check
       l = slen
@@ -1983,8 +1992,8 @@ C     221: GDNALS check
 C     222: GDALSS check
       fields(1) = 'data'
       fields(2) = 'alias'
-      fields(3) = 'data/mnew20'
-      fields(4) = 'new20'
+      fields(3) = 'new20'
+      fields(4) = 'data/mnew20'
       DO 2220 i = 1, n
       l = flen
       CALL GDALSS(fn, l, d, 'data', 4, i)
@@ -2117,10 +2126,10 @@ C     236: GDVBPX check
 C     237: GDNENT check
       CALL GDNENT(n, d, "data", 4, GDEN_S, GDEN_H + GDEN_N)
       CALL CHKOK2(ne, 237, 1, d)
-      CALL CHKIN2(ne, 237, 1, n, 5)
+      CALL CHKIN2(ne, 237, 1, n, 6)
       CALL GDNENT(n, d, "", -1, GDEN_V, GDEN_H + GDEN_N)
       CALL CHKOK2(ne, 237, 2, d)
-      CALL CHKIN2(ne, 237, 2, n, 27)
+      CALL CHKIN2(ne, 237, 2, n, 28)
 
 C     238: GDENTX check
       CALL GDENTX(l, d, "", -1, GDEN_V, GDEN_H + GDEN_N)
@@ -2128,33 +2137,34 @@ C     238: GDENTX check
       CALL CHKINT(ne, 238, l, 7)
 
 C     239: GDENTN check
-      fields( 1) = "INDEX"
-      fields( 2) = "bit"
+      fields( 1) = "bit"
+      fields( 2) = "div"
       fields( 3) = "data"
-      fields( 4) = "div"
-      fields( 5) = "lincom"
-      fields( 6) = "linterp"
-      fields( 7) = "mplex"
-      fields( 8) = "mult"
-      fields( 9) = "new1"
-      fields(10) = "new135"
-      fields(11) = "new14"
-      fields(12) = "new15"
-      fields(13) = "new16"
-      fields(14) = "new18"
-      fields(15) = "new2"
-      fields(16) = "new21"
-      fields(17) = "new3"
-      fields(18) = "new4"
-      fields(19) = "new5"
-      fields(20) = "new6"
-      fields(21) = "new7"
-      fields(22) = "new8"
-      fields(23) = "phase"
-      fields(24) = "polynom"
-      fields(25) = "recip"
-      fields(26) = "sbit"
-      fields(27) = "window"
+      fields( 4) = "mult"
+      fields( 5) = "new1"
+      fields( 6) = "new2"
+      fields( 7) = "new3"
+      fields( 8) = "new4"
+      fields( 9) = "new5"
+      fields(10) = "new6"
+      fields(11) = "new7"
+      fields(12) = "new8"
+      fields(13) = "sbit"
+      fields(14) = "INDEX"
+      fields(15) = "indir"
+      fields(16) = "mplex"
+      fields(17) = "new14"
+      fields(18) = "new15"
+      fields(19) = "new16"
+      fields(20) = "new18"
+      fields(21) = "new21"
+      fields(22) = "phase"
+      fields(23) = "recip"
+      fields(24) = "lincom"
+      fields(25) = "new135"
+      fields(26) = "window"
+      fields(27) = "linterp"
+      fields(28) = "polynom"
       DO 2390 i = 1, n
       l = flen
       CALL GDENTN(fn, l, d, "", -1, GDEN_V, GDEN_H + GDEN_N, i)
@@ -2980,6 +2990,251 @@ C     272: NULL return from gd_reference
 
       CALL GDDSCD(m)
 
+C     275: GDSARX
+      CALL GDSARX(n, d)
+      CALL CHKEOK(ne, 275, d)
+      CALL CHKINT(ne, 275, n, 5)
+
+C     276: GDMSAX
+      CALL GDMSAX(n, d, 'data', 4)
+      CALL CHKEOK(ne, 276, d)
+      CALL CHKINT(ne, 276, n, 6)
+
+C     277: GDGESA check
+      CALL GDGESA(l, n, d, 'sarray', 6)
+      CALL CHKEOK(ne, 277, d)
+      CALL CHKIN2(ne, 277, 1, l, 7)
+      CALL CHKIN2(ne, 277, 2, n, 0)
+
+C     279: GDGTSA check
+      fields(1) = 'one'
+      fields(2) = 'two'
+      fields(3) = 'three'
+      fields(4) = 'four'
+      fields(5) = 'five'
+      fields(6) = 'six'
+      fields(7) = 'seven'
+      DO 2790 i = 1, n
+      l = flen
+      CALL GDGTSA(fn, l, d, 'sarray', 6, i)
+      CALL CHKOK2(ne, 279, i, d)
+      CALL CHKIN2(ne, 279, i, l, flen)
+      CALL CHKST2(ne, 279, i, fn, fields(i))
+ 2790 CONTINUE
+
+C     282: GDPTSA check
+      fields(1) = 'abc'
+      fields(2) = 'def'
+      fields(3) = 'ghi'
+      fields(4) = 'jkl'
+      fields(5) = 'mno'
+      fields(6) = 'pqr'
+      fields(7) = 'stu'
+      DO 2820 i = 1, 7
+      l = flen
+      CALL GDPTSA(d, 'sarray', 6, i, fields(i), 3)
+      CALL CHKOK2(ne, 282, i, d)
+ 2820 CONTINUE
+
+      DO 2821 i = 1, n
+      l = flen
+      CALL GDGTSA(fn, l, d, 'sarray', 6, i)
+      CALL CHKOK2(ne, 282, i + n, d)
+      CALL CHKIN2(ne, 282, i, l, flen)
+      CALL CHKST2(ne, 282, i, fn, fields(i))
+ 2821 CONTINUE
+
+C     283: GDADSA check
+      CALL GDADSA(d, 'new283', 6, 4, 0)
+      CALL CHKOK2(ne, 283, 0, d)
+
+      CALL GDARLN(n, d, 'new283', 6)
+      CALL CHKOK2(ne, 283, 1, d)
+      CALL CHKIN2(ne, 283, 0, n, 4)
+
+      DO 2830 i = 1, 4
+      l = flen
+      CALL GDGTSA(fn, l, d, 'new283', 6, i)
+      CALL CHKOK2(ne, 283, i + 1, d)
+      CALL CHKIN2(ne, 283, i, l, flen)
+      CALL CHKST2(ne, 283, i, fn, '')
+ 2830 CONTINUE
+
+C     285: GDMDSA check
+      CALL GDMDSA(d, 'data', 4, 'mnew285', 7, 3, 0)
+      CALL CHKOK2(ne, 285, 0, d)
+
+      CALL GDARLN(n, d, 'data/mnew285', 12)
+      CALL CHKOK2(ne, 285, 1, d)
+      CALL CHKIN2(ne, 285, 0, n, 3)
+
+      DO 2850 i = 1, 3
+      l = flen
+      CALL GDGTSA(fn, l, d, 'data/mnew285', 12, i)
+      CALL CHKOK2(ne, 285, i + n, d)
+      CALL CHKIN2(ne, 285, i, l, flen)
+      CALL CHKST2(ne, 285, i, fn, '')
+ 2850 CONTINUE
+
+C     286: GDALSA check
+      CALL GDALSA(d, 'new283', 6, 3)
+      CALL CHKOK2(ne, 286, 0, d)
+
+      CALL GDARLN(n, d, 'new283', 6)
+      CALL CHKOK2(ne, 286, 1, d)
+      CALL CHKIN2(ne, 286, 0, n, 3)
+
+C     288: GDGEID check
+      l = flen
+      CALL GDGEID(fields(1), l, fields(2), l, n, d, 'indir', 5)
+      CALL CHKEOK(ne, 288, d)
+      CALL CHKIN2(ne, 288, 1, l, flen)
+      CALL CHKIN2(ne, 288, 2, n, 0)
+      CALL CHKST2(ne, 288, 3, fields(1), 'data')
+      CALL CHKST2(ne, 288, 4, fields(2), 'carray')
+
+C     289: GDADID check
+      CALL GDADID(d, 'new289', 6, 'in1', 3, 'in2', 3, 0)
+      CALL CHKOK2(ne, 289, 1, d)
+
+      l = flen
+      CALL GDGEID(fields(1), l, fields(2), l, n, d, 'new289', 6)
+      CALL CHKOK2(ne, 289, 2, d)
+      CALL CHKIN2(ne, 289, 1, l, flen)
+      CALL CHKIN2(ne, 289, 2, n, 0)
+      CALL CHKST2(ne, 289, 3, fields(1), 'in1')
+      CALL CHKST2(ne, 289, 4, fields(2), 'in2')
+
+C     290: GDMDID check
+      CALL GDMDID(d, 'data', 4, 'new290', 6, 'in3', 3, 'in4', 3)
+      CALL CHKOK2(ne, 290, 1, d)
+
+      l = flen
+      CALL GDGEID(fields(1), l, fields(2), l, n, d, 'data/new290', 11)
+      CALL CHKOK2(ne, 290, 2, d)
+      CALL CHKIN2(ne, 290, 1, l, flen)
+      CALL CHKIN2(ne, 290, 2, n, 0)
+      CALL CHKST2(ne, 290, 3, fields(1), 'in3')
+      CALL CHKST2(ne, 290, 4, fields(2), 'in4')
+
+C     291: GDALID check
+      CALL GDALID(d, 'new289', 6, 'in6', 3, 'in4', 3)
+      CALL CHKOK2(ne, 291, 1, d)
+
+      l = flen
+      CALL GDGEID(fields(1), l, fields(2), l, n, d, 'new289', 6)
+      CALL CHKOK2(ne, 291, 2, d)
+      CALL CHKIN2(ne, 291, 1, l, flen)
+      CALL CHKIN2(ne, 291, 2, n, 0)
+      CALL CHKST2(ne, 291, 3, fields(1), 'in6')
+      CALL CHKST2(ne, 291, 4, fields(2), 'in4')
+
+C     292: GDGESD check
+      l = flen
+      CALL GDGESD(fields(1), l, fields(2), l, n, d, 'sindir', 6)
+      CALL CHKEOK(ne, 292, d)
+      CALL CHKIN2(ne, 292, 1, l, flen)
+      CALL CHKIN2(ne, 292, 2, n, 0)
+      CALL CHKST2(ne, 292, 3, fields(1), 'data')
+      CALL CHKST2(ne, 292, 4, fields(2), 'sarray')
+
+C     293: GDADSD check
+      CALL GDADSD(d, 'new293', 6, 'in1', 3, 'in2', 3, 0)
+      CALL CHKOK2(ne, 293, 1, d)
+
+      l = flen
+      CALL GDGESD(fields(1), l, fields(2), l, n, d, 'new293', 6)
+      CALL CHKOK2(ne, 293, 2, d)
+      CALL CHKIN2(ne, 293, 1, l, flen)
+      CALL CHKIN2(ne, 293, 2, n, 0)
+      CALL CHKST2(ne, 293, 3, fields(1), 'in1')
+      CALL CHKST2(ne, 293, 4, fields(2), 'in2')
+
+C     294: GDMDSD check
+      CALL GDMDSD(d, 'data', 4, 'new294', 6, 'in3', 3, 'in4', 3)
+      CALL CHKOK2(ne, 294, 1, d)
+
+      l = flen
+      CALL GDGESD(fields(1), l, fields(2), l, n, d, 'data/new294', 11)
+      CALL CHKOK2(ne, 294, 2, d)
+      CALL CHKIN2(ne, 294, 1, l, flen)
+      CALL CHKIN2(ne, 294, 2, n, 0)
+      CALL CHKST2(ne, 294, 3, fields(1), 'in3')
+      CALL CHKST2(ne, 294, 4, fields(2), 'in4')
+
+C     295: GDALSD check
+      CALL GDALSD(d, 'new293', 6, 'in6', 3, 'in4', 3)
+      CALL CHKOK2(ne, 295, 1, d)
+
+      l = flen
+      CALL GDGESD(fields(1), l, fields(2), l, n, d, 'new293', 6)
+      CALL CHKOK2(ne, 295, 2, d)
+      CALL CHKIN2(ne, 295, 1, l, flen)
+      CALL CHKIN2(ne, 295, 2, n, 0)
+      CALL CHKST2(ne, 295, 3, fields(1), 'in6')
+      CALL CHKST2(ne, 295, 4, fields(2), 'in4')
+
+C     296: GDGSTD check
+      l = flen
+      CALL GDGSTD(fn, l, d, 'sindir', 6, 0, 0)
+      CALL CHKEOK(ne, 296, d)
+      CALL CHKINT(ne, 296, l, flen)
+      CALL CHKSTR(ne, 296, fn, 'abc')
+
+C     297: GDGSTP check
+      CALL GDGSTP(n, fn, d, 'sindir', 6, 0, 0, 1, 0)
+      CALL CHKEOK(ne, 297, d)
+      CALL CHKINT(ne, 297, n, 8)
+
+C     298: GDXSTP check
+      DO 2980 i = 1, 8
+      l = slen
+      CALL GDXSTP(str, l, fn, i)
+      CALL CHKIN2(ne, 298, i, l, slen)
+      CALL CHKST2(ne, 298, i, str, 'abc')
+ 2980 CONTINUE
+
+C     299: GDDSTP check
+      CALL GDDSTP(fn)
+
+C     302: GDINCA check
+      CALL GDINCN(d, 'format2', 7, 0, 'NS', 2, GD_CR + GD_EX)
+      CALL CHKEOK(ne, 302, d)
+
+C     303: GDFRNS check (read)
+      l = flen
+      CALL GDFRNS(fields(1), l, d, 2, "", -1)
+      CALL CHKEOK(ne, 303, d)
+      CALL CHKIN2(ne, 303, 1, l, flen)
+      CALL CHKST2(ne, 303, 2, fields(1), 'NS')
+
+C     304: GDFRNS check (update)
+      l = flen
+      CALL GDFRNS(fields(1), l, d, 2, "NS2", 3)
+      CALL CHKEOK(ne, 304, d)
+      CALL CHKIN2(ne, 304, 1, l, flen)
+      CALL CHKST2(ne, 304, 2, fields(1), 'NS2')
+
+C     305: GDMATN, GDMATX, GDNMAT checks
+      CALL GDMATX(m, d, "^lin", 4, 0, 0, 0)
+      CALL CHKOK2(ne, 305, 0, d)
+      CALL CHKINT(ne, 305, m, 7)
+
+      CALL GDNMAT(n, d, "^lin", 4, 0, 0, 0)
+      CALL CHKOK2(ne, 305, 1, d)
+      CALL CHKINT(ne, 305, n, 2)
+
+      CALL GDMATN(fn, l, d, "^lin", 4, 0, 0, 0, 1)
+      CALL CHKOK2(ne, 305, 3, d)
+      CALL CHKIN2(ne, 305, 4, l, flen)
+      CALL CHKST2(ne, 305, 5, fn, "lincom")
+
+      CALL GDMATN(fn, l, d, "^lin", 4, 0, 0, 0, 2)
+      CALL CHKOK2(ne, 305, 6, d)
+      CALL CHKIN2(ne, 305, 7, l, flen)
+      CALL CHKST2(ne, 305, 8, fn, "linterp")
+
+
 
 
 
