@@ -147,8 +147,11 @@ static struct {
   CONSTANT(RECIP_ENTRY,      "GD_RCE", GDMP_ENTYPE),
   CONSTANT(WINDOW_ENTRY,     "GD_WDE", GDMP_ENTYPE),
   CONSTANT(MPLEX_ENTRY,      "GD_MXE", GDMP_ENTYPE),
+  CONSTANT(INDIR_ENTRY,      "GD_IDE", GDMP_ENTYPE),
+  CONSTANT(SINDIR_ENTRY,     "GD_SDE", GDMP_ENTYPE),
   CONSTANT(CONST_ENTRY,      "GD_COE", GDMP_ENTYPE),
   CONSTANT(CARRAY_ENTRY,     "GD_CAE", GDMP_ENTYPE),
+  CONSTANT(SARRAY_ENTRY,     "GD_SAE", GDMP_ENTYPE),
   CONSTANT(STRING_ENTRY,     "GD_STE", GDMP_ENTYPE),
 
   CONSTANT(NULL,             "GD_NUL", GDMP_DTYPE),
@@ -164,6 +167,7 @@ static struct {
   CONSTANT(FLOAT64,          "GD_F64", GDMP_DTYPE),
   CONSTANT(COMPLEX64,        "GD_C64", GDMP_DTYPE),
   CONSTANT(COMPLEX128,       "GDC128", GDMP_DTYPE),
+  CONSTANT(STRING,           "GD_STR", GDMP_DTYPE),
 
   CONSTANT(DEL_META,         "GDD_MT", GDMP_RFLAG),
   CONSTANT(DEL_DATA,         "GDD_DT", GDMP_RFLAG),
@@ -237,6 +241,13 @@ static struct {
   CONSTANT(ENTRIES_HIDDEN,   "GDEN_H", GDMP_ENLIST),
   CONSTANT(ENTRIES_NOALIAS,  "GDEN_N", GDMP_ENLIST),
 
+  CONSTANT(REGEX_PCRE,       "GDRX_P", GDMP_ENLIST),
+  CONSTANT(REGEX_EXTENDED,   "GDRX_X", GDMP_ENLIST),
+  CONSTANT(REGEX_ICASE,      "GDRX_I", GDMP_ENLIST),
+  CONSTANT(REGEX_CASELESS,   "GDRX_C", GDMP_ENLIST),
+  CONSTANT(REGEX_JAVASCRIPT, "GDRX_J", GDMP_ENLIST),
+  CONSTANT(REGEX_UNICODE,    "GDRX_U", GDMP_ENLIST),
+
   CONSTANT(EN_CALC,          "GDE_CA", GDMP_EFLAG),
   CONSTANT(EN_HIDDEN,        "GDE_HI", GDMP_EFLAG),
   CONSTANT(EN_COMPSCAL,      "GDE_CS", GDMP_EFLAG),
@@ -392,6 +403,7 @@ void Fortran(void)
     printf("/\n");
   }
   printf("s/@GD_MAX_LINE_LENGTH@/%i/\n", GD_MAX_LINE_LENGTH);
+  printf("s/@SIZEOF_VOID_P@/%i/\n", SIZEOF_VOID_P);
 }
 
 void Python(void)
