@@ -1,4 +1,4 @@
-% Copyright (C) 2013 D. V. Wiebe
+% Copyright (C) 2013, 2016 D. V. Wiebe
 %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -49,13 +49,17 @@ try
   'window WINDOW linterp mult LT 4.1\n'...
   '/ALIAS alias data\n'...
   'string STRING "Zaphod Beeblebrox"\n'...
+  'sarray SARRAY one two three four five six seven\n'...
+  'data/msarray SARRAY eight nine ten eleven twelve\n'...
+  'indir INDIR data carray\n'...
+  'sindir SINDIR data sarray\n'...
   ];
   form2_data = 'const2 CONST INT8 -19\n';
 
-  fields = {'INDEX'; 'alias'; 'bit'; 'carray'; 'const'; 'data'; 'div'; ...
-  'lincom'; 'linterp'; 'mplex'; 'mult'; 'phase'; 'polynom'; 'recip'; ...
-  'sbit'; 'string'; 'window'};
-  nfields = 17;
+  fields = { 'bit'; 'div'; 'data'; 'mult'; 'sbit'; 'INDEX'; 'alias'; ...
+  'const'; 'indir'; 'mplex'; 'phase'; 'recip'; 'carray'; 'lincom'; 'sarray'; ...
+  'sindir'; 'string'; 'window'; 'linterp'; 'polynom'; };
+  nfields = 20;
 
   % Create the dirfile
   mkdir(filedir);
@@ -144,7 +148,7 @@ try
   % 26: gd_nmfields check
   try
     d = gd_nmfields(D, 'data');
-    ne = ne + check_num(26, d, 4);
+    ne = ne + check_num(26, d, 5);
   catch exc
     ne = ne + check_ok(exc, 26);
   end
@@ -153,7 +157,7 @@ try
   try
     d = gd_mfield_list(D, 'data');
     ne = ne + check_sarray(27, d, ...
-    { 'mstr'; 'mconst'; 'mcarray'; 'mlut'; });
+    { 'mstr'; 'mconst'; 'mcarray'; 'mlut'; 'msarray' });
   catch exc
     ne = ne + check_ok(exc, 27);
   end
@@ -565,7 +569,7 @@ try
   % 68: field_list_by_type
   try
     d = gd_field_list_by_type(D, GD.LINCOM_ENTRY);
-    ne = ne + check_sarray(68, d, { 'lincom', 'new2' });
+    ne = ne + check_sarray(68, d, { 'new2', 'lincom' });
   catch exc
     ne = ne + check_ok(exc, 68);
   end
@@ -573,7 +577,7 @@ try
   % 69: nfields_by_type
   try
     d = gd_nvectors(D);
-    ne = ne + check_num(69, d, 22);
+    ne = ne + check_num(69, d, 23);
   catch exc
     ne = ne + check_ok(exc, 69);
   end
@@ -581,10 +585,10 @@ try
   % 70: vector_list check
   try
     d = gd_vector_list(D);
-    ne = ne + check_sarray(70, d, ...
-    {'INDEX'; 'alias'; 'bit'; 'data'; 'div'; 'lincom'; 'linterp'; ...
-    'mplex'; 'mult'; 'new1'; 'new10'; 'new2'; 'new5'; 'new6'; 'new7'; ...
-    'new8'; 'new9'; 'phase'; 'polynom'; 'recip'; 'sbit'; 'window'});
+    ne = ne + check_sarray(70, d, { 'bit'; 'div'; 'data'; 'mult'; 'new1'; ...
+    'new2'; 'new5'; 'new6'; 'new7'; 'new8'; 'new9'; 'sbit'; 'INDEX'; ...
+    'alias'; 'indir'; 'mplex'; 'new10'; 'phase'; 'recip'; 'lincom'; ...
+    'window'; 'linterp'; 'polynom' });
   catch exc
     ne = ne + check_ok(exc, 70);
   end
@@ -1807,16 +1811,16 @@ try
   % 222: aliases
   try
     d = gd_aliases(D, 'data');
-    ne = ne + check_sarray(222, d, {'data'; 'alias'; 'data/mnew20'; 'new20'});
+    ne = ne + check_sarray(222, d, { 'data'; 'alias'; 'new20'; 'data/mnew20' });
   catch exc
     ne = ne + check_ok(exc, 222);
   end
 
   % 223: include_affix
   try
-    gd_include(D, 'format1', 0, GD.CREAT + GD.EXCL, 'A', 'Z');
+    gd_include_affix(D, 'format1', 0, 'A', 'Z', GD.CREAT + GD.EXCL);
   catch exc
-    ne = ne + check_ok2(exc, 41, 1);
+    ne = ne + check_ok2(exc, 223, 1);
   end
 
   % 226: fragment_affixes
@@ -1953,16 +1957,16 @@ try
   try
     d = gd_nentries(D, 'data', GD.SCALAR_ENTRIES, ...
     GD.ENTRIES_HIDDEN + GD.ENTRIES_NOALIAS);
-    ne = ne + check_num(237, d, 6);
+    ne = ne + check_num(237, d, 7);
   catch exc
     ne = ne + check_ok(exc, 237);
   end
 
-  % 239: gd_nentries
+  % 239: gd_entry_list
   try
     d = gd_entry_list(D, 'data', GD.SCALAR_ENTRIES, ...
     GD.ENTRIES_HIDDEN + GD.ENTRIES_NOALIAS);
-    ne = ne + check_sarray(239, d, {'mstr', 'mconst', 'mcarray', ...
+    ne = ne + check_sarray(239, d, {'mstr', 'mconst', 'mcarray', 'msarray', ...
     'mnew11', 'mnew12', 'mnew17'});
   catch exc
     ne = ne + check_ok(exc, 239);
@@ -2027,6 +2031,314 @@ try
     ne = ne + check_ok(exc, 274);
   end
 
+  % 277: gd_entry (SARRAY)
+  try
+    d = gd_entry(D, 'sarray');
+    ne = ne + check_string2(277, 1, d.field, 'sarray');
+    ne = ne + check_num2(277, 2, d.field_type, GD.SARRAY_ENTRY);
+    ne = ne + check_num2(277, 3, d.fragment_index, 0);
+    ne = ne + check_num2(277, 4, d.array_len, 7);
+  catch exc
+    ne = ne + check_ok(exc, 277);
+  end
+
+  % 278: get_sarray
+  try
+    d = gd_get_sarray(D, 'sarray');
+    ne = ne + check_sarray(278, d, {'one'; 'two'; 'three'; 'four'; 'five'; ...
+    'six'; 'seven'});
+  catch exc
+    ne = ne + check_ok(exc, 278);
+  end
+
+  % 279: gd_sarray_slice
+  try
+    d = gd_get_sarray_slice(D, 'sarray', 2, 2);
+    ne = ne + check_sarray(279, d, {'three'; 'four'});
+  catch exc
+    ne = ne + check_ok(exc, 279);
+  end
+
+  % 280: gd_sarrays
+  try
+    d = gd_sarrays(D);
+    ne = ne + check_num(280, length(d), 1);
+    ne = ne + check_sarray(280, d{1}, {'one'; 'two'; 'three'; 'four'; ...
+    'five'; 'six'; 'seven'});
+  catch exc
+    ne = ne + check_ok(exc, 280);
+  end
+
+  % 281: gd_put_sarray
+  try
+    gd_put_sarray(D, 'sarray', ...
+    {'eka'; 'dvi'; 'tri'; 'catur'; 'panca'; 'sas'; 'sapta'});
+  catch exc
+    ne = ne + check_ok2(exc, 281, 0);
+  end
+
+  try
+    d = gd_get_sarray(D, 'sarray');
+    ne = ne + check_sarray(281, d, ...
+    {'eka'; 'dvi'; 'tri'; 'catur'; 'panca'; 'sas'; 'sapta'});
+  catch exc
+    ne = ne + check_ok2(exc, 281, 1);
+  end
+
+  % 282: gd_put_sarray
+  try
+    gd_put_sarray_slice(D, 'sarray', 2, {'asta'; 'nava'});
+  catch exc
+    ne = ne + check_ok2(exc, 282, 0);
+  end
+
+  try
+    d = gd_get_sarray(D, 'sarray');
+    ne = ne + check_sarray(282, d, ...
+    {'eka'; 'dvi'; 'asta'; 'nava'; 'panca'; 'sas'; 'sapta'});
+  catch exc
+    ne = ne + check_ok2(exc, 282, 1);
+  end
+
+  % 283: add_sarray
+  try
+    gd_add_sarray(D, 'new283', {'eins'; 'zwei'; 'drei'}, 0);
+  catch exc
+    ne = ne + check_ok2(exc, 283, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new283');
+    ne = ne + check_string2(283, 1, d.field, 'new283');
+    ne = ne + check_num2(283, 2, d.field_type, GD.SARRAY_ENTRY);
+    ne = ne + check_num2(283, 3, d.fragment_index, 0);
+    ne = ne + check_num2(283, 4, d.array_len, 3);
+  catch exc
+    ne = ne + check_ok2(exc, 283, 2);
+  end
+
+  try
+    d = gd_get_sarray(D, 'new283');
+    ne = ne + check_sarray(283, d, {'eins'; 'zwei'; 'drei'});
+  catch exc
+    ne = ne + check_ok2(exc, 283, 3);
+  end
+
+  % 285: madd_sarray
+  try
+    gd_madd_sarray(D, 'data', 'mnew285', {'un'; 'deux'; 'trois'});
+  catch exc
+    ne = ne + check_ok2(exc, 285, 1);
+  end
+
+  try
+    d = gd_entry(D, 'data/mnew285');
+    ne = ne + check_string2(285, 1, d.field, 'data/mnew285');
+    ne = ne + check_num2(285, 2, d.field_type, GD.SARRAY_ENTRY);
+    ne = ne + check_num2(285, 3, d.fragment_index, 0);
+    ne = ne + check_num2(285, 4, d.array_len, 3);
+  catch exc
+    ne = ne + check_ok2(exc, 285, 2);
+  end
+
+  try
+    d = gd_get_sarray(D, 'data/mnew285');
+    ne = ne + check_sarray(285, d, {'un'; 'deux'; 'trois'});
+  catch exc
+    ne = ne + check_ok2(exc, 285, 3);
+  end
+
+  % 286: alter_sarray
+  try
+    gd_alter_sarray(D, 'new283', 2);
+  catch exc
+    ne = ne + check_ok2(exc, 286, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new283');
+    ne = ne + check_string2(286, 1, d.field, 'new283');
+    ne = ne + check_num2(286, 2, d.field_type, GD.SARRAY_ENTRY);
+    ne = ne + check_num2(286, 3, d.fragment_index, 0);
+    ne = ne + check_num2(286, 4, d.array_len, 2);
+  catch exc
+    ne = ne + check_ok2(exc, 286, 2);
+  end
+
+  % 287: gd_msarrays
+  try
+    d = gd_msarrays(D, 'data');
+    ne = ne + check_num(287, length(d), 2);
+    ne = ne + check_sarray2(287, 1, d{1}, ...
+    {'eight'; 'nine'; 'ten'; 'eleven'; 'twelve'});
+    ne = ne + check_sarray2(287, 2, d{2}, {'un'; 'deux'; 'trois'});
+  catch exc
+    ne = ne + check_ok(exc, 287);
+  end
+
+  % 288: entry (INDIR) check
+  try
+    d = gd_entry(D, 'indir');
+    ne = ne + check_string2(288, 1, d.field, 'indir');
+    ne = ne + check_num2(288, 2, d.field_type, GD.INDIR_ENTRY);
+    ne = ne + check_num2(288, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(288, 4, d.in_fields, { 'data'; 'carray'; });
+  catch exc
+    ne = ne + check_ok(exc, 288);
+  end
+
+  % 289: add entry (INDIR) check
+  try
+    gd_add_indir(D, 'new289', 'in1', 'in2', 0);
+  catch exc
+    ne = ne + check_ok2(exc, 289, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new289');
+    ne = ne + check_string2(289, 1, d.field, 'new289');
+    ne = ne + check_num2(289, 2, d.field_type, GD.INDIR_ENTRY);
+    ne = ne + check_num2(289, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(289, 4, d.in_fields, { 'in1'; 'in2'; });
+  catch exc
+    ne = ne + check_ok2(exc, 289, 2);
+  end
+
+  % 290: madd entry (INDIR) check
+  try
+    gd_madd_indir(D, 'data', 'mnew290', 'in1', 'in2');
+  catch exc
+    ne = ne + check_ok2(exc, 290, 1);
+  end
+
+  try
+    d = gd_entry(D, 'data/mnew290');
+    ne = ne + check_string2(290, 1, d.field, 'data/mnew290');
+    ne = ne + check_num2(290, 2, d.field_type, GD.INDIR_ENTRY);
+    ne = ne + check_num2(290, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(290, 4, d.in_fields, { 'in1'; 'in2'; });
+  catch exc
+    ne = ne + check_ok2(exc, 290, 2);
+  end
+
+  % 291: gd_alter_indir
+  try
+    gd_alter_indir(D, 'new289', 0, 'in6');
+  catch exc
+    ne = ne + check_ok2(exc, 291, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new289');
+    ne = ne + check_string2(291, 1, d.field, 'new289');
+    ne = ne + check_num2(291, 2, d.field_type, GD.INDIR_ENTRY);
+    ne = ne + check_num2(291, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(291, 4, d.in_fields, { 'in1'; 'in6'; });
+  catch exc
+    ne = ne + check_ok2(exc, 291, 2);
+  end
+
+  % 292: entry (SINDIR) check
+  try
+    d = gd_entry(D, 'sindir');
+    ne = ne + check_string2(292, 1, d.field, 'sindir');
+    ne = ne + check_num2(292, 2, d.field_type, GD.SINDIR_ENTRY);
+    ne = ne + check_num2(292, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(292, 4, d.in_fields, { 'data'; 'sarray'; });
+  catch exc
+    ne = ne + check_ok(exc, 292);
+  end
+
+  % 293: add entry (SINDIR) check
+  try
+    gd_add_sindir(D, 'new293', 'in1', 'in2', 0);
+  catch exc
+    ne = ne + check_ok2(exc, 293, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new293');
+    ne = ne + check_string2(293, 1, d.field, 'new293');
+    ne = ne + check_num2(293, 2, d.field_type, GD.SINDIR_ENTRY);
+    ne = ne + check_num2(293, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(293, 4, d.in_fields, { 'in1'; 'in2'; });
+  catch exc
+    ne = ne + check_ok2(exc, 293, 2);
+  end
+
+  % 294: madd entry (SINDIR) check
+  try
+    gd_madd_sindir(D, 'data', 'mnew294', 'in1', 'in2');
+  catch exc
+    ne = ne + check_ok2(exc, 294, 1);
+  end
+
+  try
+    d = gd_entry(D, 'data/mnew294');
+    ne = ne + check_string2(294, 1, d.field, 'data/mnew294');
+    ne = ne + check_num2(294, 2, d.field_type, GD.SINDIR_ENTRY);
+    ne = ne + check_num2(294, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(294, 4, d.in_fields, { 'in1'; 'in2'; });
+  catch exc
+    ne = ne + check_ok2(exc, 294, 2);
+  end
+
+  % 295: gd_alter_sindir
+  try
+    gd_alter_sindir(D, 'new293', 0, 'in6');
+  catch exc
+    ne = ne + check_ok2(exc, 295, 1);
+  end
+
+  try
+    d = gd_entry(D, 'new293');
+    ne = ne + check_string2(295, 1, d.field, 'new293');
+    ne = ne + check_num2(295, 2, d.field_type, GD.SINDIR_ENTRY);
+    ne = ne + check_num2(295, 3, d.fragment_index, 0);
+    ne = ne + check_sarray2(295, 4, d.in_fields, { 'in1'; 'in6'; });
+  catch exc
+    ne = ne + check_ok2(exc, 295, 2);
+  end
+
+  % 296: gd_getdata (SINDIR)
+  try
+    d = gd_getdata(D, 'sindir', 0, 0, 1, 0);
+    ne = ne + check_sarray(296, d, ...
+    {'eka'; 'eka'; 'eka'; 'eka'; 'eka'; 'eka'; 'eka'; 'eka';});
+  catch exc
+    ne = ne + check_ok(exc, 296);
+  end
+
+  % 302: gd_include_ns
+  try
+    gd_include(D, 'format2', 0, 'ns', GD.CREAT + GD.EXCL);
+  catch exc
+    ne = ne + check_ok2(exc, 302, 1);
+  end
+
+  % 303: gd_fragment_namespace (read)
+  try
+    d = gd_fragment_namespace(D, 2);
+    ne = ne + check_string(303, d, 'ns');
+  catch exc
+    ne = ne + check_ok(exc, 303);
+  end
+
+  % 304: gd_fragment_namespace (alter)
+  try
+    d = gd_fragment_namespace(D, 2, 'ns2');
+    ne = ne + check_string(304, d, 'ns2');
+  catch exc
+    ne = ne + check_ok(exc, 304);
+  end
+
+  % 305: gd_match_entries
+  try
+    d = gd_match_entries(D, '^lin', 0);
+    ne = ne + check_sarray(305, d, {'lincom'; 'linterp'});
+  catch exc
+    ne = ne + check_ok(exc, 305);
+  end
 
 
 
