@@ -96,7 +96,6 @@
     } \
   } while(0)
 
-extern PyObject *gdpy_exceptions[GD_N_ERROR_CODES];
 extern PyTypeObject gdpy_dirfile;
 extern PyTypeObject gdpy_entry;
 extern PyTypeObject gdpy_fragment;
@@ -198,6 +197,7 @@ extern DIRFILE *gdpy_dirfile_dirfile(struct gdpy_dirfile_t *);
 extern int gdpy_dirfile_raise(struct gdpy_dirfile_t *);
 #endif
 
+extern char *gdpy_strdup(const char*);
 extern int gdpy_report_error(DIRFILE*, char*);
 extern char *gdpy_copy_global_charenc(void);
 extern PyObject *gdpyobj_from_string(const char*, const char*);
