@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2016 D. V. Wiebe
+/* Copyright (C) 2009-2017 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -36,14 +36,16 @@ static const char *gdpy_entry_type_names[] =
   "RECIP_ENTRY",    /* 0x0B */
   "WINDOW_ENTRY",   /* 0x0C */
   "MPLEX_ENTRY",    /* 0x0D */
-  NULL,             /* 0x0E */
-  NULL,             /* 0x0F */
+  "INDIR_ENTRY",    /* 0x0E */
+  "SINDIR_ENTRY",   /* 0x0F */
   "CONST_ENTRY",    /* 0x10 */
   "STRING_ENTRY",   /* 0x11 */
   "CARRAY_ENTRY",   /* 0x12 */
+  "SARRAY_ENTRY"    /* 0x13 */
 };
-#define GDPY_N_ENTYPES 0x12
+#define GDPY_LAST_ENTYPE 0x13
 
+/* Routine unicode failure check routines */
 #define S1CHECK do { \
   if (s1 == NULL) { dreturn("%p", NULL); return NULL; } \
 } while(0)
@@ -67,8 +69,8 @@ static void gdpy_entry_delete(struct gdpy_entry_t *self)
   dtrace("%p", self);
 
   gd_free_entry_strings(self->E);
-  free(self->E);
-  free(self->char_enc);
+  PyMem_Free(self->E);
+  PyMem_Free(self->char_enc);
   PyObject_Del(self);
 
   dreturnvoid();
@@ -101,7 +103,7 @@ static PyObject *gdpyobj_from_scalar(const gd_entry_t *E, int i, gd_type_t type,
   if (E->scalar[i]) {
     /* Return the scalar field code */
     if (E->scalar_ind[i] >= 0) {
-      char *buffer = malloc(strlen(E->scalar[i]) + 23);
+      char *buffer = PyMem_Malloc(strlen(E->scalar[i]) + 23);
       if (buffer == NULL) {
         PyErr_NoMemory();
         dreturn("%p", NULL);
@@ -109,7 +111,7 @@ static PyObject *gdpyobj_from_scalar(const gd_entry_t *E, int i, gd_type_t type,
       }
       sprintf(buffer, "%s<%i>", E->scalar[i], E->scalar_ind[i]);
       pyobj = gdpyobj_from_string(buffer, char_enc);
-      free(buffer);
+      PyMem_Free(buffer);
     } else
       pyobj = gdpyobj_from_string(E->scalar[i], char_enc);
   } else /* If scalar is NULL, return the number */
@@ -161,6 +163,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       min = 0;
       break;
     case GD_CONST_ENTRY:
+    case GD_SARRAY_ENTRY:
       min = 1;
       break;
     case GD_RAW_ENTRY:
@@ -169,6 +172,8 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
     case GD_SBIT_ENTRY:
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_RECIP_ENTRY:
     case GD_PHASE_ENTRY:
     case GD_POLYNOM_ENTRY:
@@ -241,7 +246,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
         E->in_fields[i] = gdpy_string_from_pyobj(PyTuple_GetItem(parm1, i),
             char_enc, "in_fields must be string");
 
-        if (PyErr_Occurred()) {
+        if (E->in_fields[i] == NULL) {
           dreturnvoid();
           return;
         }
@@ -289,7 +294,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -307,7 +312,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -324,10 +329,12 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -335,7 +342,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[1] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 1),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[1] == NULL) {
         dreturnvoid();
         return;
       }
@@ -344,7 +351,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -371,7 +378,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -395,7 +402,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -425,7 +432,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -433,7 +440,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[1] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 1),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[1] == NULL) {
         dreturnvoid();
         return;
       }
@@ -471,7 +478,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[0] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 0),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[0] == NULL) {
         dreturnvoid();
         return;
       }
@@ -479,7 +486,7 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       E->in_fields[1] = gdpy_string_from_pyobj(PyTuple_GetItem(tuple, 1),
           char_enc, "in_fields must be string");
 
-      if (PyErr_Occurred()) {
+      if (E->in_fields[1] == NULL) {
         dreturnvoid();
         return;
       }
@@ -490,6 +497,10 @@ static void gdpy_set_entry_from_tuple(gd_entry_t *E, PyObject *tuple,
       gdpy_set_scalar_from_pyobj(PyTuple_GetItem(tuple, 3), GD_INT_TYPE,
           &E->scalar[1], char_enc, &E->EN(mplex,period), NULL);
       break;
+    case GD_SARRAY_ENTRY:
+      E->EN(scalar,array_len) =
+        (size_t)gdpy_ulong_from_pyobj(PyTuple_GetItem(tuple, 0));
+      break;
     case GD_CARRAY_ENTRY:
       E->EN(scalar,array_len) =
         (size_t)gdpy_ulong_from_pyobj(PyTuple_GetItem(tuple, 1));
@@ -529,12 +540,15 @@ static void gdpy_set_entry_from_dict(gd_entry_t *E, PyObject *parms,
    * PHASE:    in_field, shift             = 2
    * MULTIPLY: in_field1, in_field2        = 2
    * DIVIDE:   in_field1, in_field2        = 2
+   * INDIR:    in_field1, in_field2        = 2
+   * SINDIR:   in_field1, in_field2        = 2
    * RECIP:    in_field, dividend          = 2
    * POLYNOM:  in_field, a                 = 2
    * WINDOW:   in_field1, in_field2, op, thresh = 4
    * MPLEX:    in_field1, in_field2, val, max = 4
    * CONST:    type                        = 1
    * CARRAY:   type, array_len             = 2
+   * SARRAY:   array_len                   = 1
    * STRING:   (none)                      = 0
    * INDEX:    (none)                      = 0
    */
@@ -569,6 +583,8 @@ static void gdpy_set_entry_from_dict(gd_entry_t *E, PyObject *parms,
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
       key[0] = "in_field1";
       key[1] = "in_field2";
       size = 2;
@@ -607,6 +623,10 @@ static void gdpy_set_entry_from_dict(gd_entry_t *E, PyObject *parms,
       key[1] = "array_len";
       size = 2;
       break;
+    case GD_SARRAY_ENTRY:
+      key[0] = "array_len";
+      size = 1;
+      break;
     case GD_CONST_ENTRY:
       key[0] = "type";
       size = 1;
@@ -675,7 +695,7 @@ static int gdpy_entry_init(struct gdpy_entry_t *self, PyObject *args,
   E.field = gdpy_string_from_pyobj(field_name, self->char_enc,
       "field name should be string");
 
-  if (PyErr_Occurred()) {
+  if (E.field == NULL) {
     dreturn("%i", -1);
     return -1;
   }
@@ -683,7 +703,7 @@ static int gdpy_entry_init(struct gdpy_entry_t *self, PyObject *args,
   E.field_type = field_type;
 
   /* check for valid field type */
-  if (E.field_type > GDPY_N_ENTYPES || E.field_type <= 0 ||
+  if (E.field_type > GDPY_LAST_ENTYPE || E.field_type <= 0 ||
       gdpy_entry_type_names[E.field_type] == NULL)
   {
     PyErr_SetString(PyExc_ValueError,
@@ -714,9 +734,10 @@ static int gdpy_entry_init(struct gdpy_entry_t *self, PyObject *args,
   }
 
   if (self->E == NULL) {
-    self->E = malloc(sizeof(gd_entry_t));
+    self->E = PyMem_Malloc(sizeof(gd_entry_t));
 
     if (self->E == NULL) {
+      PyErr_NoMemory();
       dreturn("%i", -1);
       return -1;
     }
@@ -757,12 +778,12 @@ static int gdpy_entry_setname(struct gdpy_entry_t *self, PyObject *value,
   s = gdpy_string_from_pyobj(value, self->char_enc,
       "field name should be string");
 
-  if (PyErr_Occurred()) {
+  if (s == NULL) {
     dreturn("%i", -1);
     return -1;
   }
 
-  free(self->E->field);
+  PyMem_Free(self->E->field);
   self->E->field = s;
 
   dreturn("%i", 0);
@@ -863,10 +884,11 @@ static PyObject *gdpy_entry_getinfields(struct gdpy_entry_t *self,
       s1 = gdpyobj_from_string(self->E->in_fields[0], self->char_enc);
       S1CHECK;
       tuple = Py_BuildValue("(N)", s1);
-      S1CHECK;
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_WINDOW_ENTRY:
     case GD_MPLEX_ENTRY:
       s1 = gdpyobj_from_string(self->E->in_fields[0], self->char_enc);
@@ -881,6 +903,7 @@ static PyObject *gdpy_entry_getinfields(struct gdpy_entry_t *self,
     case GD_INDEX_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_STRING_ENTRY:
       PyErr_Format(PyExc_AttributeError, "'pygetdata.entry' "
           "attribute 'in_fields' not available for entry type %s",
@@ -925,17 +948,18 @@ static int gdpy_entry_setinfields(struct gdpy_entry_t *self, PyObject *value,
         return -1;
       }
 
-      for (i = 0; i < self->E->EN(lincom,n_fields); ++i)
+      for (i = 0; i < self->E->EN(lincom,n_fields); ++i) {
         s[i] = gdpy_string_from_pyobj(PyTuple_GetItem(value, i), self->char_enc,
             "in_fields should be strings");
 
-      if (PyErr_Occurred()) {
-        dreturn("%i", -1);
-        return -1;
+        if (s[i] == NULL) {
+          dreturn("%i", -1);
+          return -1;
+        }
       }
 
       for (i = 0; i < self->E->EN(lincom,n_fields); ++i) {
-        free(self->E->in_fields[i]);
+        PyMem_Free(self->E->in_fields[i]);
         self->E->in_fields[i] = s[i];
       }
       break;
@@ -966,16 +990,18 @@ static int gdpy_entry_setinfields(struct gdpy_entry_t *self, PyObject *value,
             "in_fields should be strings");
       }
 
-      if (PyErr_Occurred()) {
+      if (s[0] == NULL) {
         dreturn("%i", -1);
         return -1;
       }
 
-      free(self->E->in_fields[0]);
+      PyMem_Free(self->E->in_fields[0]);
       self->E->in_fields[0] = s[0];
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_WINDOW_ENTRY:
     case GD_MPLEX_ENTRY:
       if (value == NULL) {
@@ -999,17 +1025,18 @@ static int gdpy_entry_setinfields(struct gdpy_entry_t *self, PyObject *value,
         return -1;
       }
 
-      for (i = 0; i < 2; ++i)
+      for (i = 0; i < 2; ++i) {
         s[i] = gdpy_string_from_pyobj(PyTuple_GetItem(value, i), self->char_enc,
             "in_fields should be strings");
 
-      if (PyErr_Occurred()) {
-        dreturn("%i", -1);
-        return -1;
+        if (s[i] == NULL) {
+          dreturn("%i", -1);
+          return -1;
+        }
       }
 
       for (i = 0; i < 2; ++i) {
-        free(self->E->in_fields[i]);
+        PyMem_Free(self->E->in_fields[i]);
         self->E->in_fields[i] = s[i];
       }
       break;
@@ -1019,6 +1046,7 @@ static int gdpy_entry_setinfields(struct gdpy_entry_t *self, PyObject *value,
     case GD_INDEX_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_STRING_ENTRY:
       PyErr_Format(PyExc_AttributeError, "'pygetdata.entry' "
           "attribute 'in_fields' not available for entry type %s",
@@ -1166,12 +1194,12 @@ static int gdpy_entry_setspf(struct gdpy_entry_t *self, PyObject *value,
       &spf, "spf");
 
   if (PyErr_Occurred()) {
-    free(scalar);
+    PyMem_Free(scalar);
     dreturn("%i", -1);
     return -1;
   }
 
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
   self->E->EN(raw,spf) = spf;
 
@@ -1186,7 +1214,9 @@ static PyObject *gdpy_entry_getarraylen(struct gdpy_entry_t *self,
 
   dtrace("%p, %p", self, closure);
 
-  if (self->E->field_type == GD_CARRAY_ENTRY) {
+  if (self->E->field_type == GD_CARRAY_ENTRY ||
+      self->E->field_type == GD_SARRAY_ENTRY)
+  {
       obj = PyLong_FromUnsignedLong(self->E->EN(scalar,array_len));
   } else
     PyErr_Format(PyExc_AttributeError, "'pygetdata.entry' "
@@ -1282,11 +1312,11 @@ static int gdpy_entry_setnfields(struct gdpy_entry_t *self, PyObject *value,
 
   /* free extra terms */
   for (i = n; i < self->E->EN(lincom,n_fields); ++i)
-    free(self->E->in_fields[i]);
+    PyMem_Free(self->E->in_fields[i]);
 
   /* initialise new terms */
   for (i = self->E->EN(lincom,n_fields); i < n; ++i) {
-    self->E->in_fields[i] = strdup("");
+    self->E->in_fields[i] = gdpy_strdup("");
     self->E->EN(lincom,m)[i] = self->E->EN(lincom,b)[i] = 0;
   }
 
@@ -1383,7 +1413,7 @@ static int gdpy_entry_setm(struct gdpy_entry_t *self, PyObject *value,
 
   if (PyErr_Occurred()) {
     for (i = 0; i < GD_MAX_LINCOM; ++i)
-      free(scalar[i]);
+      PyMem_Free(scalar[i]);
     dreturn("%i", -1);
     return -1;
   }
@@ -1394,7 +1424,7 @@ static int gdpy_entry_setm(struct gdpy_entry_t *self, PyObject *value,
       comp_scal = GD_EN_COMPSCAL;
     gd_cs2cs_(self->E->EN(lincom,cm)[i], cm[i]);
     self->E->EN(lincom,m)[i] = m[i];
-    free(self->E->scalar[i]);
+    PyMem_Free(self->E->scalar[i]);
     self->E->scalar[i] = scalar[i];
   }
   self->E->flags |= comp_scal;
@@ -1490,7 +1520,7 @@ static int gdpy_entry_setb(struct gdpy_entry_t *self, PyObject *value,
 
   if (PyErr_Occurred()) {
     for (i = 0; i < GD_MAX_LINCOM; ++i)
-      free(scalar[i]);
+      PyMem_Free(scalar[i]);
     dreturn("%i", -1);
     return -1;
   }
@@ -1501,7 +1531,7 @@ static int gdpy_entry_setb(struct gdpy_entry_t *self, PyObject *value,
       comp_scal = GD_EN_COMPSCAL;
     gd_cs2cs_(self->E->EN(lincom,cb)[i], cb[i]);
     self->E->EN(lincom,b)[i] = b[i];
-    free(self->E->scalar[i + GD_MAX_LINCOM]);
+    PyMem_Free(self->E->scalar[i + GD_MAX_LINCOM]);
     self->E->scalar[i + GD_MAX_LINCOM] = scalar[i];
   }
   self->E->flags |= comp_scal;
@@ -1555,7 +1585,7 @@ static int gdpy_entry_settable(struct gdpy_entry_t *self, PyObject *value,
     return -1;
   }
 
-  free(self->E->EN(linterp,table));
+  PyMem_Free(self->E->EN(linterp,table));
   self->E->EN(linterp,table) = s;
 
   dreturn("%i", 0);
@@ -1602,13 +1632,13 @@ static int gdpy_entry_setbitnum(struct gdpy_entry_t *self, PyObject *value,
   gdpy_set_scalar_from_pyobj(value, GD_INT_TYPE, &scalar, self->char_enc,
       &bitnum, "bitnum");
   if (PyErr_Occurred()) {
-    free(scalar);
+    PyMem_Free(scalar);
     dreturn("%i", -1);
     return -1;
   }
 
   self->E->EN(bit,bitnum) = bitnum;
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
 
   dreturn("%i", 0);
@@ -1662,7 +1692,7 @@ static int gdpy_entry_setnumbits(struct gdpy_entry_t *self, PyObject *value,
   }
 
   self->E->EN(bit,numbits) = numbits;
-  free(self->E->scalar[1]);
+  PyMem_Free(self->E->scalar[1]);
   self->E->scalar[1] = scalar;
 
   dreturn("%i", 0);
@@ -1738,7 +1768,7 @@ static int gdpy_entry_setdividend(struct gdpy_entry_t *self, PyObject *value,
   self->E->flags |= comp_scal;
   gd_cs2cs_(self->E->EN(recip,cdividend), cdividend);
   self->E->EN(recip,dividend) = dividend;
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
 
   dreturn("%i", 0);
@@ -1788,7 +1818,7 @@ static int gdpy_entry_setshift(struct gdpy_entry_t *self, PyObject *value,
   }
 
   self->E->EN(phase,shift) = shift;
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
 
   dreturn("%i", 0);
@@ -1839,7 +1869,7 @@ static int gdpy_entry_setcountval(struct gdpy_entry_t *self, PyObject *value,
   }
 
   self->E->EN(mplex,count_val) = count_val;
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
 
   dreturn("%i", 0);
@@ -1890,7 +1920,7 @@ static int gdpy_entry_setperiod(struct gdpy_entry_t *self, PyObject *value,
   }
 
   self->E->EN(mplex,period) = period;
-  free(self->E->scalar[1]);
+  PyMem_Free(self->E->scalar[1]);
   self->E->scalar[1] = scalar;
 
   dreturn("%i", 0);
@@ -1908,7 +1938,7 @@ static PyObject *gdpy_entry_geta(struct gdpy_entry_t *self, void *closure)
     obj = PyTuple_New(self->E->EN(polynom,poly_ord) + 1);
     for (i = 0; i <= self->E->EN(polynom,poly_ord); ++i) {
       PyObject *scalar = gdpyobj_from_scalar(self->E, i, GD_COMPLEX128,
-          &self->E->EN(lincom,ca)[i], self->char_enc);
+          &self->E->EN(polynom,ca)[i], self->char_enc);
       if (scalar == NULL) {
         Py_DECREF(obj);
         obj = NULL;
@@ -1990,7 +2020,7 @@ static int gdpy_entry_seta(struct gdpy_entry_t *self, PyObject *value,
   for (i = 0; i <= self->E->EN(polynom,poly_ord); ++i) {
     self->E->EN(polynom,a)[i] = a[i];
     gd_cs2cs_(self->E->EN(polynom,ca)[i], ca[i]);
-    free(self->E->scalar[i]);
+    PyMem_Free(self->E->scalar[i]);
     self->E->scalar[i] = scalar[i];
   }
   self->E->flags |= comp_scal;
@@ -2077,6 +2107,9 @@ static PyObject *gdpy_entry_getparms(struct gdpy_entry_t *self, void *closure)
       tuple = Py_BuildValue("(iI)", self->E->EN(scalar,const_type),
           self->E->EN(scalar,array_len));
       break;
+    case GD_SARRAY_ENTRY:
+      tuple = Py_BuildValue("(I)", self->E->EN(scalar,array_len));
+      break;
     case GD_RAW_ENTRY:
       s1 = gdpyobj_from_scalar(self->E, 0, GD_UINT_TYPE, &self->E->EN(raw,spf),
           self->char_enc);
@@ -2092,6 +2125,8 @@ static PyObject *gdpy_entry_getparms(struct gdpy_entry_t *self, void *closure)
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
       s1 = gdpyobj_from_string(self->E->in_fields[0], self->char_enc);
       S1CHECK;
       s2 = gdpyobj_from_string(self->E->in_fields[1], self->char_enc);
@@ -2367,7 +2402,7 @@ static int gdpy_entry_setthreshold(struct gdpy_entry_t *self, PyObject *value,
     return -1;
   }
 
-  free(self->E->scalar[0]);
+  PyMem_Free(self->E->scalar[0]);
   self->E->scalar[0] = scalar;
   self->E->EN(window,threshold) = t;
 
@@ -2413,6 +2448,7 @@ static PyObject *gdpy_entry_repr(struct gdpy_entry_t *self)
   /* the parameter tuple */
   parms = gdpy_entry_getparms(self, NULL);
   if (PyErr_Occurred()) {
+    Py_DECREF(name);
     dreturn("%p", NULL);
     return NULL;
   }
@@ -2429,9 +2465,13 @@ static PyObject *gdpy_entry_repr(struct gdpy_entry_t *self)
         gdpy_entry_type_names[self->E->field_type], name,
         self->E->fragment_index, parms, self->char_enc);
 
+#if PY_MAJOR_VERSION < 3
   /* now encode */
   repr = PyUnicode_AsEncodedString(urepr, "ascii", "backslashreplace");
   Py_DECREF(urepr);
+#else
+  repr = urepr;
+#endif
   Py_DECREF(name);
   Py_DECREF(parms);
 
