@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2017 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -31,9 +31,63 @@ int gd_col_count = 0;
 char gd_debug_col[GD_COL_SIZE + 1] = "";
 #endif
 
+/* compare two strings parent/namea and nameb of length total and lenb */
+static int _GD_pstrlencmp(const char *parent, size_t plen,
+    const char *namea, size_t total, const char *nameb, size_t lenb)
+{
+  int c;
+  dtrace("\"%s\", %" PRIuSIZE ", \"%s\", %" PRIuSIZE ", \"%s\", %" PRIuSIZE,
+      parent, plen, namea, total, nameb, lenb);
+
+  if (total < lenb)  {
+    dreturn("%i <", -1);
+    return -1;
+  }
+
+  if (lenb < total) {
+    dreturn("%i <", 1);
+    return 1;
+  }
+
+  /* Compare in two parts */
+  c = memcmp(parent, nameb, plen);
+  if (c == 0)
+    c = nameb[plen] - '/';
+  if (c == 0)
+    c = memcmp(namea, nameb + plen + 1, total - plen - 1);
+
+  dreturn("%i", c);
+  return c;
+}
+
+/* compare two strings namea and nameb of length lena and lenb */
+static int _GD_strlencmp(const char *namea, size_t lena,
+    const char *nameb, size_t lenb)
+{
+  int c;
+  dtrace("\"%s\", %" PRIuSIZE ", \"%s\", %" PRIuSIZE, namea, lena, nameb, lenb);
+
+  if (lena < lenb) {
+    dreturn("%i <", -1);
+    return -1;
+  }
+
+  if (lenb < lena) {
+    dreturn("%i <", 1);
+    return 1;
+  }
+
+  c = memcmp(namea, nameb, lena);
+
+  dreturn("%i", c);
+  return c;
+}
+
 int _GD_EntryCmp(const void *a, const void *b)
 {
-  return strcmp((*(gd_entry_t**)a)->field, (*(gd_entry_t**)b)->field);
+  const gd_entry_t *A = *((gd_entry_t**)a);
+  const gd_entry_t *B = *((gd_entry_t**)b);
+  return _GD_strlencmp(A->field, A->e->len, B->field, B->e->len);
 }
 
 /* _GD_GetLine: read non-comment line from format file.  The line is newly
@@ -95,27 +149,88 @@ gd_type_t _GD_LegacyType(char c)
   return GD_UNKNOWN;
 }
 
-/* Binary search to find the field */
+/* Like _GD_FindField, but look for a subfield code with the parent and
+ * subfield names separated.  This is only used when looking for a field code
+ * which specified a subfield by alias name.  The plen count includes the / */
+static gd_entry_t *_GD_FindFieldWithParent(const char *restrict parent,
+    size_t plen, const char *restrict name, size_t len, gd_entry_t *const *list,
+    unsigned int u, int dealias, unsigned int *restrict index)
+{
+  size_t total;
+  int c;
+  unsigned int i, l = 0;
+
+  dtrace("\"%s\", %" PRIuSIZE ", \"%s\", %" PRIuSIZE ", %p, %u, %i, %p",
+      parent, plen, name, len, list, u, dealias, index);
+
+  /* Drop an initial dot */
+  if (parent[0] == '.' && len > 1) {
+    parent++;
+    plen--;
+  }
+
+  total = plen + len + 1;
+
+  /* Binary search */
+  while (l < u) {
+    i = (l + u) / 2;
+    c = _GD_pstrlencmp(parent, plen, name, total, list[i]->field,
+        list[i]->e->len);
+    if (c < 0)
+      u = i;
+    else if (c > 0)
+      l = i + 1;
+    else {
+      gd_entry_t *E = list[i];
+      if (dealias && E && E->field_type == GD_ALIAS_ENTRY)
+        E = E->e->entry[0];
+
+      if (index != NULL)
+        *index = i;
+
+      dreturn("%p", E);
+      return E;
+    }
+  }
+
+  /* Not found */
+  dreturn("%p", NULL);
+  return NULL;
+}
+
+/* Binary search list of length u to find field_code.  len = strlen(field_code).
+ * If the field found is an alias, the target will be returned if dealias is
+ * non-zero */
 gd_entry_t *_GD_FindField(const DIRFILE *restrict D,
-    const char *restrict field_code, gd_entry_t *const *list, unsigned int u,
-    int dealias, unsigned int *restrict index)
+    const char *restrict field_code, size_t len, gd_entry_t *const *list,
+    unsigned int u, int dealias, unsigned int *restrict index)
 {
   int c;
-  char *ptr;
+  const char *ptr;
   gd_entry_t *E = NULL;
   unsigned int i, l = 0;
   const unsigned int ou = u;
 
-  dtrace("%p, \"%s\", %p, %u, %i, %p", D, field_code, list, u, dealias, index);
+  dtrace("%p, \"%s\", %" PRIuSIZE ", %p, %u, %i, %p", D, field_code, len, list,
+      u, dealias, index);
 
   /* handle FILEFRAM */
-  if (D->standards < 6 && (D->flags & GD_PEDANTIC) &&
-      strcmp(field_code, "FILEFRAM") == 0)
+  if (D->standards < 6 && (D->flags & GD_PEDANTIC) && len == 8 &&
+      memcmp(field_code, "FILEFRAM", 8) == 0)
+  {
     field_code = "INDEX";
+    len = 5;
+  }
+
+  /* Drop an initial dot */
+  if (field_code[0] == '.' && len > 1) {
+    field_code++;
+    len--;
+  }
 
   while (l < u) {
     i = (l + u) / 2;
-    c = strcmp(field_code, list[i]->field);
+    c = _GD_strlencmp(field_code, len, list[i]->field, list[i]->e->len);
     if (c < 0)
       u = i;
     else if (c > 0)
@@ -137,28 +252,15 @@ gd_entry_t *_GD_FindField(const DIRFILE *restrict D,
     *index = u;
 
   /* not found perhaps it's an subfield of an aliased field? */
-  if ((ptr = (char*)strchr(field_code, '/'))) {
-    char *new_code = strdup(field_code);
-    if (new_code) {
-      new_code[ptr - field_code] = '\0';
-      E = _GD_FindField(D, new_code, list, ou, 0, NULL);
-      free(new_code);
+  if (dealias && (ptr = memchr(field_code, '/', len)) != NULL) {
+    E = _GD_FindField(D, field_code, ptr - field_code, list, ou, 0, NULL);
 
-      if (E && E->field_type == GD_ALIAS_ENTRY && E->e->entry[0]) {
-        size_t plen = strlen(E->e->entry[0]->field);
-        new_code = (char*)malloc(plen + strlen(ptr) + 2);
-        if (new_code) {
-          strcpy(new_code, E->e->entry[0]->field);
-          new_code[plen] = '/';
-          strcpy(new_code + plen + 1, ptr + 1);
-
-          E = _GD_FindField(D, new_code, list, ou, 1, NULL);
-
-          free(new_code);
-        }
-      } else
-        E = NULL;
-    }
+    if (E && E->field_type == GD_ALIAS_ENTRY && E->e->entry[0])
+      E = _GD_FindFieldWithParent(E->e->entry[0]->field,
+          E->e->entry[0]->e->len, ptr + 1, len - (ptr - field_code) - 1, list,
+          ou, 1, NULL);
+    else
+      E = NULL;
   }
 
   dreturn("%p", E);
@@ -186,24 +288,12 @@ void* _GD_Alloc(DIRFILE* D, gd_type_t type, size_t n)
   void* ptr = NULL;
 
   dtrace("%p, 0x%x, %" PRIuSIZE, D, type, n);
-  if (n == 0) {
+  if (type == GD_NULL)
+    ; /* just return the NULL */
+  else if (n * GD_SIZE(type) == 0)
     _GD_InternalError(D);
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  if (type == GD_NULL) {
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  if (GD_SIZE(type) == 0) {
-    _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  ptr = _GD_Malloc(D, n * GD_SIZE(type));
+  else
+    ptr = _GD_Malloc(D, n * GD_SIZE(type));
 
   dreturn("%p", ptr);
   return ptr;
@@ -254,7 +344,7 @@ static int lutcmp(const void* a, const void* b)
 
 /* _GD_ReadLinterpFile: Read in the linterp data for this field
 */
-void _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
+int _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
 {
   FILE *fp;
   struct gd_lut_ *ptr;
@@ -270,23 +360,23 @@ void _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
 
   if (E->e->u.linterp.table_file == NULL)
     if (_GD_SetTablePath(D, E, E->e)) {
-      dreturnvoid();
-      return;
+      dreturn("%i", 1);
+      return 1;
     }
 
   fd = gd_OpenAt(D, E->e->u.linterp.table_dirfd, E->e->u.linterp.table_file,
       O_RDONLY, 0666);
   if (fd == -1) {
     _GD_SetError(D, GD_E_IO, GD_E_IO_OPEN, E->EN(linterp,table), 0, NULL);
-    dreturnvoid();
-    return;
+    dreturn("%i", 1);
+    return 1;
   }
 
   fp = fdopen(fd, "rb");
   if (fp == NULL) {
     _GD_SetError(D, GD_E_IO, GD_E_IO_OPEN, E->EN(linterp,table), 0, NULL);
-    dreturnvoid();
-    return;
+    dreturn("%i", 1);
+    return 1;
   }
 
   E->e->u.linterp.lut = _GD_Malloc(D, buf_len * sizeof(*E->e->u.linterp.lut));
@@ -309,7 +399,9 @@ void _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
       /* line too long */
       _GD_SetError(D, GD_E_LINE_TOO_LONG, 0, E->EN(linterp,table), linenum,
           NULL);
-    else 
+    else if (errno)
+      _GD_SetError(D, GD_E_IO, GD_E_IO_READ, E->EN(linterp,table), 0, NULL);
+    else
       /* no data in file! */
       _GD_SetError(D, GD_E_LUT, GD_E_LUT_LENGTH, E->EN(linterp,table), 0, NULL);
     goto LUT_ERROR;
@@ -379,15 +471,16 @@ void _GD_ReadLinterpFile(DIRFILE *restrict D, gd_entry_t *restrict E)
     qsort(E->e->u.linterp.lut, i, sizeof(struct gd_lut_), lutcmp);
 
   fclose(fp);
-  dreturnvoid();
-  return;
+  dreturn("%i", 0);
+  return 0;
 
 LUT_ERROR:
   free(E->e->u.linterp.lut);
   E->e->u.linterp.lut = NULL;
   free(line);
   fclose(fp);
-  dreturnvoid();
+  dreturn("%i", 1);
+  return 1;
 }
 
 /* _GD_GetIndex: get LUT index.
@@ -505,7 +598,7 @@ void _GD_LinterpData(DIRFILE *restrict D, void *restrict data, gd_type_t type,
     case GD_FLOAT64:    LINTERP(double  ); break;
     case GD_COMPLEX64:  LINTERPC(float  ); break;
     case GD_COMPLEX128: LINTERPC(double ); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
+    default:         _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -762,33 +855,24 @@ void _GD_InvertData(DIRFILE *restrict D, void *restrict data,
 
 #ifdef GD_NO_C99_API
 #undef INVERTC
-#undef INVERT
 
 #define INVERTC(t) \
   do { \
     for (i = 0; i < n_read; i++) { \
-      const t d = ((t *)data)[2 * i] * ((t *)data)[2 * i] + \
-      ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
-      ((t *)data)[2 * i] = (t)((dividend[0] * ((t *)data)[2 * i] + \
-          dividend[1] * ((t *)data)[2 * i + 1]) / d); \
-      ((t *)data)[2 * i + 1] = (t)((dividend[1] * ((t *)data)[2 * i] - \
-          dividend[0] * ((t *)data)[2 * i]) / d); \
-    } \
-  } while (0)
-
-#define INVERT(t) \
-  do { \
-    for (i = 0; i < n_read; i++) { \
-      const t d = ((t *)data)[2 * i] * ((t *)data)[2 * i] + \
-      ((t *)data)[2 * i + 1] * ((t *)data)[2 * i + 1]; \
-      ((t *)data)[2 * i] = (t)((dividend[0] * ((t *)data)[2 * i] - \
-          dividend[1] * ((t *)data)[2 * i + 1]) / d); \
+      const t re = ((t*)data)[2 * i]; \
+      const t im = ((t*)data)[2 * i + 1]; \
+      const t d = (re * re + im * im); \
+      ((t *)data)[2 * i] = (t)((dividend[0] * re + dividend[1] * im) / d); \
+      ((t *)data)[2 * i + 1] = (t)((dividend[1] * re - dividend[0] * im) / d); \
     } \
   } while (0)
 
 #endif
 
-/* Invert a vector */
+/* Invert a vector using a complex valued dividend.  When this is
+ * called, return_type is always complex valued, too (because DoField
+ * decides to compute the full complex value first, and then convert
+ * to the real type afterwards. */
 void _GD_CInvertData(DIRFILE *restrict D, void *restrict data,
     gd_type_t return_type, GD_DCOMPLEXA(dividend), size_t n_read)
 {
@@ -799,35 +883,26 @@ void _GD_CInvertData(DIRFILE *restrict D, void *restrict data,
 
   switch(return_type) {
     case GD_NULL:                         break;
-    case GD_UINT8:      INVERT(uint8_t);  break;
-    case GD_INT8:       INVERT(int8_t);   break;
-    case GD_UINT16:     INVERT(uint16_t); break;
-    case GD_INT16:      INVERT(int16_t);  break;
-    case GD_UINT32:     INVERT(uint32_t); break;
-    case GD_INT32:      INVERT(int32_t);  break;
-    case GD_UINT64:     INVERT(uint64_t); break;
-    case GD_INT64:      INVERT(int64_t);  break;
-    case GD_FLOAT32:    INVERT(float);    break;
-    case GD_FLOAT64:    INVERT(double);   break;
     case GD_COMPLEX64:  INVERTC(float);   break;
     case GD_COMPLEX128: INVERTC(double);  break;
-    default:            _GD_InternalError(D);
+    default:            _GD_InternalError(D); /* Other types not allowed */
   }
 
   dreturnvoid();
 }
 
-int _GD_GetRepr(DIRFILE *restrict D, const char *restrict field_code_in,
-    char **restrict field_code, int err)
+/* Look for a representation suffix in field_code.  If one is found, decrement
+ * (*len) by two.  Returns the representation found.
+ */
+int _GD_GetRepr(const char *restrict field_code, size_t *restrict len)
 {
-  int repr = GD_REPR_NONE;
-  const int field_code_len = strlen(field_code_in);
+  int repr = -1;
 
-  dtrace("%p, \"%s\", %p, %i", D, field_code_in, field_code, err);
+  dtrace("\"%s\", %p", field_code, len);
 
   /* find the representation, if any */
-  if (field_code_len > 2 && field_code_in[field_code_len - 2] == '.') {
-    switch (field_code_in[field_code_len - 1]) {
+  if (*len > 2 && field_code[*len - 2] == '.') {
+    switch (field_code[*len - 1]) {
       case 'r':
         repr = GD_REPR_REAL;
         break;
@@ -840,65 +915,21 @@ int _GD_GetRepr(DIRFILE *restrict D, const char *restrict field_code_in,
       case 'a':
         repr = GD_REPR_ARG;
         break;
-      default:
-        if (err)
-          _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0,
-              field_code_in);
-        dreturn("%i", 0);
-        return 0;
+      case 'z':
+        repr = GD_REPR_NONE;
+        break;
     }
+  }
 
-    /* make a copy of the field code without the representation */
-    *field_code = _GD_Strdup(D, field_code_in);
-    if (*field_code)
-      (*field_code)[field_code_len - 2] = '\0';
-  } else
-    *field_code = (char *)field_code_in;
+  if (repr == -1) /* no match */
+    repr = GD_REPR_NONE;
+  else
+    *len -= 2;
 
-  dreturn("%i", repr);
+  dreturn("0x%X (%" PRIuSIZE ")", repr, *len);
   return repr;
 }
 
-/* Ensure that an input field has been identified (with error checking) */
-int _GD_BadInput(DIRFILE *D, const gd_entry_t *E, int i, gd_entype_t t, int err)
-{
-  char *code;
-
-  dtrace("%p, %p, %i, 0x%X, %i", D, E, i, t, err);
-
-  if (E->e->entry[i] == NULL) {
-    E->e->entry[i] = _GD_FindFieldAndRepr(D, E->in_fields[i], &code,
-        &E->e->repr[i], NULL, 1, err);
-
-    if (E->e->entry[i] == NULL) {
-      dreturn("%i", 1);
-      return 1;
-    }
-
-    if (code != E->in_fields[i])
-      free(code);
-  }
-
-  /* check field type */
-  if (t == GD_NO_ENTRY) {
-    /* scalar entries not allowed */
-    if (E->e->entry[i]->field_type & GD_SCALAR_ENTRY_BIT) {
-      _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_FORMAT, E->field, 0,
-          E->e->entry[i]->field);
-      dreturn("%i", 1);
-      return 1;
-    }
-  } else if (E->e->entry[i]->field_type != t) {
-    _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_FORMAT, E->field, 0,
-        E->e->entry[i]->field);
-    dreturn("%i", 1);
-    return 1;
-  }
-
-  dreturn("%i", 0);
-  return 0;
-}
-
 /* Find an entry without a representation; this function will de-alias */
 gd_entry_t *_GD_FindEntry(DIRFILE *restrict D, const char *restrict field_code)
 {
@@ -906,11 +937,9 @@ gd_entry_t *_GD_FindEntry(DIRFILE *restrict D, const char *restrict field_code)
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->n_dot > 0)
-    E = _GD_FindField(D, field_code, D->dot_list, D->n_dot, 1, NULL);
-
   if (E == NULL)
-    E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+    E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+        1, NULL);
 
   if (E == NULL)
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
@@ -921,40 +950,46 @@ gd_entry_t *_GD_FindEntry(DIRFILE *restrict D, const char *restrict field_code)
 
 /* Find the entry and the representation */
 gd_entry_t *_GD_FindFieldAndRepr(DIRFILE *restrict D,
-    const char *restrict field_code_in, char **restrict field_code,
-    int *restrict repr, unsigned int *restrict index, int set, int err)
+    const char *restrict field_code, int *restrict repr,
+    unsigned int *restrict index, int err)
 {
+  size_t old_len, len;
   gd_entry_t *E = NULL;
 
-  dtrace("%p, \"%s\", %p, %p, %p, %i, %i", D, field_code_in, field_code, repr,
-      index, set, err);
+  dtrace("%p, \"%s\", %p, %p, %i", D, field_code, repr, index, err);
 
-  if (D->n_dot > 0)
-    E = _GD_FindField(D, field_code_in, D->dot_list, D->n_dot, 1, NULL);
+  old_len = len = strlen(field_code);
 
-  if (E == NULL) {
-    *repr = _GD_GetRepr(D, field_code_in, field_code, err);
+  /* Find a representaiton suffix.  We check this first in order to give
+   * the caller a chance to distinguish ambiguous codes by appending a .z */
+  *repr = _GD_GetRepr(field_code, &len);
 
-    if (D->error) {
-      dreturn("%p", NULL);
-      return NULL;
+  /* repr found? -- it's not enough to check that repr != GD_REPR_NONE, since
+   * _GD_GetRepr returns GD_REPR_NONE in that case
+   */
+  if (old_len != len) {
+    E = _GD_FindField(D, field_code, len, D->entry, D->n_entries, 1, index);
+  
+    /* SARRAY, SINDIR, STRING only permit the .z suffix */
+    if (E && *repr != GD_REPR_NONE && (E->field_type == GD_SARRAY_ENTRY
+          || E->field_type == GD_SINDIR_ENTRY
+          || E->field_type == GD_STRING_ENTRY))
+    {
+      E = NULL; /* Force match failed */
     }
-  } else {
+  }
+
+  /* No repr, or match failed. Try the whole code. */
+  if (E == NULL)  {
     *repr = GD_REPR_NONE;
-    *field_code = (char *)field_code_in;
+    E = _GD_FindField(D, field_code, old_len, D->entry, D->n_entries, 1, index);
   }
 
-  if (E == NULL || index != NULL)
-    E = _GD_FindField(D, *field_code, D->entry, D->n_entries, 1, index);
+  /* Set error, if requested */
+  if (E == NULL && err)
+    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
 
-  if (E == NULL && set) {
-    if (err)
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code_in);
-    if (field_code_in != *field_code)
-      free(*field_code);
-  }
-
-  dreturn("%p %i", E, *repr);
+  dreturn("%p 0x%X", E, *repr);
   return E;
 }
 
@@ -1001,7 +1036,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
   if (car && !_GD_AbsPath(cdr)) {
     if (!_GD_AbsPath(car)) {
       /* car is not abosulte -- don't bother trying to do anything fancy */
-      res = (char*)malloc(strlen(car) + strlen(cdr) + 2);
+      res = malloc(strlen(car) + strlen(cdr) + 2);
       if (res == NULL) {
         dreturn("%p", NULL);
         return NULL;
@@ -1029,7 +1064,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
   } else if (_GD_AbsPath(cdr)) {
     /* cdr is absolute: make res "/" and copy cdr relative to / into work;
      * ignore car */
-    res = (char*)malloc(res_size = PATH_MAX);
+    res = malloc(res_size = PATH_MAX);
     if (res == NULL) {
       dreturn("%p", NULL);
       return NULL;
@@ -1053,8 +1088,8 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
      * work CWD/cdr, relative to / if getcwd returned an absolute path.  If
      * getcwd fails, just use cdr, and abandon hope of an absolute path.
      */
-    res = (char*)malloc(res_size = PATH_MAX);
-    work = (char*)malloc(PATH_MAX);
+    res = malloc(res_size = PATH_MAX);
+    work = malloc(PATH_MAX);
     if (res == NULL || work == NULL) {
       free(res);
       dreturn("%p", NULL);
@@ -1074,7 +1109,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
       res_root = res_len = 0;
     } else {
       if ((len = strlen(work) + 2 + strlen(cdr)) < PATH_MAX) {
-        ptr = (char*)realloc(work, len);
+        ptr = realloc(work, len);
         if (ptr == NULL) {
           free(res);
           free(work);
@@ -1134,7 +1169,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
       /* a path element, copy it to res */
       len = strlen(cur) + 1;
       if (res_len + len >= res_size) {
-        ptr = (char*)realloc(res, res_size += (len < 1000) ? 1000 : len);
+        ptr = realloc(res, res_size += (len < 1000) ? 1000 : len);
         if (ptr == NULL) {
           free(res);
           free(work);
@@ -1160,7 +1195,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
             if (*end) {
               len = strlen(end) + 1;
               if (res_len + len >= res_size) {
-                ptr = (char*)realloc(res, res_size += len);
+                ptr = realloc(res, res_size += len);
                 if (ptr == NULL) {
                   free(res);
                   free(work);
@@ -1248,7 +1283,7 @@ char *_GD_CanonicalPath(const char *car, const char *cdr)
               len--;
             }
 
-            new_work = (char*)malloc(len);
+            new_work = malloc(len);
             if (new_work == NULL) {
               free(res);
               free(work);
@@ -1271,7 +1306,7 @@ _GD_CanonicalPath_DONE:
   free(work);
 
   /* trim */
-  ptr = (char*)realloc(res, res_len + 1);
+  ptr = realloc(res, res_len + 1);
   if (ptr)
     res = ptr;
 
@@ -1434,6 +1469,11 @@ void _GD_ReleaseDir(DIRFILE *D, int dirfd)
   dreturnvoid();
 }
 
+/* Caller's memory manager */
+void *(*_GD_CMalloc)(size_t) = malloc;
+void (*_GD_CFree)(void*) = free;
+char *(*_GD_CStrdup)(const char*) = strdup;
+
 /* allocation boilerplates */
 void *_GD_Malloc(DIRFILE *D, size_t size)
 {
