@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2015 D. V. Wiebe
+/* Copyright (C) 2008-2017 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -73,7 +73,7 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_GzipStrerr
 #else
 #define GD_EF_PROVIDES 0
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERICNOP_SET
 #endif
   GD_EXT_ENCODING_GENOP(GD_GZIP_ENCODED, ".gz", GD_EF_ECOR | GD_EF_OOP, "Gzip",
       "gzip"),
@@ -90,7 +90,7 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_Bzip2Read, &_GD_Bzip2Size, &_GD_Bzip2Write, &_GD_NopSync, \
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_Bzip2Strerr
 #else
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERICNOP_SET
 #define GD_EF_PROVIDES 0
 #endif
   GD_EXT_ENCODING_GENOP(GD_BZIP2_ENCODED, ".bz2", GD_EF_ECOR | GD_EF_OOP,
@@ -107,7 +107,7 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_SlimRead, &_GD_SlimSize, NULL /* WRITE */, &_GD_NopSync, \
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_SlimStrerr
 #else
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERICNOP_SET
 #define GD_EF_PROVIDES 0
 #endif
   GD_EXT_ENCODING_GENOP(GD_SLIM_ENCODED, ".slm", GD_EF_ECOR, "Slim", "slim"),
@@ -125,7 +125,7 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_LzmaRead, &_GD_LzmaSize, &_GD_LzmaWrite, &_GD_LzmaSync, \
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_LzmaStrerr
 #else
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERIC_SET
 #define GD_EF_PROVIDES 0
 #endif
   GD_EXT_ENCODING_GEN(GD_LZMA_ENCODED, ".xz", GD_EF_ECOR | GD_EF_OOP, "Lzma",
@@ -141,7 +141,7 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_LzmaRead, &_GD_LzmaSize, NULL /* WRITE */, NULL /* SYNC */, \
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_LzmaStrerr
 #else
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERIC_SET
 #define GD_EF_PROVIDES 0
 #endif
   GD_EXT_ENCODING_GEN(GD_LZMA_ENCODED, ".lzma", GD_EF_ECOR, "Lzma", "lzma"),
@@ -207,10 +207,10 @@ struct encoding_t _GD_ef[GD_N_SUBENCODINGS] = {
   &_GD_FlacRead, &_GD_FlacSize, &_GD_FlacWrite, &_GD_NopSync, \
   &_GD_GenericMove, &_GD_GenericUnlink, &_GD_FlacStrerr
 #else
-#define GD_INT_FUNCS GD_EF_NULL_SET
+#define GD_INT_FUNCS GD_EF_GENERICNOP_SET
 #define GD_EF_PROVIDES 0
 #endif
-  GD_EXT_ENCODING_GENOP(GD_FLAC_ENCODED, ".flac", GD_EF_ECOR | GD_EF_OOP,
+  GD_EXT_ENCODING_GENOP(GD_FLAC_ENCODED, ".flac", GD_EF_SWAP | GD_EF_OOP,
       "Flac", "flac"),
 #undef GD_INT_FUNCS
 #undef GD_EF_PROVIDES
@@ -286,7 +286,7 @@ int _GD_MissingFramework(int encoding, unsigned int funcs)
     lt_dlhandle lib;
 
     /* make the library name */
-    library = (char *)malloc(sizeof(GETDATA_MODULEDIR) +
+    library = malloc(sizeof(GETDATA_MODULEDIR) +
         strlen(_GD_ef[encoding].affix) + sizeof(GD_GETDATA_VERSION) + 13);
     if (!library) {
       _GD_ef[encoding].provides = 0;
@@ -450,7 +450,8 @@ int _GD_FiniRawIO(DIRFILE *D, const gd_entry_t *E, int fragment, int flags)
 
         do {
           n_to_write = n_read = (*_GD_ef[E->e->u.raw.file[0].subenc].read)(
-              E->e->u.raw.file, buffer, E->EN(raw,data_type), GD_BUFFER_SIZE);
+              E->e->u.raw.file, buffer, E->EN(raw,data_type),
+              GD_BUFFER_SIZE / GD_SIZE(E->EN(raw,data_type)));
           if (n_read < 0) {
             free(buffer);
             _GD_SetEncIOError(D, GD_E_IO_READ, E->e->u.raw.file + 0);
@@ -467,7 +468,7 @@ int _GD_FiniRawIO(DIRFILE *D, const gd_entry_t *E, int fragment, int flags)
             }
             n_to_write -= n_wrote;
           }
-        } while (n_read == GD_BUFFER_SIZE);
+        } while (n_read > 0);
 
         free(buffer);
       }
@@ -602,6 +603,8 @@ int _GD_InitRawIO(DIRFILE *D, const gd_entry_t *E, const char *filebase,
         return 1;
       }
     }
+    if (oop_write)
+      E->e->u.raw.file[1].subenc = E->e->u.raw.file[0].subenc;
   }
 
   if (filebase == NULL)
@@ -679,7 +682,7 @@ static unsigned long _GD_ResolveEncoding(DIRFILE *restrict D,
   for (i = 0; _GD_ef[i].scheme != GD_ENC_UNSUPPORTED; i++) {
     if (scheme == GD_AUTO_ENCODED || scheme == _GD_ef[i].scheme) {
       if (_GD_ef[i].ext) {
-        candidate = (char *)malloc(len + strlen(_GD_ef[i].ext) + 1);
+        candidate = malloc(len + strlen(_GD_ef[i].ext) + 1);
         if (!candidate)
           continue;
 
@@ -905,33 +908,15 @@ int gd_alter_encoding(DIRFILE* D, unsigned long encoding, int fragment,
 
   dtrace("%p, %lu, %i, %i", D, (unsigned long)encoding, fragment, move);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if ((D->flags & GD_ACCMODE) != GD_RDWR) {
+  if ((D->flags & GD_ACCMODE) != GD_RDWR)
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (fragment < GD_ALL_FRAGMENTS || fragment >= D->n_fragment) {
+  else if (fragment < GD_ALL_FRAGMENTS || fragment >= D->n_fragment)
     _GD_SetError(D, GD_E_BAD_INDEX, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (!_GD_EncodingUnderstood(encoding)) {
+  else if (!_GD_EncodingUnderstood(encoding))
     _GD_SetError(D, GD_E_UNKNOWN_ENCODING, GD_E_UNENC_TARGET, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
-
-  if (fragment == GD_ALL_FRAGMENTS) {
+  else if (fragment == GD_ALL_FRAGMENTS) {
     for (i = 0; i < D->n_fragment; ++i) {
       _GD_RecodeFragment(D, encoding, i, move);
 
@@ -941,8 +926,7 @@ int gd_alter_encoding(DIRFILE* D, unsigned long encoding, int fragment,
   } else
     _GD_RecodeFragment(D, encoding, fragment, move);
 
-  dreturn("%i", (D->error) ? -1 : 0);
-  return (D->error) ? -1 : 0;
+  GD_RETURN_ERROR(D);
 }
 
 unsigned long gd_encoding(DIRFILE* D, int fragment) gd_nothrow
@@ -952,11 +936,7 @@ unsigned long gd_encoding(DIRFILE* D, int fragment) gd_nothrow
 
   dtrace("%p, %i", D, fragment);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", 0);
-    return 0;
-  }
+  GD_RETURN_IF_INVALID(D, "%i", 0);
 
   if (fragment < 0 || fragment >= D->n_fragment) {
     _GD_SetError(D, GD_E_BAD_INDEX, 0, NULL, 0, NULL);
@@ -964,8 +944,6 @@ unsigned long gd_encoding(DIRFILE* D, int fragment) gd_nothrow
     return 0;
   }
 
-  _GD_ClearError(D);
-
   /* Attempt to figure out the encoding, if it's not known */
   if (D->fragment[fragment].encoding == GD_AUTO_ENCODED) {
     /* locate a RAW field in this fragment */
@@ -1004,8 +982,8 @@ int gd_encoding_support(unsigned long encoding) gd_nothrow
 
   /* make sure we have a valid encoding */
   if (!_GD_EncodingUnderstood(encoding)) {
-    dreturn("%i", -1);
-    return -1;
+    dreturn("%i", GD_E_UNKNOWN_ENCODING);
+    return GD_E_UNKNOWN_ENCODING;
   }
 
   /* spin up ltdl if needed */
@@ -1030,8 +1008,8 @@ int gd_encoding_support(unsigned long encoding) gd_nothrow
     }
 
   /* nope */
-  dreturn("%i", -1);
-  return -1;
+  dreturn("%i", GD_E_UNSUPPORTED);
+  return GD_E_UNSUPPORTED;
 }
 
 /* This is basically the non-existant POSIX funcion mkstempat.  There are two
