@@ -37,12 +37,23 @@ void _GD_FreeFL(struct gd_flist_ *fl)
     free(fl->carray_value_list);
   }
 
+  if (fl->sarray_value_list) {
+    for (i = 0; fl->sarray_value_list[i] != NULL; ++i)
+      free(fl->sarray_value_list[i]);
+    free(fl->sarray_value_list);
+  }
+
   dreturnvoid();
 }
 
 void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
 {
-  int i;
+  int i, j;
+  size_t n;
+
+  /* If priv == 0, then we've been called via gd_free_entry_strings and
+   * need to use the caller's free function on the public strings */
+  void (*free_)(void*) = priv ? free : _GD_CFree;
 
   dtrace("%p, %p, %i", D, entry, priv);
 
@@ -51,19 +62,24 @@ void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
     return;
   }
 
-  free(entry->field);
+  free_(entry->field);
 
   switch(entry->field_type) {
     case GD_LINCOM_ENTRY:
-      for (i = 0; i < entry->EN(lincom,n_fields); ++i) {
-        free(entry->in_fields[i]);
-        free(entry->scalar[i]);
-        free(entry->scalar[i + GD_MAX_LINCOM]);
+      /* Don't trust the caller */
+      j = entry->EN(lincom,n_fields);
+      if (j > GD_MAX_LINCOM)
+        j = GD_MAX_LINCOM;
+
+      for (i = 0; i < j; ++i) {
+        free_(entry->in_fields[i]);
+        free_(entry->scalar[i]);
+        free_(entry->scalar[i + GD_MAX_LINCOM]);
       }
       break;
     case GD_LINTERP_ENTRY:
-      free(entry->in_fields[0]);
-      free(entry->EN(linterp,table));
+      free_(entry->in_fields[0]);
+      free_(entry->EN(linterp,table));
       if (priv) {
         if (entry->e->u.linterp.table_dirfd > 0)
           _GD_ReleaseDir(D, entry->e->u.linterp.table_dirfd);
@@ -74,29 +90,44 @@ void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
       break;
     case GD_DIVIDE_ENTRY:
     case GD_MULTIPLY_ENTRY:
-      free(entry->in_fields[1]);
-      free(entry->in_fields[0]);
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
+      free_(entry->in_fields[1]);
+      free_(entry->in_fields[0]);
       break;
     case GD_BIT_ENTRY:
     case GD_SBIT_ENTRY:
-      free(entry->scalar[0]);
-      free(entry->scalar[1]);
-      free(entry->in_fields[0]);
+      free_(entry->scalar[0]);
+      free_(entry->scalar[1]);
+      free_(entry->in_fields[0]);
       break;
     case GD_PHASE_ENTRY:
     case GD_RECIP_ENTRY:
-      free(entry->scalar[0]);
-      free(entry->in_fields[0]);
+      free_(entry->scalar[0]);
+      free_(entry->in_fields[0]);
       break;
     case GD_POLYNOM_ENTRY:
-      free(entry->in_fields[0]);
-      for (i = 0; i <= entry->EN(polynom,poly_ord); ++i)
-        free(entry->scalar[i]);
+      free_(entry->in_fields[0]);
+
+      /* Don't trust the caller */
+      j = entry->EN(polynom,poly_ord);
+      if (j > GD_MAX_POLYORD)
+        j = GD_MAX_POLYORD;
+
+      for (i = 0; i <= j; ++i)
+        free_(entry->scalar[i]);
       break;
     case GD_STRING_ENTRY:
       if (priv)
         free(entry->e->u.string);
       break;
+    case GD_SARRAY_ENTRY:
+      if (priv) {
+        for (n = 0; n < entry->EN(scalar,array_len); ++n)
+          free(((char **)entry->e->u.scalar.d)[n]);
+        free(entry->e->u.scalar.d);
+      }
+      break;
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
       if (priv) {
@@ -105,7 +136,7 @@ void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
       }
       break;
     case GD_RAW_ENTRY:
-      free(entry->scalar[0]);
+      free_(entry->scalar[0]);
       if (priv) {
         free(entry->e->u.raw.filebase);
         free(entry->e->u.raw.file[0].name);
@@ -113,18 +144,18 @@ void _GD_FreeE(DIRFILE *restrict D, gd_entry_t *restrict entry, int priv)
       }
       break;
     case GD_WINDOW_ENTRY:
-      free(entry->scalar[0]);
-      free(entry->in_fields[0]);
-      free(entry->in_fields[1]);
+      free_(entry->scalar[0]);
+      free_(entry->in_fields[0]);
+      free_(entry->in_fields[1]);
       break;
     case GD_MPLEX_ENTRY:
-      free(entry->scalar[0]);
-      free(entry->scalar[1]);
-      free(entry->in_fields[0]);
-      free(entry->in_fields[1]);
+      free_(entry->scalar[0]);
+      free_(entry->scalar[1]);
+      free_(entry->in_fields[0]);
+      free_(entry->in_fields[1]);
       break;
     case GD_ALIAS_ENTRY:
-      free(entry->in_fields[0]);
+      free_(entry->in_fields[0]);
       break;
     case GD_INDEX_ENTRY:
     case GD_NO_ENTRY:
@@ -164,21 +195,12 @@ int _GD_GetScalar(DIRFILE *restrict D, const char *restrict scalar,
 {
   gd_entry_t* C = NULL;
   int repr, e = 0;
-  char* field_code;
   int index = *index_in;
 
   dtrace("%p, \"%s\", %p(%i), 0x%02X, %p, %p", D, scalar, index_in, *index_in,
       type, data, E);
 
-  C = _GD_FindFieldAndRepr(D, scalar, &field_code, &repr, NULL, 0, 1);
-
-  if (D->error) {
-    dreturn("%i", 1);
-    return 1;
-  }
-
-  if (field_code != scalar)
-    free(field_code);
+  C = _GD_FindFieldAndRepr(D, scalar, &repr, NULL, 0);
 
   if (C == NULL)
     e = GD_E_SCALAR_CODE;
@@ -214,6 +236,8 @@ int _GD_GetScalar(DIRFILE *restrict D, const char *restrict scalar,
   return e;
 }
 
+/* Like _GD_GetScalar, but with more error checking.  And a different calling
+ * convention */
 static int _GD_GetScalar2(DIRFILE *restrict D, gd_entry_t *restrict E, int i,
     gd_type_t type, void *restrict data, int err)
 {
@@ -304,7 +328,8 @@ int _GD_CalculateEntry(DIRFILE *restrict D, gd_entry_t *restrict E, int err)
       switch (E->EN(window,windop)) {
         case GD_WINDOP_EQ:
         case GD_WINDOP_NE:
-          e = _GD_GetScalar2(D, E, 0, GD_INT64, &E->EN(window,threshold.i), err);
+          e = _GD_GetScalar2(D, E, 0, GD_INT64, &E->EN(window,threshold.i),
+              err);
           break;
         case GD_WINDOP_SET:
         case GD_WINDOP_CLR:
@@ -328,6 +353,9 @@ int _GD_CalculateEntry(DIRFILE *restrict D, gd_entry_t *restrict E, int err)
     case GD_STRING_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_INDEX_ENTRY:
     case GD_ALIAS_ENTRY:
       break;
@@ -347,18 +375,12 @@ int _GD_CalculateEntry(DIRFILE *restrict D, gd_entry_t *restrict E, int err)
 
 char* gd_raw_filename(DIRFILE* D, const char* field_code) gd_nothrow
 {
-  char *filename;
+  char *filename0, *filename;
   gd_entry_t *E;
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  _GD_ClearError(D);
-
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
+  GD_RETURN_IF_INVALID(D, "%p", NULL);
 
   /* Check field */
   E = _GD_FindEntry(D, field_code);
@@ -375,7 +397,7 @@ char* gd_raw_filename(DIRFILE* D, const char* field_code) gd_nothrow
   }
 
   if (E->e->u.raw.file[0].name == NULL) {
-    /* ensure encoding sybtype is known */
+    /* ensure encoding subtype is known */
     if (!_GD_Supports(D, E, GD_EF_NAME)) {
       dreturn("%p", NULL);
       return NULL;
@@ -394,9 +416,23 @@ char* gd_raw_filename(DIRFILE* D, const char* field_code) gd_nothrow
     }
   }
 
-  filename = _GD_MakeFullPath(D, D->fragment[E->fragment_index].dirfd,
+  filename0 = _GD_MakeFullPath(D, D->fragment[E->fragment_index].dirfd,
       E->e->u.raw.file->name, 1);
 
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  /* move to caller's heap */
+  if (_GD_CMalloc != malloc) {
+    filename = _GD_CStrdup(filename0);
+    free(filename0);
+    if (filename == NULL)
+      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  } else
+    filename = filename0;
+
   dreturn("%p", filename);
   return filename;
 }
@@ -408,20 +444,12 @@ int gd_entry(DIRFILE* D, const char* field_code, gd_entry_t* entry) gd_nothrow
 
   dtrace("%p, \"%s\", %p", D, field_code, entry);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", 1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   E = _GD_FindEntry(D, field_code);
 
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (D->error) 
+    GD_RETURN_ERROR(D);
 
   /* Calculate the entry, if necessary */
   if (!(E->flags & GD_EN_CALC))
@@ -431,70 +459,73 @@ int gd_entry(DIRFILE* D, const char* field_code, gd_entry_t* entry) gd_nothrow
   memcpy(entry, E, sizeof(gd_entry_t));
   entry->e = NULL;
 
-  /* duplicate strings */
-  entry->field = strdup(E->field);
+  /* duplicate strings using the caller's memory manager */
+  entry->field = _GD_CStrdup(E->field);
 
   switch(E->field_type) {
     case GD_LINCOM_ENTRY:
       for (i = 0; i < E->EN(lincom,n_fields); ++i) {
-        entry->in_fields[i] = strdup(E->in_fields[i]);
+        entry->in_fields[i] = _GD_CStrdup(E->in_fields[i]);
         if (E->scalar[i])
-          entry->scalar[i] = strdup(E->scalar[i]);
+          entry->scalar[i] = _GD_CStrdup(E->scalar[i]);
         if (E->scalar[i + GD_MAX_LINCOM])
-          entry->scalar[i + GD_MAX_LINCOM] = strdup(E->scalar[i +
+          entry->scalar[i + GD_MAX_LINCOM] = _GD_CStrdup(E->scalar[i +
               GD_MAX_LINCOM]);
       }
       break;
     case GD_LINTERP_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
-      entry->EN(linterp,table) = strdup(E->EN(linterp,table));
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
+      entry->EN(linterp,table) = _GD_CStrdup(E->EN(linterp,table));
       break;
     case GD_POLYNOM_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
       for (i = 0; i <= E->EN(polynom,poly_ord); ++i)
         if (E->scalar[i])
-          entry->scalar[i] = strdup(E->scalar[i]);
+          entry->scalar[i] = _GD_CStrdup(E->scalar[i]);
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
-      entry->in_fields[1] = strdup(E->in_fields[1]);
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
+      entry->in_fields[1] = _GD_CStrdup(E->in_fields[1]);
       break;
     case GD_RECIP_ENTRY:
     case GD_PHASE_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
       if (E->scalar[0])
-        entry->scalar[0] = strdup(E->scalar[0]);
+        entry->scalar[0] = _GD_CStrdup(E->scalar[0]);
       break;
     case GD_BIT_ENTRY:
     case GD_SBIT_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
       if (E->scalar[0])
-        entry->scalar[0] = strdup(E->scalar[0]);
+        entry->scalar[0] = _GD_CStrdup(E->scalar[0]);
       if (E->scalar[1])
-        entry->scalar[1] = strdup(E->scalar[1]);
+        entry->scalar[1] = _GD_CStrdup(E->scalar[1]);
       break;
     case GD_RAW_ENTRY:
       if (E->scalar[0])
-        entry->scalar[0] = strdup(E->scalar[0]);
+        entry->scalar[0] = _GD_CStrdup(E->scalar[0]);
       break;
     case GD_WINDOW_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
-      entry->in_fields[1] = strdup(E->in_fields[1]);
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
+      entry->in_fields[1] = _GD_CStrdup(E->in_fields[1]);
       if (E->scalar[0])
-        entry->scalar[0] = strdup(E->scalar[0]);
+        entry->scalar[0] = _GD_CStrdup(E->scalar[0]);
       break;
     case GD_MPLEX_ENTRY:
-      entry->in_fields[0] = strdup(E->in_fields[0]);
-      entry->in_fields[1] = strdup(E->in_fields[1]);
+      entry->in_fields[0] = _GD_CStrdup(E->in_fields[0]);
+      entry->in_fields[1] = _GD_CStrdup(E->in_fields[1]);
       if (E->scalar[0])
-        entry->scalar[0] = strdup(E->scalar[0]);
+        entry->scalar[0] = _GD_CStrdup(E->scalar[0]);
       if (E->scalar[1])
-        entry->scalar[1] = strdup(E->scalar[1]);
+        entry->scalar[1] = _GD_CStrdup(E->scalar[1]);
       break;
     case GD_INDEX_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_STRING_ENTRY:
     case GD_NO_ENTRY:
     case GD_ALIAS_ENTRY:
@@ -511,13 +542,10 @@ const char *gd_alias_target(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
+  GD_RETURN_IF_INVALID(D, "%p", NULL);
 
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+  E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+      0, NULL);
 
   if (E == NULL) {
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
@@ -538,34 +566,21 @@ const char *gd_alias_target(DIRFILE *D, const char *field_code) gd_nothrow
 const char **gd_aliases(DIRFILE *D, const char *field_code) gd_nothrow
 {
   gd_entry_t *E;
-  int j = 1;
-  unsigned u, n;
+  unsigned u, n = 1, len = 10;
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
+  GD_RETURN_IF_INVALID(D, "%p", NULL);
 
-  n = gd_naliases(D, field_code) + 1;
-
-  if (D->error) {
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+  E = _GD_FindEntry(D, field_code);
 
   if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%p", NULL);
     return NULL;
   }
 
-  E->e->alias_list = (const char**)_GD_Realloc(D, E->e->alias_list,
-      sizeof(const char *) * n);
+  free(E->e->alias_list);
+  E->e->alias_list = _GD_Malloc(D, sizeof(const char *) * len);
 
   if (D->error) {
     dreturn("%p", NULL);
@@ -578,11 +593,22 @@ const char **gd_aliases(DIRFILE *D, const char *field_code) gd_nothrow
     if (D->entry[u]->field_type == GD_ALIAS_ENTRY &&
         D->entry[u]->e->entry[0] == E)
     {
-      E->e->alias_list[j++] = D->entry[u]->field;
+      if (n == len - 1) {
+        const char **ptr = _GD_Realloc(D, E->e->alias_list,
+            sizeof(const char*) * (len *= 2));
+        if (ptr == NULL) {
+          free(E->e->alias_list);
+          E->e->alias_list = NULL;
+          dreturn("%p", NULL);
+          return NULL;
+        }
+        E->e->alias_list = ptr;
+      }
+      E->e->alias_list[n++] = D->entry[u]->field;
     }
 
   /* terminate */
-  E->e->alias_list[j] = NULL;
+  E->e->alias_list[n] = NULL;
 
   dreturn("%p", E->e->alias_list);
   return E->e->alias_list;
@@ -595,16 +621,11 @@ unsigned int gd_naliases(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%u", 0);
-    return 0;
-  }
+  GD_RETURN_IF_INVALID(D, "%u", 0);
 
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+  E = _GD_FindEntry(D, field_code);
 
   if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%u", 0);
     return 0;
   }
@@ -626,13 +647,7 @@ gd_entype_t gd_entry_type(DIRFILE* D, const char* field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", GD_NO_ENTRY);
-    return GD_NO_ENTRY;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_IF_INVALID(D, "%i", GD_NO_ENTRY);
 
   E = _GD_FindEntry(D, field_code);
 
@@ -651,21 +666,14 @@ int gd_fragment_index(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  _GD_ClearError(D);
+  E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+      0, NULL);
 
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
-
-  if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (E == NULL)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0,
+        field_code);
 
   dreturn("%i", E->fragment_index);
   return E->fragment_index;
@@ -677,40 +685,26 @@ int gd_hide(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if ((D->flags & GD_ACCMODE) != GD_RDWR) {
+  GD_RETURN_ERR_IF_INVALID(D);
+
+  if ((D->flags & GD_ACCMODE) != GD_RDWR)
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
+  else {
+    E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+        0, NULL);
+
+    if (E == NULL)
+      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    else if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT)
+      _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+          D->fragment[E->fragment_index].cname);
+    else if (!(E->flags & GD_EN_HIDDEN)) {
+      E->flags |= GD_EN_HIDDEN;
+      D->fragment[E->fragment_index].modified = 1;
+    }
   }
 
-  _GD_ClearError(D);
-
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
-
-  if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT) {
-    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
-        D->fragment[E->fragment_index].cname);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (!(E->flags & GD_EN_HIDDEN)) {
-    E->flags |= GD_EN_HIDDEN;
-    D->fragment[E->fragment_index].modified = 1;
-  }
-
-  dreturn("%i", 0);
-  return 0;
+  GD_RETURN_ERROR(D);
 }
 
 int gd_hidden(DIRFILE *D, const char *field_code) gd_nothrow
@@ -719,21 +713,14 @@ int gd_hidden(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  _GD_ClearError(D);
+  E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+      0, NULL);
 
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
-
-  if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (E == NULL)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0,
+        field_code);
 
   dreturn("%i", (E->flags & GD_EN_HIDDEN) ? 1 : 0);
   return (E->flags & GD_EN_HIDDEN) ? 1 : 0;
@@ -745,83 +732,123 @@ int gd_unhide(DIRFILE *D, const char *field_code) gd_nothrow
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if ((D->flags & GD_ACCMODE) != GD_RDWR) {
+  GD_RETURN_ERR_IF_INVALID(D);
+
+  if ((D->flags & GD_ACCMODE) != GD_RDWR)
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
+  else {
+    E = _GD_FindField(D, field_code, strlen(field_code), D->entry, D->n_entries,
+        0, NULL);
+
+    if (E == NULL)
+      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
+    else if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT)
+      _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+          D->fragment[E->fragment_index].cname);
+    else if (E->flags & GD_EN_HIDDEN) {
+      E->flags &= ~GD_EN_HIDDEN;
+      D->fragment[E->fragment_index].modified = 1;
+    }
   }
 
-  _GD_ClearError(D);
+  GD_RETURN_ERROR(D);
+}
 
-  E = _GD_FindField(D, field_code, D->entry, D->n_entries, 0, NULL);
+/* This function silently drops a representation suffix from field_code. */
+int gd_validate(DIRFILE *D, const char *field_code) gd_nothrow
+{
+  int repr;
+  gd_entry_t* E;
 
-  if (E == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-    dreturn("%i", -1);
-    return -1;
+  dtrace("%p, \"%s\"", D, field_code);
+
+  GD_RETURN_ERR_IF_INVALID(D);
+
+  E = _GD_FindFieldAndRepr(D, field_code, &repr, NULL, 1);
+
+  if (D->error)
+    GD_RETURN_ERROR(D);
+
+  /* calculate scalars */
+  if (!(E->flags & GD_EN_CALC))
+    _GD_CalculateEntry(D, E, 1);
+
+  if (D->error)
+    GD_RETURN_ERROR(D);
+
+  /* check input fields */
+  _GD_FindInputs(D, E, 1);
+
+  GD_RETURN_ERROR(D);
+}
+
+/* Ensure that an input field has been identified (with error checking) */
+static int _GD_BadInput(DIRFILE *D, const gd_entry_t *E, int i, gd_entype_t t,
+    int err)
+{
+  dtrace("%p, %p, %i, 0x%X, %i", D, E, i, t, err);
+
+  if (E->e->entry[i]) { /* Job done */
+    dreturn("%i", 0);
+    return 0;
   }
 
-  if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT) {
-    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
-        D->fragment[E->fragment_index].cname);
-    dreturn("%i", -1);
-    return -1;
+  if (E->e->entry[i] == NULL) {
+    E->e->entry[i] = _GD_FindFieldAndRepr(D, E->in_fields[i], &E->e->repr[i],
+        NULL, err);
+
+    if (E->e->entry[i] == NULL) {
+      dreturn("%i", 1);
+      return 1;
+    }
   }
 
-  if (E->flags & GD_EN_HIDDEN) {
-    E->flags &= ~GD_EN_HIDDEN;
-    D->fragment[E->fragment_index].modified = 1;
+  /* check field type */
+  if (t == GD_NO_ENTRY) {
+    /* scalar entries not allowed */
+    if (E->e->entry[i]->field_type & GD_SCALAR_ENTRY_BIT) {
+      _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_FORMAT, E->field, 0,
+          E->e->entry[i]->field);
+      E->e->entry[i] = NULL;
+      dreturn("%i", 1);
+      return 1;
+    }
+  } else if (E->e->entry[i]->field_type != t) {
+    _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_FORMAT, E->field, 0,
+        E->e->entry[i]->field);
+    E->e->entry[i] = NULL;
+    dreturn("%i", 1);
+    return 1;
   }
 
   dreturn("%i", 0);
   return 0;
 }
 
-int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
+int _GD_FindInputs(DIRFILE *D, gd_entry_t *E, int err) gd_nothrow
 {
-  int i, repr;
-  gd_entry_t* E;
-  char *field_code;
+  int i;
 
-  dtrace("%p, \"%s\"", D, field_code_in);
+  dtrace("%p, %p, %i", D, E, err);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
-
-  E = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1, 1);
-
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (field_code != field_code_in)
-    free(field_code);
-
-  /* calculate scalars */
-  if (!(E->flags & GD_EN_CALC))
-    _GD_CalculateEntry(D, E, 1);
-
-  /* check input fields */
   switch (E->field_type) {
     case GD_LINCOM_ENTRY:
       for (i = 0; i < E->EN(lincom,n_fields); ++i)
-        _GD_BadInput(D, E, i, GD_NO_ENTRY, 1);
+        _GD_BadInput(D, E, i, GD_NO_ENTRY, err);
+      break;
+    case GD_INDIR_ENTRY:
+      _GD_BadInput(D, E, 0, GD_NO_ENTRY, err);
+      _GD_BadInput(D, E, 1, GD_CARRAY_ENTRY, err);
+      break;
+    case GD_SINDIR_ENTRY:
+      _GD_BadInput(D, E, 0, GD_NO_ENTRY, err);
+      _GD_BadInput(D, E, 1, GD_SARRAY_ENTRY, err);
       break;
     case GD_DIVIDE_ENTRY:
     case GD_MULTIPLY_ENTRY:
     case GD_WINDOW_ENTRY:
     case GD_MPLEX_ENTRY:
-      _GD_BadInput(D, E, 1, GD_NO_ENTRY, 1);
+      _GD_BadInput(D, E, 1, GD_NO_ENTRY, err);
       /* fallthrough */
     case GD_LINTERP_ENTRY:
     case GD_BIT_ENTRY:
@@ -829,11 +856,12 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
     case GD_POLYNOM_ENTRY:
     case GD_SBIT_ENTRY:
     case GD_RECIP_ENTRY:
-      _GD_BadInput(D, E, 0, GD_NO_ENTRY, 1);
+      _GD_BadInput(D, E, 0, GD_NO_ENTRY, err);
       /* Fallthrough */
     case GD_RAW_ENTRY:
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_STRING_ENTRY:
     case GD_INDEX_ENTRY:
     case GD_NO_ENTRY:
@@ -841,29 +869,17 @@ int gd_validate(DIRFILE *D, const char *field_code_in) gd_nothrow
       break;
   }
 
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  dreturn("%i", 0);
-  return 0;
+  GD_RETURN_ERROR(D);
 }
 
 char *gd_linterp_tablename(DIRFILE *D, const char *field_code) gd_nothrow
 {
   gd_entry_t *E;
-  char *table;
+  char *table0, *table;
 
   dtrace("%p, \"%s\"", D, field_code);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_IF_INVALID(D, "%p", NULL);
 
   E = _GD_FindEntry(D, field_code);
 
@@ -885,9 +901,23 @@ char *gd_linterp_tablename(DIRFILE *D, const char *field_code) gd_nothrow
       return NULL;
     }
 
-  table = _GD_MakeFullPath(D, E->e->u.linterp.table_dirfd,
+  table0 = _GD_MakeFullPath(D, E->e->u.linterp.table_dirfd,
       E->e->u.linterp.table_file, 1);
 
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  /* move to caller's heap */
+  if (_GD_CMalloc != malloc) {
+    table = _GD_CStrdup(table0);
+    free(table0);
+    if (table == NULL)
+      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+  } else
+    table = table0;
+
   dreturn("%s", table);
   return table;
 }
