@@ -44,14 +44,16 @@ struct gd_flacdata {
   FILE* stream;
   unsigned bps; /* bits per sample */
   unsigned cps; /* channels per sample */
+  int swap; /* Byte-swapping required */
 
   int stream_end;
   int error; /* error flag */
   int *errnum; /* file->error */
 
+  /* Decoder (reader) only */
   char *data; /* Dechannelised frame of data */
   unsigned dlen; /* Length in samples of data */
-  unsigned pos; /* offset into the frame data */
+  unsigned pos; /* offset in samples into the frame data */
   off64_t base; /* sample number of the start of the frame */
 };
 
@@ -59,7 +61,7 @@ struct gd_flacdata {
  * open, idata = 0 otherwise idata = -1. */
 
 /* The decoder callback */
-static FLAC__StreamDecoderWriteStatus _GD_FlacWriteCallback(
+static FLAC__StreamDecoderWriteStatus _GD_FlacDecodeCallback(
     const FLAC__StreamDecoder *decoder gd_unused_, const FLAC__Frame *frame,
     const FLAC__int32 *const buffer[], void *client_data)
 {
@@ -79,14 +81,14 @@ static FLAC__StreamDecoderWriteStatus _GD_FlacWriteCallback(
   if (gdfl->cps != FLAC__stream_decoder_get_channels(gdfl->codec.d)) {
     gdfl->error = 1;
     *gdfl->errnum = GD_FLAC_E_CPS;
-    dreturn("%s", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
+    dreturn("%s (cps)", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
     return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
   }
 
   if (gdfl->bps != FLAC__stream_decoder_get_bits_per_sample(gdfl->codec.d)) {
     gdfl->error = 1;
     *gdfl->errnum = GD_FLAC_E_BPS;
-    dreturn("%s", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
+    dreturn("%s (bps)", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
     return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
   }
 
@@ -97,7 +99,7 @@ static FLAC__StreamDecoderWriteStatus _GD_FlacWriteCallback(
     if (gdfl->data == NULL) {
       gdfl->error = 1;
       *gdfl->errnum = GD_FLAC_E_MEM;
-      dreturn("%s", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
+      dreturn("%s (alloc)", "FLAC__STREAM_DECODER_WRITE_STATUS_ABORT");
       return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
     }
     gdfl->dlen = frame->header.blocksize;
@@ -110,6 +112,10 @@ static FLAC__StreamDecoderWriteStatus _GD_FlacWriteCallback(
     for (u = 0; u < frame->header.blocksize; ++u)
       /* there's only one channel in this case */
       gdfl->data[u] = (int8_t)buffer[0][u];
+  else if (gdfl->swap)
+    for (u = 0; u < frame->header.blocksize; ++u)
+      for (c = gdfl->cps; c > 0; --c)
+        *(ptr++) = (int16_t)buffer[c - 1][u];
   else
     for (u = 0; u < frame->header.blocksize; ++u)
       for (c = 0; c < gdfl->cps; ++c)
@@ -135,14 +141,14 @@ static void _GD_FlacErrorCallback(const FLAC__StreamDecoder *decoder gd_unused_,
 }
 
 static struct gd_flacdata *_GD_FlacDoOpen(int dirfd, struct gd_raw_file_* file,
-    gd_type_t data_type, unsigned int mode)
+    gd_type_t data_type, int swap, unsigned int mode)
 {
   int fd, status;
   struct gd_flacdata *gdfl;
   FILE *stream;
   const char *fdmode = "rb";
 
-  dtrace("%i, %p, 0x%X, 0x%X", dirfd, file, data_type, mode);
+  dtrace("%i, %p, 0x%X, %i, 0x%X", dirfd, file, data_type, swap, mode);
 
   if (mode & GD_FILE_READ) {
     fd = gd_OpenAt(file->D, dirfd, file->name, O_RDONLY | O_BINARY, 0666);
@@ -184,9 +190,11 @@ static struct gd_flacdata *_GD_FlacDoOpen(int dirfd, struct gd_raw_file_* file,
   if (GD_SIZE(data_type) <= 2) {
     gdfl->bps = GD_SIZE(data_type) * 8;
     gdfl->cps = 1;
+    gdfl->swap = 0;
   } else {
     gdfl->bps = 16;
     gdfl->cps = GD_SIZE(data_type) / 2;
+    gdfl->swap = swap;
   }
 
   if (mode & GD_FILE_READ) {
@@ -196,7 +204,7 @@ static struct gd_flacdata *_GD_FlacDoOpen(int dirfd, struct gd_raw_file_* file,
     }
 
     status = FLAC__stream_decoder_init_FILE(gdfl->codec.d, stream,
-          _GD_FlacWriteCallback, NULL, _GD_FlacErrorCallback, gdfl);
+          _GD_FlacDecodeCallback, NULL, _GD_FlacErrorCallback, gdfl);
 
     if (status != FLAC__STREAM_DECODER_INIT_STATUS_OK) {
       file->error = GD_FLAC_E_SDIS | status;
@@ -240,11 +248,11 @@ OPEN_ERROR:
 }
 
 int _GD_FlacOpen(int dirfd, struct gd_raw_file_* file, gd_type_t data_type,
-    int swap gd_unused_, unsigned int mode)
+    int swap, unsigned int mode)
 {
-  dtrace("%i, %p, 0x%X, <unused>, 0x%X", dirfd, file, data_type, mode);
+  dtrace("%i, %p, 0x%X, %i, 0x%X", dirfd, file, data_type, swap, mode);
 
-  file->edata = _GD_FlacDoOpen(dirfd, file, data_type, mode);
+  file->edata = _GD_FlacDoOpen(dirfd, file, data_type, swap, mode);
 
   if (file->edata == NULL) {
     dreturn("%i", 1);
@@ -268,7 +276,8 @@ static size_t _GD_FlacOutput(struct gd_flacdata *gdfl, gd_type_t data_type,
     ns = gdfl->dlen - gdfl->pos;
 
   if (ns > 0) {
-    memcpy(output, gdfl->data + gdfl->pos, ns * GD_SIZE(data_type));
+    memcpy(output, gdfl->data + gdfl->pos * GD_SIZE(data_type),
+        ns * GD_SIZE(data_type));
     gdfl->pos += ns;
   }
 
@@ -346,6 +355,10 @@ ssize_t _GD_FlacWrite(struct gd_raw_file_ *file, const void *data,
     if (gdfl->bps == 8)
       for (i = 0; i < nmemb; ++i)
         buffer[0][i] = *(i8++);
+    else if (gdfl->swap)
+      for (i = 0; i < nmemb; ++i)
+        for (c = gdfl->cps; c > 0; --c)
+          buffer[c - 1][i] = *(i16++);
     else
       for (i = 0; i < nmemb; ++i)
         for (c = 0; c < gdfl->cps; ++c)
@@ -378,22 +391,22 @@ WRITE_ERROR:
   return n;
 }
 
-off64_t _GD_FlacSeek(struct gd_raw_file_* file, off64_t count,
+off64_t _GD_FlacSeek(struct gd_raw_file_* file, off64_t offset,
     gd_type_t data_type, unsigned int mode)
 {
   struct gd_flacdata *gdfl;
 
-  dtrace("%p, %" PRId64 ", 0x%X, 0x%X", file, (int64_t)count, data_type, mode);
+  dtrace("%p, %" PRId64 ", 0x%X, 0x%X", file, (int64_t)offset, data_type, mode);
 
-  gdfl = (struct gd_flacdata *)(file[(mode == GD_FILE_WRITE) ? 1 : 0].edata);
-
-  /* nothing to do */
-  if (gdfl->base + gdfl->pos == count) {
-    dreturn("%" PRId64, (int64_t)count);
-    return count;
-  }
+  gdfl = (struct gd_flacdata *)(file->edata);
 
   if (mode == GD_FILE_WRITE) {
+    /* nothing to do */
+    if (file->pos == offset) {
+      dreturn("%" PRId64, (int64_t)offset);
+      return offset;
+    }
+
     /* we only get here when we need to pad */
     char *zero = malloc(GD_BUFFER_SIZE);
     if (zero == NULL) {
@@ -403,22 +416,27 @@ off64_t _GD_FlacSeek(struct gd_raw_file_* file, off64_t count,
     }
     memset(zero, 0, GD_BUFFER_SIZE);
 
-    count -= file->pos;
-    while (gdfl->base < count) {
-      int n = GD_BUFFER_SIZE / GD_SIZE(data_type);;
-      if (n > count)
-        n = count;
+    while (file->pos < offset) {
+      off64_t remaining = offset - file->pos;
+      int n = GD_BUFFER_SIZE / GD_SIZE(data_type);
+      if (n > remaining)
+        n = remaining;
 
-      _GD_FlacWrite(file + 1, zero, GD_UINT8, n);
+      _GD_FlacWrite(file, zero, GD_UINT8, n);
       if (file->error) {
         free(zero);
         dreturn("%i", -1);
         return -1;
       }
-      count -= n;
     }
     free(zero);
   } else {
+    /* nothing to do */
+    if (gdfl->base + gdfl->pos == offset) {
+      dreturn("%" PRId64, (int64_t)offset);
+      return offset;
+    }
+
     int eof = 0;
     off64_t last = FLAC__stream_decoder_get_total_samples(gdfl->codec.d) - 1;
 
@@ -428,12 +446,12 @@ off64_t _GD_FlacSeek(struct gd_raw_file_* file, off64_t count,
       return 0;
     }
 
-    if (count > last) {
-      count = last;
+    if (offset > last) {
+      offset = last;
       eof = 1;
     }
 
-    if (!FLAC__stream_decoder_seek_absolute(gdfl->codec.d, count)) {
+    if (!FLAC__stream_decoder_seek_absolute(gdfl->codec.d, offset)) {
       file->error = GD_FLAC_E_SDS |
         FLAC__stream_decoder_get_state(gdfl->codec.d);
       dreturn("%i", -1);
@@ -450,11 +468,11 @@ off64_t _GD_FlacSeek(struct gd_raw_file_* file, off64_t count,
       gdfl->pos = gdfl->dlen;
 
       gdfl->stream_end = 1;
-      count++;
+      offset++;
     }
   }
   
-  file->pos = count;
+  file->pos = offset;
 
   dreturn("%" PRId64, (int64_t)file->pos);
   return file->pos;
@@ -482,14 +500,14 @@ int _GD_FlacClose(struct gd_raw_file_ *file)
 }
 
 off64_t _GD_FlacSize(int dirfd, struct gd_raw_file_ *file, gd_type_t data_type,
-    int swap gd_unused_)
+    int swap)
 {
   struct gd_flacdata *gdfl;
   off_t n;
 
-  dtrace("%i, %p, 0x%X, <unused>", dirfd, file, data_type);
+  dtrace("%i, %p, 0x%X, %i", dirfd, file, data_type, swap);
 
-  gdfl = _GD_FlacDoOpen(dirfd, file, data_type, GD_FILE_READ);
+  gdfl = _GD_FlacDoOpen(dirfd, file, data_type, swap, GD_FILE_READ);
 
   if (gdfl == NULL) {
     dreturn("%i", -1);
