@@ -22,12 +22,32 @@
 
 #define GD_MAX_PRETTY_FIELD_WIDTH 80
 
+/* If we're writing to 'file', call the appropriate sync function */
+static void _GD_SyncRaw(DIRFILE *D, struct gd_raw_file_ *file)
+{
+  dtrace("%p, %p", D, file);
+  
+  if ((D->flags & GD_ACCMODE) == GD_RDWR && file->idata >= 0 && 
+      (file->mode & GD_FILE_WRITE) && _GD_ef[file->subenc].sync != NULL &&
+      (*_GD_ef[file->subenc].sync)(file))
+  {
+    _GD_SetEncIOError(D, GD_E_IO_WRITE, file);
+  }
+
+  dreturnvoid();
+}
+
 void _GD_Flush(DIRFILE *D, gd_entry_t *E, int syn, int clo)
 {
   int i;
 
   dtrace("%p, %p, %i, %i", D, E, syn, clo);
 
+  if (E == NULL) {
+    dreturnvoid();
+    return;
+  }
+
   if (++D->recurse_level >= GD_MAX_RECURSE_LEVEL) {
     _GD_SetError(D, GD_E_RECURSE_LEVEL, GD_E_RECURSE_CODE, NULL, 0, E->field);
     D->recurse_level--;
@@ -35,37 +55,36 @@ void _GD_Flush(DIRFILE *D, gd_entry_t *E, int syn, int clo)
     return;
   }
 
+  _GD_FindInputs(D, E, 0);
+
   switch(E->field_type) {
     case GD_RAW_ENTRY:
-      if (E->e->u.raw.file[0].idata >= 0 ||
-          ((_GD_ef[E->e->u.raw.file[0].subenc].flags & GD_EF_OOP) &&
-           (E->e->u.raw.file[1].idata >= 0)))
+      if (syn) {
+        /* We can't get here when a temporary file is open, so if we're dealing
+         * with an GD_EF_OOP encoding and file[1] is open, then it's a
+         * write-side file which we should sync.
+         */
+        if (_GD_ef[E->e->u.raw.file[0].subenc].flags & GD_EF_OOP)
+          _GD_SyncRaw(D, E->e->u.raw.file + 1);
+        else
+          _GD_SyncRaw(D, E->e->u.raw.file);
+      }
+      if (clo && D->error == 0 && (E->e->u.raw.file[0].idata >= 0 ||
+            ((_GD_ef[E->e->u.raw.file[0].subenc].flags & GD_EF_OOP) &&
+             (E->e->u.raw.file[1].idata >= 0))))
       {
-        if (syn && (D->flags & GD_ACCMODE) == GD_RDWR &&
-            (E->e->u.raw.file[0].mode & GD_FILE_WRITE) &&
-            _GD_ef[E->e->u.raw.file[0].subenc].sync != NULL &&
-            (*_GD_ef[E->e->u.raw.file[0].subenc].sync)(E->e->u.raw.file))
-        {
-          _GD_SetEncIOError(D, GD_E_IO_WRITE, E->e->u.raw.file + 0);
-        } else if (clo && _GD_FiniRawIO(D, E, E->fragment_index,
-              GD_FINIRAW_KEEP))
-        {
-          ; /* error already set */
-        }
+        _GD_FiniRawIO(D, E, E->fragment_index, GD_FINIRAW_KEEP);
       }
       break;
     case GD_LINCOM_ENTRY:
-      for (i = 0; i < E->EN(lincom,n_fields); ++i) {
-        if (!_GD_BadInput(D, E, i, GD_NO_ENTRY, 0))
-            _GD_Flush(D, E->e->entry[i], syn, clo);
-      }
+      for (i = 0; i < E->EN(lincom,n_fields); ++i)
+        _GD_Flush(D, E->e->entry[i], syn, clo);
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
     case GD_WINDOW_ENTRY:
     case GD_MPLEX_ENTRY:
-      if (!_GD_BadInput(D, E, 1, GD_NO_ENTRY, 0))
-        _GD_Flush(D, E->e->entry[1], syn, clo);
+      _GD_Flush(D, E->e->entry[1], syn, clo);
       /* fallthrough */
     case GD_LINTERP_ENTRY:
     case GD_BIT_ENTRY:
@@ -73,10 +92,13 @@ void _GD_Flush(DIRFILE *D, gd_entry_t *E, int syn, int clo)
     case GD_POLYNOM_ENTRY:
     case GD_SBIT_ENTRY:
     case GD_RECIP_ENTRY:
-      if (!_GD_BadInput(D, E, 0, GD_NO_ENTRY, 0))
-        _GD_Flush(D, E->e->entry[0], syn, clo);
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
+      _GD_Flush(D, E->e->entry[0], syn, clo);
+      break;
     case GD_CONST_ENTRY:
     case GD_CARRAY_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_STRING_ENTRY:
     case GD_INDEX_ENTRY:
     case GD_NO_ENTRY:
@@ -290,30 +312,31 @@ WRITE_ERR:
 
 /* write a field code, taking care of stripping off affixes; returns the length
  * written */
-#define GD_WFC_SPACE      0x1
-#define GD_WFC_SCALAR     0x2
+#define GD_WFC_SPACE     0x01 /* Write a trailing space */
+#define GD_WFC_SCALAR    0x02 /* This is a scalar field code */
+#define GD_WFC_EARLY     0x04 /* This is an early field (no namespaces!) */
+#define GD_WFC_NAME      0x08 /* This is a field name (no repr) */
 static ssize_t _GD_WriteFieldCode(DIRFILE *D, FILE *stream, int me,
     const char *code, int index, int permissive, int standards, unsigned flags)
 {
   ssize_t len;
   char *ptr;
-
-  const int scalar     = flags & GD_WFC_SCALAR;
-  const int space      = flags & GD_WFC_SPACE;
+  const unsigned strip_flags = GD_CO_NSROOT | GD_CO_ASSERT
+    | ((flags & GD_WFC_EARLY) ? GD_CO_EARLY : 0)
+    | ((flags & GD_WFC_NAME) ? 0 : GD_CO_REPR)
+    | ((permissive || D->standards >= 10) ? GD_CO_REPRZ : 0);
 
   dtrace("%p, %p, %i, \"%s\", %i, %i, %i, 0x%X", D, stream, me, code, index,
       permissive, standards, flags);
 
-  ptr = _GD_MungeCode(D, NULL, 0, D->fragment[me].prefix,
-      D->fragment[me].suffix, NULL, NULL, code, NULL, NULL, GD_MC_RQ_PARTS);
+  ptr = _GD_StripCode(D, me, code, strip_flags);
 
   len = _GD_StringEscapeise(stream, ptr, 0, permissive, standards);
 
   /* If a scalar field code could be interpreted as a number, we must force
    * interpretation as a field code by appending a <0> scalar index, which is
-   * valid for both CARRAY and CONST fields.
-   */
-  if (len > 0 && scalar && index == -1)
+   * valid for both CARRAY and CONST fields.  */
+  if (len > 0 && (flags & GD_WFC_SCALAR) && index == -1)
     if (_GD_TokToNum(ptr, standards, !permissive, NULL, NULL, NULL, NULL) != -1)
     {
       if (permissive || standards >= 8) {
@@ -326,7 +349,7 @@ static ssize_t _GD_WriteFieldCode(DIRFILE *D, FILE *stream, int me,
     }
 
   /* append a space */
-  if (space && len > 0) {
+  if ((flags & GD_WFC_SPACE) && len > 0) {
     if (fputc(' ', stream) == EOF)
       len = -1;
     else
@@ -339,16 +362,17 @@ static ssize_t _GD_WriteFieldCode(DIRFILE *D, FILE *stream, int me,
   return len;
 }
 
-/* write a field, padding to the specified length */
+/* write a field name, padding to the specified length */
 static ssize_t _GD_PadField(DIRFILE *D, FILE *stream, int me, const char *in,
-    ssize_t len, int permissive, int standards)
+    ssize_t len, int early, int permissive, int standards)
 {
   ssize_t i;
 
-  dtrace("%p, %p, %i, \"%s\", %" PRIuSIZE ", %i, %i", D, stream, me, in, len,
-      permissive, standards);
+  dtrace("%p, %p, %i, \"%s\", %" PRIdSIZE ", %i, %i, %i", D, stream, me, in,
+      len, early, permissive, standards);
 
-  i = _GD_WriteFieldCode(D, stream, me, in, 0, permissive, standards, 0);
+  i = _GD_WriteFieldCode(D, stream, me, in, 0, permissive, standards,
+      GD_WFC_NAME | early);
 
   if (i >= 0)
     for (; i < len; ++i)
@@ -363,17 +387,17 @@ static ssize_t _GD_PadField(DIRFILE *D, FILE *stream, int me, const char *in,
 
 /* Write a litteral parameter or CONST name or CARRAY element */
 static int _GD_WriteConst(DIRFILE *D, FILE *stream, int me, int permissive,
-    int type, const void* value, const char *scalar, int index,
+    int type, const void* value, const char *scalar, int index, int early,
     const char *postamble)
 {
   int e;
 
-  dtrace("%p, %p, %i, %i, 0x%X, %p, \"%s\", %i, \"%s\"", D, stream, me,
-      permissive, type, value, scalar, index, postamble);
+  dtrace("%p, %p, %i, %i, 0x%X, %p, \"%s\", %i, %i, \"%s\"", D, stream, me,
+      permissive, type, value, scalar, index, early, postamble);
 
   if (scalar != NULL) {
     if (_GD_WriteFieldCode(D, stream, me, scalar, index, permissive,
-          D->standards, GD_WFC_SCALAR) < 0)
+          D->standards, GD_WFC_SCALAR | (early ? GD_WFC_EARLY : 0)) < 0)
     {
       dreturn("%i", -1);
       return -1;
@@ -410,6 +434,14 @@ static int _GD_WriteConst(DIRFILE *D, FILE *stream, int me, int permissive,
   return e;
 }
 
+#define GD_WRITE_INFIELD(n,f) \
+  (_GD_WriteFieldCode(D, stream, me, E->in_fields[n], 0, permissive, \
+                      D->standards, f | early) < 0)
+
+#define GD_WRITE_CONST(t,l,n,p) \
+  (_GD_WriteConst(D, stream, me, permissive, t, &(l), E->scalar[n], \
+                  E->scalar_ind[n], early, p) < 0)
+
 /* Write a field specification line */
 static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
     int me, int meta, size_t max_len, int pretty, int permissive)
@@ -419,6 +451,7 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
   ssize_t len, pos;
   char buffer[1000];
   size_t z;
+  const unsigned early = (E->flags & GD_EN_EARLY) ? GD_WFC_EARLY : 0;
 
   dtrace("%p, %p, %p, %i, %i, %" PRIuSIZE ", %i, %i", D, stream, E, me, meta,
       max_len, pretty, permissive);
@@ -434,42 +467,55 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
     if (fputs("/ALIAS ", stream) == EOF)
       goto WRITE_ERR;
     if (_GD_WriteFieldCode(D, stream, me, E->field, 0, permissive, D->standards,
-          GD_WFC_SPACE) < 0)
+          GD_WFC_SPACE | GD_WFC_NAME | early) < 0)
     {
       goto WRITE_ERR;
     }
-    if (_GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-          D->standards, 0) < 0)
-    {
+
+    if (GD_WRITE_INFIELD(0, 0))
       goto WRITE_ERR;
-    }
+
     if (fputc('\n', stream) == EOF)
       goto WRITE_ERR;
   } else {
-    ptr = E->field;
-
-    /* From Standards Version 7 and on, just use Barth-style */
+    /* For DSV 6, use the "META" directive.  Later verisons can just use
+     * the "parent/subfield" style of metafield definition (AKA "Barth-style")
+     */
     if (meta && D->standards < 7) {
+      /* Write "META <parent> " for DSV == 6 */
       if (fputs("META ", stream) == EOF)
         goto WRITE_ERR;
-      if (_GD_StringEscapeise(stream, ptr, 1, permissive, D->standards) < 0)
+
+      /* There's no need to use WriteFieldCode here because DSV < 7 doesn't
+       * support affixes */
+      if (_GD_StringEscapeise(stream, E->field, 1, permissive,
+            D->standards) < 0)
+      {
         goto WRITE_ERR;
+      }
+
       if (fputc(' ', stream) == EOF)
         goto WRITE_ERR;
-      ptr = strchr(E->field, '/') + 1;
-    }
 
-    /* field name */
-    if (_GD_PadField(D, stream, me, ptr, max_len, permissive, D->standards) < 0)
+      /* Advance field name pointer */
+      ptr = strchr(E->field, '/') + 1;
+    } else
+      ptr = E->field;
+
+    /* field name -- with DSV >=7, for metafields, this will be parent/subfield
+     */
+    if (_GD_PadField(D, stream, me, ptr, max_len, early, permissive,
+          D->standards) < 0)
+    {
       goto WRITE_ERR;
+    }
 
     switch(E->field_type) {
       case GD_RAW_ENTRY:
         if (fprintf(stream, " RAW%s %s ", pretty ? "     " : "", (permissive ||
                 D->standards >= 5) ? _GD_TypeName(D, E->EN(raw,data_type)) :
               _GD_OldTypeName(D, E->EN(raw,data_type))) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_UINT_TYPE,
-              &E->EN(raw,spf), E->scalar[0], E->scalar_ind[0], "") < 0)
+            GD_WRITE_CONST(GD_UINT_TYPE, E->EN(raw,spf), 0, ""))
         {
           goto WRITE_ERR;
         }
@@ -481,29 +527,20 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
           goto WRITE_ERR;
         }
         for (i = 0; i < E->EN(lincom,n_fields); ++i) {
-          if (_GD_WriteFieldCode(D, stream, me, E->in_fields[i], 0, permissive,
-                D->standards, GD_WFC_SPACE) < 0)
-          {
+          if (GD_WRITE_INFIELD(i, GD_WFC_SPACE))
             goto WRITE_ERR;
-          }
 
           if (E->flags & GD_EN_COMPSCAL) {
-            if (_GD_WriteConst(D, stream, me, permissive, GD_COMPLEX128,
-                  &E->EN(lincom,cm)[i], E->scalar[i], E->scalar_ind[i], " ") < 0
-                || _GD_WriteConst(D, stream, me, permissive, GD_COMPLEX128,
-                  &E->EN(lincom,cb)[i], E->scalar[i + GD_MAX_LINCOM],
-                  E->scalar_ind[i + GD_MAX_LINCOM],
-                  i == E->EN(lincom,n_fields) - 1 ? "" : " ") < 0)
+            if (GD_WRITE_CONST(GD_COMPLEX128, E->EN(lincom,cm)[i], i, " ") ||
+                GD_WRITE_CONST(GD_COMPLEX128, E->EN(lincom,cb)[i], i +
+                  GD_MAX_LINCOM, i == E->EN(lincom,n_fields) - 1 ? "" : " "))
             {
               goto WRITE_ERR;
             }
           } else {
-            if (_GD_WriteConst(D, stream, me, permissive, GD_FLOAT64,
-                  &E->EN(lincom,m)[i], E->scalar[i], E->scalar_ind[i], " ") < 0
-                || _GD_WriteConst(D, stream, me, permissive, GD_FLOAT64,
-                  &E->EN(lincom,b)[i], E->scalar[i + GD_MAX_LINCOM],
-                  E->scalar_ind[i + GD_MAX_LINCOM],
-                  i == E->EN(lincom,n_fields) - 1 ? "" : " ") < 0)
+            if (GD_WRITE_CONST(GD_FLOAT64, E->EN(lincom,m)[i], i, " ") ||
+                GD_WRITE_CONST(GD_FLOAT64, E->EN(lincom,b)[i], i +
+                  GD_MAX_LINCOM, i == E->EN(lincom,n_fields) - 1 ? "" : " "))
             {
               goto WRITE_ERR;
             }
@@ -512,8 +549,7 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
         break;
       case GD_LINTERP_ENTRY:
         if (fprintf(stream, " LINTERP%s ", pretty ? " " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
             _GD_StringEscapeise(stream, E->EN(linterp,table), 0, permissive,
               D->standards) < 0)
         {
@@ -522,75 +558,59 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
         break;
       case GD_BIT_ENTRY:
         if (fprintf(stream, " BIT%s ", pretty ? "     " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(bit,bitnum), E->scalar[0], E->scalar_ind[0], " ") < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(bit,numbits), E->scalar[1], E->scalar_ind[1], "") < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(bit,bitnum), 0, " ") ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(bit,numbits), 1, ""))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_DIVIDE_ENTRY:
         if (fprintf(stream, " DIVIDE%s ", pretty ? "  " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[1], 0, permissive,
-              D->standards, 0) < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) || GD_WRITE_INFIELD(1, 0))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_RECIP_ENTRY:
         if (fprintf(stream, " RECIP%s ", pretty ? "   " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_COMPLEX128,
-              &E->EN(recip,cdividend), E->scalar[0], E->scalar_ind[0], "") < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_CONST(GD_COMPLEX128, E->EN(recip,cdividend), 0, ""))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_MULTIPLY_ENTRY:
         if (fputs(" MULTIPLY ", stream) == EOF ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[1], 0, permissive,
-              D->standards, 0) < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) || GD_WRITE_INFIELD(1, 0))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_PHASE_ENTRY:
         if (fprintf(stream, " PHASE%s ", pretty ? "   " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT64,
-              &E->EN(phase,shift), E->scalar[0], E->scalar_ind[0], "") < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_CONST(GD_INT64, E->EN(phase,shift), 0, ""))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_POLYNOM_ENTRY:
         if (fprintf(stream, " POLYNOM%s ", pretty ? " " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE))
         {
           goto WRITE_ERR;
         }
         for (i = 0; i <= E->EN(polynom,poly_ord); ++i)
           if (E->flags & GD_EN_COMPSCAL) {
-            if (_GD_WriteConst(D, stream, me, permissive, GD_COMPLEX128,
-                  &E->EN(polynom,ca)[i], E->scalar[i], E->scalar_ind[i],
-                  (i == E->EN(polynom,poly_ord)) ?  "" : " ") < 0)
+            if (GD_WRITE_CONST(GD_COMPLEX128, E->EN(polynom,ca)[i], i,
+                  (i == E->EN(polynom,poly_ord)) ?  "" : " "))
             {
               goto WRITE_ERR;
             }
           } else {
-            if (_GD_WriteConst(D, stream, me, permissive, GD_FLOAT64,
-                  &E->EN(polynom,a)[i], E->scalar[i], E->scalar_ind[i],
-                  (i == E->EN(polynom,poly_ord)) ?  "" : " ") < 0)
+            if (GD_WRITE_CONST(GD_FLOAT64, E->EN(polynom,a)[i], i,
+                  (i == E->EN(polynom,poly_ord)) ?  "" : " "))
             {
               goto WRITE_ERR;
             }
@@ -598,22 +618,17 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
         break;
       case GD_SBIT_ENTRY:
         if (fprintf(stream, " SBIT%s ", pretty ? "    " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(bit,bitnum), E->scalar[0], E->scalar_ind[0], " ") < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(bit,numbits), E->scalar[1], E->scalar_ind[1], "") < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(bit,bitnum), 0, " ") ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(bit,numbits), 1, ""))
         {
           goto WRITE_ERR;
         }
         break;
       case GD_WINDOW_ENTRY:
         if (fprintf(stream, " WINDOW%s ", pretty ? "  " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[1], 0, permissive,
-              D->standards, 0) < 0 ||
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_INFIELD(1, GD_WFC_SPACE) ||
             fprintf(stream, " %s ", _GD_WindopName(D, E->EN(window,windop)))
             < 0)
         {
@@ -622,42 +637,26 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
         switch (E->EN(window,windop)) {
           case GD_WINDOP_EQ:
           case GD_WINDOP_NE:
-            if (_GD_WriteConst(D, stream, me, permissive, GD_INT64,
-                  &E->EN(window,threshold.i), E->scalar[0], E->scalar_ind[0],
-                  "") < 0)
-            {
+            if (GD_WRITE_CONST(GD_INT64, E->EN(window,threshold.i), 0, ""))
               goto WRITE_ERR;
-            }
             break;
           case GD_WINDOP_SET:
           case GD_WINDOP_CLR:
-            if (_GD_WriteConst(D, stream, me, permissive, GD_UINT64,
-                  &E->EN(window,threshold.u), E->scalar[0], E->scalar_ind[0],
-                  "") < 0)
-            {
+            if (GD_WRITE_CONST(GD_UINT64, E->EN(window,threshold.u), 0, ""))
               goto WRITE_ERR;
-            }
             break;
           default:
-            if (_GD_WriteConst(D, stream, me, permissive, GD_FLOAT64,
-                  &E->EN(window,threshold.r), E->scalar[0], E->scalar_ind[0],
-                  "") < 0)
-            {
+            if (GD_WRITE_CONST(GD_FLOAT64, E->EN(window,threshold.r), 0, ""))
               goto WRITE_ERR;
-            }
             break;
         }
         break;
       case GD_MPLEX_ENTRY:
         if (fprintf(stream, " MPLEX%s ", pretty ? "   " : "") < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[0], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteFieldCode(D, stream, me, E->in_fields[1], 0, permissive,
-              D->standards, GD_WFC_SPACE) < 0 ||
-            _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(mplex,count_val), E->scalar[0], E->scalar_ind[0], " ") < 0
-            || _GD_WriteConst(D, stream, me, permissive, GD_INT_TYPE,
-              &E->EN(mplex,period), E->scalar[1], E->scalar_ind[1], "") < 0)
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) ||
+            GD_WRITE_INFIELD(1, GD_WFC_SPACE) ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(mplex,count_val), 0, " ") ||
+            GD_WRITE_CONST(GD_INT_TYPE, E->EN(mplex,period), 1, ""))
         {
           goto WRITE_ERR;
         }
@@ -724,6 +723,50 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
           goto WRITE_ERR;
         }
         break;
+      case GD_SARRAY_ENTRY:
+        pos = fprintf(stream, " SARRAY%s", pretty ? "  " : "");
+        if (pos < 0)
+          goto WRITE_ERR;
+
+        for (z = 0; z < E->EN(scalar,array_len); ++z) {
+          if (fputc(' ', stream) == EOF)
+            goto WRITE_ERR;
+          pos++;
+
+          /* compute length */
+          len = _GD_StringEscapeise(NULL, ((char**)E->e->u.scalar.d)[z], 0,
+              permissive, D->standards);
+
+          /* don't write lines that are too long
+           * also, add one to length for the trailing '\n' */
+          if (GD_SSIZE_T_MAX - (len + 1) <= pos) {
+            _GD_SetError(D, GD_E_LINE_TOO_LONG, GD_E_LONG_FLUSH, E->field, 0,
+                NULL);
+            dreturn("%i", -1);
+            return -1;
+          }
+
+          if (_GD_StringEscapeise(stream, ((char**)E->e->u.scalar.d)[z], 0,
+                permissive, D->standards) < 0)
+          {
+            goto WRITE_ERR;
+          }
+        }
+        break;
+      case GD_INDIR_ENTRY:
+        if (fprintf(stream, " INDIR%s ", pretty ? "   " : "") < 0 ||
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) || GD_WRITE_INFIELD(1, 0))
+        {
+          goto WRITE_ERR;
+        }
+        break;
+      case GD_SINDIR_ENTRY:
+        if (fprintf(stream, " SINDIR%s ", pretty ? "  " : "") < 0 ||
+            GD_WRITE_INFIELD(0, GD_WFC_SPACE) || GD_WRITE_INFIELD(1, 0))
+        {
+          goto WRITE_ERR;
+        }
+        break;
       case GD_INDEX_ENTRY:
       case GD_ALIAS_ENTRY:
       case GD_NO_ENTRY:
@@ -742,7 +785,7 @@ static int _GD_FieldSpec(DIRFILE* D, FILE* stream, const gd_entry_t* E,
     if (fputs("/HIDDEN ", stream) == EOF)
       goto WRITE_ERR;
     if (_GD_WriteFieldCode(D, stream, me, E->field, 0, permissive, D->standards,
-          0) < 0)
+          GD_WFC_NAME | early) < 0)
     {
       goto WRITE_ERR;
     }
@@ -760,6 +803,88 @@ WRITE_ERR:
   return -1;
 }
 
+static int WriteInclude(DIRFILE *D, int i, int j, size_t ns_offset,
+    int permissive, FILE *stream)
+{
+  const char *ns = NULL, *px = NULL;
+  char *sx = NULL;
+  int free_sx = 0;
+
+  dtrace("%p, %i, %i, %" PRIuSIZE ", %i, %p", D, i, j, ns_offset,
+      permissive, stream);
+
+  if (D->fragment[j].nsl > D->fragment[i].nsl)
+    ns = D->fragment[j].ns + ns_offset;
+
+  if (D->fragment[j].pxl)
+    px = D->fragment[j].px + D->fragment[i].pxl;
+
+  if (D->fragment[j].sxl > 0) {
+    if (D->fragment[i].sxl == 0)
+      sx = D->fragment[j].sx;
+    else if (D->fragment[j].sxl == D->fragment[i].sxl)
+      sx = NULL;
+    else {
+      free_sx = 1;
+      sx = _GD_Strdup(D, D->fragment[j].sx);
+      if (sx)
+        sx[D->fragment[j].sxl - D->fragment[i].sxl] = 0;
+      else
+        goto WRITE_ERR;
+    }
+  }
+
+  if (fprintf(stream, "%sINCLUDE ", (D->standards >= 5) ? "/" : "") < 0 ||
+      _GD_StringEscapeise(stream, D->fragment[j].ename, 0, permissive,
+        D->standards) < 0)
+  {
+    goto WRITE_ERR;
+  }
+
+  if (ns || px || sx)
+    if (fputc(' ', stream) == EOF)
+      goto WRITE_ERR;
+
+  if (ns) {
+    if (_GD_StringEscapeise(stream, ns, 0, permissive, D->standards) < 0)
+      goto WRITE_ERR;
+    if (fputc('.', stream) == EOF)
+      goto WRITE_ERR;
+  }
+
+  /* An empty prefix must be written if there's a suffix, but no namespace */
+  if (px || (sx && !ns)) {
+    if (fputc(' ', stream) == EOF || _GD_StringEscapeise(stream, px, 0,
+          permissive, D->standards) < 0)
+    {
+      goto WRITE_ERR;
+    }
+  }
+
+  if (sx) {
+    if (fputc(' ', stream) == EOF || _GD_StringEscapeise(stream, sx, 0,
+          permissive, D->standards) < 0)
+    {
+      goto WRITE_ERR;
+    }
+  }
+
+  if (fputc('\n', stream) == EOF)
+    goto WRITE_ERR;
+
+  if (free_sx)
+    free(sx);
+
+  dreturn("%i", 0);
+  return 0;
+
+WRITE_ERR:
+  if (free_sx)
+    free(sx);
+  dreturn("%i", 1);
+  return 1;
+}
+
 static void _GD_FlushFragment(DIRFILE* D, int i, int permissive)
 {
   int j;
@@ -967,40 +1092,16 @@ static void _GD_FlushFragment(DIRFILE* D, int i, int permissive)
   }
 
   /* The includes */
-  if (permissive || D->standards >= 3)
+  if (permissive || D->standards >= 3) {
+    size_t ns_offset = D->fragment[i].nsl;
+    if (ns_offset)
+      ns_offset++; /* for the dot */
+
     for (j = 0; j < D->n_fragment; ++j)
-      if (D->fragment[j].parent == i) {
-        char *prefix = _GD_MungeCode(D, NULL, 0, D->fragment[i].prefix, NULL,
-            NULL, NULL, D->fragment[j].prefix, NULL, NULL, GD_MC_RQ_PARTS);
-        char *suffix = _GD_MungeCode(D, NULL, 0, NULL, D->fragment[i].suffix,
-            NULL, NULL, D->fragment[j].suffix, NULL, NULL, GD_MC_RQ_PARTS);
-
-        if (fprintf(stream, "%sINCLUDE ", (D->standards >= 5) ? "/" : "") < 0 ||
-            _GD_StringEscapeise(stream, D->fragment[j].ename, 0, permissive,
-              D->standards) < 0)
-        {
+      if (D->fragment[j].parent == i)
+        if (WriteInclude(D, i, j, ns_offset, permissive, stream))
           goto WRITE_ERR;
-        }
-        if (prefix || suffix) {
-          if (fputc(' ', stream) == EOF || _GD_StringEscapeise(stream, prefix,
-                0, permissive, D->standards) < 0)
-          {
-            goto WRITE_ERR;
-          }
-          free(prefix);
-        }
-
-        if (suffix) {
-          if (fputc(' ', stream) == EOF || _GD_StringEscapeise(stream, suffix,
-                0, permissive, D->standards) < 0)
-          {
-            goto WRITE_ERR;
-          }
-          free(suffix);
-        }
-        if (fputc('\n', stream) == EOF)
-          goto WRITE_ERR;
-      }
+  }
 
   /* The fields */
   for (u = 0; u < D->n_entries; ++u)
@@ -1020,12 +1121,13 @@ static void _GD_FlushFragment(DIRFILE* D, int i, int permissive)
     }
 
   /* REFERENCE is written at the end, because its effect can propagate
-   * upwards */
+   * upwards.  In the WriteFieldCode call, early is always zero because the
+   * REFERENCE directive appeared in DSV 6 */
   if (permissive || D->standards >= 6)
     if (D->fragment[i].ref_name != NULL) {
       if (fputs("/REFERENCE ", stream) == EOF ||
           _GD_WriteFieldCode(D, stream, i, D->fragment[i].ref_name, 0,
-            permissive, D->standards, 0) < 0 ||
+            permissive, D->standards, GD_WFC_NAME) < 0 ||
           fputc('\n', stream) == EOF)
       {
         goto WRITE_ERR;
@@ -1089,54 +1191,30 @@ int gd_metaflush(DIRFILE* D)
 {
   dtrace("%p", D);
 
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if ((D->flags & GD_ACCMODE) == GD_RDONLY) {
-    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  if ((D->flags & GD_ACCMODE) == GD_RDONLY)
+    GD_SET_RETURN_ERROR(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
 
   _GD_FlushMeta(D, GD_ALL_FRAGMENTS, 0);
 
-  dreturn("%i", (D->error == GD_E_OK) ? 0 : -1);
-  return (D->error == GD_E_OK) ? 0 : -1;
+  GD_RETURN_ERROR(D);
 }
 
 int gd_rewrite_fragment(DIRFILE* D, int fragment)
 {
   dtrace("%p, %i", D, fragment);
 
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", 1);
-    return -1;
-  }
-
-  if (fragment < GD_ALL_FRAGMENTS || fragment >= D->n_fragment) {
+  if (fragment < GD_ALL_FRAGMENTS || fragment >= D->n_fragment)
     _GD_SetError(D, GD_E_BAD_INDEX, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if ((D->flags & GD_ACCMODE) == GD_RDONLY) {
+  else if ((D->flags & GD_ACCMODE) == GD_RDONLY)
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  else
+    _GD_FlushMeta(D, fragment, 1);
 
-  _GD_FlushMeta(D, fragment, 1);
-
-  dreturn("%i", (D->error == GD_E_OK) ? 0 : -1);
-  return (D->error == GD_E_OK) ? 0 : -1;
+  GD_RETURN_ERROR(D);
 }
 
 static int _GD_SyncOrClose(DIRFILE* D, const char* field_code, int syn, int clo)
@@ -1146,11 +1224,9 @@ static int _GD_SyncOrClose(DIRFILE* D, const char* field_code, int syn, int clo)
 
   dtrace("%p, \"%s\", %i, %i", D, field_code, syn, clo);
 
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if (D->flags & GD_INVALID) /* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-  else if (field_code == NULL) {
+  if (field_code == NULL) {
     _GD_FlushMeta(D, GD_ALL_FRAGMENTS, 0);
     if (!D->error)
       for (i = 0; i < D->n_entries; ++i)
@@ -1163,8 +1239,7 @@ static int _GD_SyncOrClose(DIRFILE* D, const char* field_code, int syn, int clo)
       _GD_Flush(D, E, syn, clo);
   }
 
-  dreturn("%i", (D->error == GD_E_OK) ? 0 : -1);
-  return (D->error == GD_E_OK) ? 0 : -1;
+  GD_RETURN_ERROR(D);
 }
 
 int gd_sync(DIRFILE *D, const char *field_code)
@@ -1203,26 +1278,28 @@ int gd_flush(DIRFILE *D, const char *field_code)
   return ret;
 }
 
-#define GD_VERS_GE_1  0xFFFFFFFEUL
-#define GD_VERS_GE_2  0xFFFFFFFCUL
-#define GD_VERS_GE_3  0xFFFFFFF8UL
-#define GD_VERS_GE_4  0xFFFFFFF0UL
-#define GD_VERS_GE_5  0xFFFFFFE0UL
-#define GD_VERS_GE_6  0xFFFFFFC0UL
-#define GD_VERS_GE_7  0xFFFFFF80UL
-#define GD_VERS_GE_8  0xFFFFFF00UL
-#define GD_VERS_GE_9  0xFFFFFE00UL
+#define GD_VERS_GE_1   0xFFFFFFFEUL
+#define GD_VERS_GE_2   0xFFFFFFFCUL
+#define GD_VERS_GE_3   0xFFFFFFF8UL
+#define GD_VERS_GE_4   0xFFFFFFF0UL
+#define GD_VERS_GE_5   0xFFFFFFE0UL
+#define GD_VERS_GE_6   0xFFFFFFC0UL
+#define GD_VERS_GE_7   0xFFFFFF80UL
+#define GD_VERS_GE_8   0xFFFFFF00UL
+#define GD_VERS_GE_9   0xFFFFFE00UL
+#define GD_VERS_GE_10  0xFFFFFC00UL
 
-#define GD_VERS_LE_0  0x00000001UL
-#define GD_VERS_LE_1  0x00000003UL
-#define GD_VERS_LE_2  0x00000007UL
-#define GD_VERS_LE_3  0x0000000fUL
-#define GD_VERS_LE_4  0x0000001fUL
-#define GD_VERS_LE_5  0x0000003fUL
-#define GD_VERS_LE_6  0x0000007fUL
-#define GD_VERS_LE_7  0x000000ffUL
-#define GD_VERS_LE_8  0x000001ffUL
-#define GD_VERS_LE_9  0x000003ffUL
+#define GD_VERS_LE_0   0x00000001UL
+#define GD_VERS_LE_1   0x00000003UL
+#define GD_VERS_LE_2   0x00000007UL
+#define GD_VERS_LE_3   0x0000000fUL
+#define GD_VERS_LE_4   0x0000001fUL
+#define GD_VERS_LE_5   0x0000003fUL
+#define GD_VERS_LE_6   0x0000007fUL
+#define GD_VERS_LE_7   0x000000ffUL
+#define GD_VERS_LE_8   0x000001ffUL
+#define GD_VERS_LE_9   0x000003ffUL
+#define GD_VERS_LE_10  0x000007ffUL
 
 uint64_t _GD_FindVersion(DIRFILE *D)
 {
@@ -1236,7 +1313,7 @@ uint64_t _GD_FindVersion(DIRFILE *D)
     D->av &= GD_VERS_GE_3;
 
   for (i = 0; D->av && i < (unsigned int)D->n_fragment; ++i) {
-    if (D->fragment[i].prefix || D->fragment[i].suffix)
+    if (D->fragment[i].px || D->fragment[i].sx)
       D->av &= GD_VERS_GE_9;
     else if (D->fragment[i].byte_sex & GD_ARM_FLAG)
       /* on an arm-endian platform, the arm flag is set by /ENDIAN directives
@@ -1308,6 +1385,11 @@ uint64_t _GD_FindVersion(DIRFILE *D)
         case GD_STRING_ENTRY:
           D->av &= GD_VERS_GE_6;
           break;
+        case GD_SARRAY_ENTRY:
+        case GD_INDIR_ENTRY:
+        case GD_SINDIR_ENTRY:
+          D->av &= GD_VERS_GE_10;
+          break;
         case GD_BIT_ENTRY:
           if (D->entry[i]->EN(bit,numbits) > 1)
             D->av &= GD_VERS_GE_1;
@@ -1354,8 +1436,10 @@ uint64_t _GD_FindVersion(DIRFILE *D)
           D->av &= GD_VERS_GE_6;
           break;
         case '.':
-          if (D->entry[i]->flags & GD_EN_DOTTED)
+          if (D->entry[i]->flags & GD_EN_EARLY)
             D->av &= GD_VERS_LE_5;
+          else
+            D->av &= GD_VERS_GE_10;
           break;
         case '&':
         case ';':
@@ -1406,13 +1490,7 @@ int gd_dirfile_standards(DIRFILE *D, int vers) gd_nothrow
 
   dtrace("%p, %i", D, vers);
 
-  _GD_ClearError(D);
-
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   if (~D->flags & GD_HAVE_VERSION)
     _GD_FindVersion(D);
@@ -1431,10 +1509,8 @@ int gd_dirfile_standards(DIRFILE *D, int vers) gd_nothrow
   if (vers < 0 || vers > GD_DIRFILE_STANDARDS_VERSION ||
       ~D->av & (1ULL << vers))
   {
-    _GD_SetError(D, GD_E_ARGUMENT, (D->av == 0) ? GD_E_ARG_NO_VERS :
+    GD_SET_RETURN_ERROR(D, GD_E_ARGUMENT, (D->av == 0) ? GD_E_ARG_NO_VERS :
         GD_E_ARG_BAD_VERS, NULL, vers, NULL);
-    dreturn("%i", -1);
-    return -1;
   }
 
   D->standards = vers;
