@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2017 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -49,10 +49,8 @@
     case GD_FLOAT64:    EXTRACT_REPRR2(        double, ot,fabs,fargs); break; \
     case GD_COMPLEX64:  EXTRACT_REPRC2(         float, ot); break; \
     case GD_COMPLEX128: EXTRACT_REPRC2(        double, ot); break; \
-    case GD_NULL: \
-      break; \
-    default: \
-      _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, in_type, NULL); \
+    case GD_NULL:                                           break; \
+    default: _GD_InternalError(D); \
   }
 
 #define EXTRACT_REPRC2(it,ot) \
@@ -108,16 +106,14 @@ static void _GD_ExtractRepr(DIRFILE *restrict D, const void *restrict cdata,
     } \
   } while(0)
 
-    case GD_COMPLEX64:  EXTRACT_REPRS(         float); break;
-    case GD_COMPLEX128: EXTRACT_REPRS(        double); break;
+    case GD_COMPLEX64:  EXTRACT_REPRS(         float);  break;
+    case GD_COMPLEX128: EXTRACT_REPRS(        double);  break;
 #else
-    case GD_COMPLEX64:  EXTRACT_REPRS( float complex); break;
-    case GD_COMPLEX128: EXTRACT_REPRS(double complex); break;
+    case GD_COMPLEX64:  EXTRACT_REPRS( float _Complex); break;
+    case GD_COMPLEX128: EXTRACT_REPRS(double _Complex); break;
 #endif
-    case GD_NULL:                                      break;
-    default:
-      _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-      break;
+    case GD_NULL:                                       break;
+    default:            _GD_InternalError(D);           break;
   }
 
   dreturnvoid();
@@ -232,11 +228,10 @@ static size_t _GD_DoRaw(DIRFILE *restrict D, gd_entry_t *restrict E, off64_t s0,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p)", D, E, (int64_t)s0, ns,
       return_type, data_out);
 
-  if (s0 < E->EN(raw,spf) * D->fragment[E->fragment_index].frame_offset)
-    zero_pad = E->EN(raw,spf) * D->fragment[E->fragment_index].frame_offset -
-      s0;
-  else
-    s0 -= E->EN(raw,spf) * D->fragment[E->fragment_index].frame_offset;
+  if (ns * E->e->u.raw.size == 0) {
+    dreturn("%i", 0);
+    return 0;
+  }
 
   databuffer = _GD_Malloc(D, ns * E->e->u.raw.size);
   if (databuffer == NULL) {
@@ -244,45 +239,29 @@ static size_t _GD_DoRaw(DIRFILE *restrict D, gd_entry_t *restrict E, off64_t s0,
     return 0;
   }
 
-  if (zero_pad > 0) {
-    /* frame offset in samples */
-    const off64_t foffs = E->EN(raw,spf) *
-      D->fragment[E->fragment_index].frame_offset;
+  if (s0 < E->EN(raw,spf) * D->fragment[E->fragment_index].frame_offset)
+    zero_pad = E->EN(raw,spf) * D->fragment[E->fragment_index].frame_offset -
+      s0;
 
+  /* Generate padding before frameoffset */
+  if (zero_pad > 0) {
     zeroed_samples = _GD_FillZero(databuffer, E->EN(raw,data_type),
         (zero_pad > ns) ? ns : zero_pad);
     ns -= zeroed_samples;
-
-     /* Padding up to the end of the frameoffset, results in a "real" file
-      * position.  In this case we need to make sure the underlying file is
-      * actually at the BOF, for consistency.
-      */
-    if (s0 + (off64_t)zeroed_samples == foffs) /* ie. file->pos is zero */
-      _GD_Seek(D, E, foffs, GD_SEEK_SET);
-    else 
-      E->e->u.raw.file[0].pos = s0 + zeroed_samples - foffs;
-    s0 = 0;
+    s0 += zeroed_samples;
   }
 
+  /* We need to seek if we zero padded to get the file->pos in the right place
+   */
+  if (ns > 0 || zero_pad)
+    /* This will open the file if it's not open already */
+    if (_GD_Seek(D, E, s0, GD_FILE_READ)) {
+      free(databuffer);
+      dreturn("%i", 0);
+      return 0;
+    }
+
   if (ns > 0) {
-    /* open the file (and cache the fp) if it hasn't been opened yet. */
-    if (_GD_InitRawIO(D, E, NULL, -1, NULL, GD_EF_SEEK | GD_EF_READ,
-          GD_FILE_READ, _GD_FileSwapBytes(D, E)))
-    {
-      free(databuffer);
-      dreturn("%i", 0);
-      return 0;
-    }
-
-    if ((*_GD_ef[E->e->u.raw.file[0].subenc].seek)(E->e->u.raw.file, s0,
-          E->EN(raw,data_type), GD_FILE_READ) == -1)
-    {
-      _GD_SetEncIOError(D, GD_E_IO_READ, E->e->u.raw.file + 0);
-      free(databuffer);
-      dreturn("%i", 0);
-      return 0;
-    }
-
     samples_read = (*_GD_ef[E->e->u.raw.file[0].subenc].read)(E->e->u.raw.file,
           databuffer + zeroed_samples * E->e->u.raw.size, E->EN(raw,data_type),
           ns);
@@ -432,7 +411,7 @@ static size_t _GD_DoRaw(DIRFILE *restrict D, gd_entry_t *restrict E, off64_t s0,
     } \
   } while (0)
 #else
-#define POLYNOMC(t) POLYNOM(complex t)
+#define POLYNOMC(t) POLYNOM(_Complex t)
 #endif
 
 #define POLYNOM(t) \
@@ -472,8 +451,7 @@ static void _GD_PolynomData(DIRFILE *restrict D, void *restrict data,
       case GD_FLOAT64:    POLYNOM(  double); break;
       case GD_COMPLEX64:  POLYNOMC(  float); break;
       case GD_COMPLEX128: POLYNOMC( double); break;
-      default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                          break;
+      default:         _GD_InternalError(D); break;
     }
   }
 
@@ -607,20 +585,9 @@ static void _GD_CPolynomData(DIRFILE *restrict D, void *restrict data,
   } else {
     switch (type) {
       case GD_NULL:                          break;
-      case GD_INT8:
-      case GD_UINT8:
-      case GD_INT16:
-      case GD_UINT16:
-      case GD_INT32:
-      case GD_UINT32:
-      case GD_INT64:
-      case GD_UINT64:
-      case GD_FLOAT32:
-      case GD_FLOAT64: _GD_InternalError(D); break;
       case GD_COMPLEX64:  POLYNOMC(  float); break;
       case GD_COMPLEX128: POLYNOMC( double); break;
-      default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                          break;
+      default:         _GD_InternalError(D); break;
     }
   }
 
@@ -636,7 +603,7 @@ static void _GD_CPolynomData(DIRFILE *restrict D, void *restrict data,
     } \
   } while (0)
 #else
-#define MULTIPLYC(t) MULTIPLY(complex t)
+#define MULTIPLYC(t) MULTIPLY(_Complex t)
 #endif
 
 #define MULTIPLY(t) \
@@ -666,8 +633,7 @@ static void _GD_MultiplyData(DIRFILE *restrict D, void *restrict A,
     case GD_FLOAT64:    MULTIPLY(  double); break;
     case GD_COMPLEX64:  MULTIPLYC(  float); break;
     case GD_COMPLEX128: MULTIPLYC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:          _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -702,20 +668,9 @@ static void _GD_CMultiplyData(DIRFILE *restrict D, void *restrict A,
 
   switch (type) {
     case GD_NULL:                           break;
-    case GD_UINT8:
-    case GD_INT8:
-    case GD_UINT16:
-    case GD_INT16:
-    case GD_UINT32:
-    case GD_INT32:
-    case GD_UINT64:
-    case GD_INT64:
-    case GD_FLOAT32:
-    case GD_FLOAT64: _GD_InternalError(D);  break;
     case GD_COMPLEX64:  MULTIPLYC(  float); break;
     case GD_COMPLEX128: MULTIPLYC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:          _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -730,7 +685,7 @@ static void _GD_CMultiplyData(DIRFILE *restrict D, void *restrict A,
     } \
   } while(0)
 #else
-#define DIVIDEC(t) DIVIDE(complex t)
+#define DIVIDEC(t) DIVIDE(_Complex t)
 #endif
 
 #define DIVIDE(t) \
@@ -761,8 +716,7 @@ static void _GD_DivideData(DIRFILE *restrict D, void *restrict A,
     case GD_FLOAT64:    DIVIDE(  double); break;
     case GD_COMPLEX64:  DIVIDEC(  float); break;
     case GD_COMPLEX128: DIVIDEC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:        _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -798,20 +752,9 @@ static void _GD_CDivideData(DIRFILE *restrict D, void *restrict A,
 
   switch (type) {
     case GD_NULL:                         break;
-    case GD_UINT8:
-    case GD_INT8:
-    case GD_UINT16:
-    case GD_INT16:
-    case GD_UINT32:
-    case GD_INT32:
-    case GD_UINT64:
-    case GD_INT64:
-    case GD_FLOAT32:
-    case GD_FLOAT64: _GD_InternalError(D); break;
     case GD_COMPLEX64:  DIVIDEC(  float); break;
     case GD_COMPLEX128: DIVIDEC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:        _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -882,8 +825,7 @@ static void _GD_WindowData(DIRFILE *restrict D, void *restrict A,
     case GD_FLOAT64:    WINDOW(  double,NaN) break;
     case GD_COMPLEX64:  WINDOWC(  float,NaN) break;
     case GD_COMPLEX128: WINDOWC( double,NaN) break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:           _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -939,8 +881,7 @@ static void _GD_MplexData(DIRFILE *restrict D, void *restrict A,
     case GD_FLOAT64:    MPLEX(  double); break;
     case GD_COMPLEX64:  MPLEXC(  float); break;
     case GD_COMPLEX128: MPLEXC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:       _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -963,13 +904,8 @@ static size_t _GD_DoLincom(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* input field checks */
+  /* Get SPF for all inputs */
   for (i = 0; i < E->EN(lincom,n_fields); ++i) {
-    if (_GD_BadInput(D, E, i, GD_NO_ENTRY, 1)) {
-      dreturn("%i", 0);
-      return 0;
-    }
-
     spf[i] = _GD_GetSPF(D, E->e->entry[i]);
     if (D->error != GD_E_OK) {
       dreturn("%i", 0);
@@ -1097,17 +1033,6 @@ static size_t _GD_DoMultiply(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* Check input fields */
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
-  if (_GD_BadInput(D, E, 1, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* find the samples per frame of the first field */
   spf1 = _GD_GetSPF(D, E->e->entry[0]);
   if (D->error != GD_E_OK) {
@@ -1190,12 +1115,6 @@ static size_t _GD_DoRecip(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* Check input fields */
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* read the first field and record the number of samples returned */
   n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp,
       return_type, data_out);
@@ -1236,14 +1155,6 @@ static size_t _GD_DoDivide(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* Check input fields */
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1) ||
-      _GD_BadInput(D, E, 1, GD_NO_ENTRY, 1))
-  {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* read the first field and record the number of samples returned */
   n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp,
       return_type, data_out);
@@ -1334,11 +1245,6 @@ static size_t _GD_DoBit(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %i, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E, is_signed,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (is_signed)
     tmpbuf = _GD_Malloc(D, num_samp * sizeof(int64_t));
   else
@@ -1387,11 +1293,6 @@ static size_t _GD_DoPhase(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp +
       E->EN(phase,shift), num_samp, return_type, data_out);
 
@@ -1411,11 +1312,6 @@ static size_t _GD_DoLinterp(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* allocate a temporary buffer */
   data_in = _GD_Alloc(D, GD_FLOAT64, num_samp);
 
@@ -1453,11 +1349,6 @@ static size_t _GD_DoPolynom(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* read the input field */
   n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp,
       return_type, data_out);
@@ -1499,17 +1390,6 @@ static size_t _GD_DoWindow(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* Check input fields */
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
-  if (_GD_BadInput(D, E, 1, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* find the samples per frame of the input field */
   spf1 = _GD_GetSPF(D, E->e->entry[0]);
   if (D->error != GD_E_OK) {
@@ -1605,17 +1485,6 @@ static size_t _GD_DoMplex(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, return_type, data_out);
 
-  /* Check input fields */
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
-  if (_GD_BadInput(D, E, 1, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* find the samples per frame of the input field */
   spf1 = _GD_GetSPF(D, E->e->entry[0]);
   if (D->error != GD_E_OK) {
@@ -1795,6 +1664,88 @@ static size_t _GD_DoConst(DIRFILE *restrict D, const gd_entry_t *restrict E,
   return len;
 }
 
+/* simple */
+static void _GD_IndirData(char *restrict cbuf, gd_type_t ctype,
+        const int64_t *restrict ibuf, size_t n, const void *carray, size_t len)
+{
+  size_t i;
+  const int size = GD_SIZE(ctype);
+
+  /* INDIR can only address the first 2**63 entries in a CARRAY */
+  int64_t ilen =
+#if SIZEOF_SIZE_T == 8
+    (len > GD_INT64_MAX) ? GD_INT64_MAX :
+#endif
+    len;
+
+  dtrace("%p, 0x%X, %p, %" PRIuSIZE ", %p, %" PRIuSIZE, cbuf, ctype, ibuf, n,
+      carray, len);
+
+  for (i = 0; i < n; ++i)
+    if (ibuf[i] < 0 || ibuf[i] >= ilen)
+      _GD_FillZero(cbuf + size * i, ctype, 1);
+    else
+      memcpy(cbuf + size * i, (const char*)carray + size * ibuf[i], size);
+
+  dreturnvoid();
+}
+
+/* _GD_DoIndir: Read from an indir. */
+static size_t _GD_DoIndir(DIRFILE *restrict D, const gd_entry_t *restrict E,
+    off64_t first_samp, size_t num_samp, gd_type_t return_type,
+    void *restrict data_out)
+{
+  size_t n_read;
+  int64_t *ibuf = NULL;
+  void *cbuf = NULL;
+  gd_type_t ctype;
+
+  dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
+      (int64_t)first_samp, num_samp, return_type, data_out);
+
+  /* index buffer */
+  ibuf = _GD_Alloc(D, GD_INT64, num_samp);
+  if (D->error) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* read the index field and record the number of samples returned */
+  n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp,
+      GD_INT64, ibuf);
+  
+  if (D->error || n_read == 0) {
+    free(ibuf);
+    dreturn("%i", 0);
+    return 0;
+  }
+  
+  /* the intermediate buffer: it has the load type of the carray */
+  ctype = _GD_ConstType(D, E->e->entry[1]->EN(scalar,const_type));
+  cbuf = _GD_Alloc(D, ctype, n_read);
+  if (D->error) {
+    free(ibuf);
+    dreturn("%i", 0);
+    return 0;
+  }
+  
+  _GD_IndirData(cbuf, GD_SIZE(ctype), ibuf, n_read,
+      E->e->entry[1]->e->u.scalar.d, E->e->entry[1]->EN(scalar,array_len));
+  
+  free(ibuf);
+  
+  /* type convert into output buffer */
+  _GD_ConvertType(D, cbuf, ctype, data_out, return_type, n_read);
+  
+  free(cbuf);
+  
+  if (D->error)
+    n_read = 0;
+  
+  dreturn("%" PRIuSIZE, n_read);
+  return n_read;
+}
+
 /* _GD_DoField: Locate the field in the database and read it.
 */
 size_t _GD_DoField(DIRFILE *restrict D, gd_entry_t *restrict E, int repr,
@@ -1928,7 +1879,12 @@ size_t _GD_DoField(DIRFILE *restrict D, gd_entry_t *restrict E, int repr,
     case GD_CARRAY_ENTRY:
       n_read = _GD_DoConst(D, E, first_samp, num_samp, return_type, data_out);
       break;
+    case GD_INDIR_ENTRY:
+      n_read = _GD_DoIndir(D, E, first_samp, num_samp, return_type, data_out);
+      break;
     case GD_STRING_ENTRY:
+    case GD_SARRAY_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_ALIAS_ENTRY:
     case GD_NO_ENTRY:
       /* Can't get here */
@@ -1949,44 +1905,97 @@ size_t _GD_DoField(DIRFILE *restrict D, gd_entry_t *restrict E, int repr,
   return n_read;
 }
 
+/* this returns string vector data; it is called for SINDIRs instead of making
+ * a call to DoField */
+static size_t _GD_DoSindir(DIRFILE *D, gd_entry_t *E, off64_t first_samp,
+    size_t num_samp, gd_type_t return_type, const char **data_out)
+{
+  size_t i, n_read = 0;
+  int64_t *ibuf = NULL;
+  int64_t len;
+
+  dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
+      (int64_t)first_samp, num_samp, return_type, data_out);
+
+  if (_GD_FindInputs(D, E, 1)) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* check return type */
+  if (return_type != GD_STRING && return_type != GD_NULL) {
+    _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, return_type, NULL);
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* short circuit: no data requested */
+  if (num_samp == 0) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* index buffer */
+  ibuf = _GD_Alloc(D, GD_INT64, num_samp);
+  if (D->error) {
+    dreturn("%i", 0);
+    return 0;
+  }
+
+  /* read the index field and record the number of samples returned */
+  n_read = _GD_DoField(D, E->e->entry[0], E->e->repr[0], first_samp, num_samp,
+      GD_INT64, ibuf);
+
+  if (n_read == 0 || return_type == GD_NULL) {
+    free(ibuf);
+    dreturn("%" PRIuSIZE, n_read);
+    return n_read;
+  }
+
+  /* SINDIR can only address the first 2**63 entries in a SARRAY */
+  len =
+#if SIZEOF_SIZE_T == 8
+    (E->e->entry[1]->EN(scalar,array_len) > GD_INT64_MAX) ? GD_INT64_MAX :
+#endif
+    E->e->entry[1]->EN(scalar,array_len);
+
+  for (i = 0; i < n_read; ++i)
+    if (ibuf[i] < 0 || ibuf[i] >= len)
+      data_out[i] = NULL;
+    else
+      data_out[i] = ((const char **)E->e->entry[1]->e->u.scalar.d)[ibuf[i]];
+
+  free(ibuf);
+
+  dreturn("%" PRIuSIZE, n_read);
+  return n_read;
+}
+
 /* this function is little more than a public boilerplate for _GD_DoField */
-size_t gd_getdata64(DIRFILE* D, const char *field_code_in, off64_t first_frame,
+size_t gd_getdata64(DIRFILE* D, const char *field_code, off64_t first_frame,
     off64_t first_samp, size_t num_frames, size_t num_samp,
     gd_type_t return_type, void *data_out)
 {
   size_t n_read = 0;
   gd_entry_t* entry;
-  char* field_code;
   int repr;
   unsigned int spf;
 
   dtrace("%p, \"%s\", %" PRId64 ", %" PRId64 ", %" PRIuSIZE ", %" PRIuSIZE
-      ", 0x%X, %p", D, field_code_in, (int64_t)first_frame, (int64_t)first_samp,
+      ", 0x%X, %p", D, field_code, (int64_t)first_frame, (int64_t)first_samp,
       num_frames, num_samp, return_type, data_out);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", 0);
-    return 0;
-  }
+  GD_RETURN_IF_INVALID(D, "%i", 0);
 
-  _GD_ClearError(D);
-
-  entry = _GD_FindFieldAndRepr(D, field_code_in, &field_code, &repr, NULL, 1,
-      1);
+  entry = _GD_FindFieldAndRepr(D, field_code, &repr, NULL, 1);
 
   if (D->error) {
     dreturn("%i", 0);
     return 0;
   }
 
-  if (entry->field_type & GD_SCALAR_ENTRY_BIT)
+  if (entry->field_type & GD_SCALAR_ENTRY_BIT) {
     _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_CALLER, NULL, 0, field_code);
-
-  if (field_code != field_code_in)
-    free(field_code);
-
-  if (D->error) {
     dreturn("%i", 0);
     return 0;
   }
@@ -1996,7 +2005,7 @@ size_t gd_getdata64(DIRFILE* D, const char *field_code_in, off64_t first_frame,
     first_frame = 0;
   }
 
-  if (first_frame > 0 || num_frames > 0) {
+  if (first_frame || num_frames) {
     /* get the samples per frame */
     spf = _GD_GetSPF(D, entry);
 
@@ -2025,8 +2034,18 @@ size_t gd_getdata64(DIRFILE* D, const char *field_code_in, off64_t first_frame,
     return 0;
   }
 
-  n_read = _GD_DoField(D, entry, repr, first_samp, num_samp, return_type,
-      data_out);
+  if (entry->field_type == GD_SINDIR_ENTRY)
+    n_read = _GD_DoSindir(D, entry, first_samp, num_samp, return_type,
+        data_out);
+  else {
+    if (return_type != GD_NULL &&
+        _GD_BadType(GD_DIRFILE_STANDARDS_VERSION, return_type))
+    {
+      _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, return_type, NULL);
+    } else
+      n_read = _GD_DoField(D, entry, repr, first_samp, num_samp, return_type,
+          data_out);
+  }
 
   dreturn("%" PRIuSIZE, n_read);
   return n_read;
