@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2015 D. V. Wiebe
+/* Copyright (C) 2008-2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -21,63 +21,50 @@
 #include "internal.h"
 
 /* Create new affixes given the current affixes and the new parts indicated
- * on a /INCLUDE line.  Also, change the root namespace, if necessary.  The
- * caller is responsible for freeing strings on error */
+ * on a /INCLUDE line.  If there's a new namespace, clear the parser's current
+ * namespace.  The caller is responsible for zeroing all the output parameters
+ * before the call.
+ *
+ * Returns 1 on error.  In this case, the caller is responsible for free'ing
+ * all the returned buffers.
+ *
+ * If a new rootspace is created, *newns is set to 1 (to let the caller know it
+ * needs to zero p->ns). */
 static int _GD_SetFieldAffixes(DIRFILE *D, const struct parser_state *p, int me,
-    const char *prefix_in, const char *suffix_in, char **prefix, char **suffix)
+    const char *pxin, const char *sxin, char **ns, size_t *nsl, char **px,
+    size_t *pxl, char **sx, size_t *sxl, int *newns)
 {
-  dtrace("%p, %p, %i, \"%s\", \"%s\", %p, %p", D, p, me, prefix_in, suffix_in,
-      prefix, suffix);
+  dtrace("%p, %p, %i, \"%s\", \"%s\", %p, %p, %p, %p, %p, %p, %p", D, p, me,
+      pxin, sxin, ns, nsl, px, pxl, sx, sxl, newns);
 
   /* suffix first, for some reason */
-  if (suffix_in && suffix_in[0] != '\0') {
-    if (_GD_ValidateField(suffix_in, p->standards, p->pedantic, GD_VF_AFFIX,
-          NULL))
-    {
+  if (sxin && sxin[0] != '\0') {
+    if (_GD_ValidateField(sxin, 0, p->standards, p->pedantic, GD_VF_AFFIX)) {
       _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line,
-          suffix_in);
+          sxin);
       dreturn("%i", 1);
       return 1;
     }
 
-    if (D->fragment[me].suffix == NULL)
-      *suffix = _GD_Strdup(D, suffix_in);
-    else {
-      *suffix = _GD_Malloc(D, strlen(D->fragment[me].suffix) + strlen(suffix_in)
-          + 1);
-      if (*suffix)
-        sprintf(*suffix, "%s%s", suffix_in, D->fragment[me].suffix);
+    if (D->fragment[me].sx == NULL) {
+      *sxl = strlen(sxin);
+      *sx = _GD_Malloc(D, *sxl + 1);
+      if (*sx)
+        memcpy(*sx, sxin, *sxl + 1); /* includes terminating NUL */
+    } else {
+      size_t sxinl = strlen(sxin);
+      *sxl = D->fragment[me].sxl + sxinl + 1;
+      *sx = _GD_Malloc(D, *sxl);
+      if (*sx) {
+        memcpy(*sx, sxin, sxinl);
+        memcpy(*sx + sxinl, D->fragment[me].sx, D->fragment[me].sxl + 1);
+      }
     }
-  } else if (D->fragment[me].suffix)
-    *suffix = _GD_Strdup(D, D->fragment[me].suffix);
-
-  if (D->error) {
-    dreturn("%i", 1);
-    return 1;
-  }
-
-  /* now the prefix */
-  if (prefix_in && prefix_in[0] != '\0') {
-    if (_GD_ValidateField(prefix_in, p->standards, p->pedantic, GD_VF_AFFIX,
-          NULL))
-    {
-      _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line,
-          prefix_in);
-      dreturn("%i", 1);
-      return 1;
-    }
-
-    if (D->fragment[me].prefix == NULL)
-      *prefix = _GD_Strdup(D, prefix_in);
-    else {
-      *prefix = _GD_Malloc(D, strlen(D->fragment[me].prefix) + strlen(prefix_in)
-          + 1);
-      if (*prefix)
-        sprintf(*prefix, "%s%s", D->fragment[me].prefix, prefix_in);
-    }
-  } else {
-    if (D->fragment[me].prefix)
-      *prefix = _GD_Strdup(D, D->fragment[me].prefix);
+  } else if (D->fragment[me].sx) {
+    *sxl = D->fragment[me].sxl;
+    *sx = _GD_Malloc(D, *sxl + 1);
+    if (*sx)
+      memcpy(*sx, D->fragment[me].sx, *sxl + 1); /* includes terminating NUL */
   }
 
   if (D->error) {
@@ -85,30 +72,139 @@ static int _GD_SetFieldAffixes(DIRFILE *D, const struct parser_state *p, int me,
     return 1;
   }
 
-  dreturn("%i (\"%s\", \"%s\")", 0, *prefix, *suffix);
+  /* now the namespace */
+  if (GD_PVERS_GE(*p, 10)) {
+    const char *nsin = NULL;
+    size_t nsinl;
+
+    if (pxin && pxin[0] != '\0') {
+      /* try to find a namespace */
+      for (nsinl = strlen(pxin); nsinl > 0; --nsinl)
+        if (pxin[nsinl - 1] == '.')
+          break;
+
+      if (nsinl > 0) { /* found a namespace */
+        /* ignore a leading dot: ns is always relative to root */
+        if (pxin[0] == '.') {
+          pxin++;
+          nsinl--;
+        }
+        nsin = pxin;
+        pxin += nsinl;
+        if (nsinl)
+          nsinl--; /* drop the '.' we found */
+      }
+    }
+
+    if (nsin == NULL && p->ns) {
+      /* If there's a current namespace but no namespace specified in the
+       * /INCLUDE directive, then the current namespace becomes the new
+       * rootspace */
+      nsin = p->ns;
+      nsinl = p->nsl;
+    }
+
+    if (nsin) {
+      /* Signal caller to push and reset the parer's current namespace */
+      *newns = 1;
+
+      if (_GD_ValidateField(nsin, 0, p->standards, p->pedantic, GD_VF_NS)) {
+        _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line,
+            *ns);
+        dreturn("%i", 1);
+        return 1;
+      }
+
+      if (D->fragment[me].ns == NULL) {
+        *nsl = nsinl;
+        *ns = _GD_Malloc(D, nsinl + 1);
+        if (*ns) {
+          memcpy(*ns, nsin, nsinl);
+          (*ns)[nsinl] = 0; /* nsin isn't necessarily terminated */
+        }
+      } else {
+        *nsl = D->fragment[me].nsl + nsinl;
+        if (nsinl)
+          (*nsl)++; /* for the intervening '.' */
+
+        *ns = _GD_Malloc(D, *nsl + 1);
+        if (*ns) {
+          memcpy(*ns, D->fragment[me].ns, D->fragment[me].nsl);
+          if (nsinl) {
+            (*ns)[D->fragment[me].nsl] = '.';
+            memcpy(*ns + D->fragment[me].nsl + 1, nsin, nsinl);
+          }
+          (*ns)[*nsl] = 0;
+        }
+      }
+    } else if (D->fragment[me].ns) {
+      *nsl = D->fragment[me].nsl;
+      *ns = _GD_Malloc(D, *nsl + 1);
+      if (*ns)
+        memcpy(*ns, D->fragment[me].ns, *nsl + 1);
+    }
+  }
+
+  /* Now the prefix */
+  if (pxin && pxin[0] != '\0') {
+    if (_GD_ValidateField(pxin, 0, p->standards, p->pedantic, GD_VF_AFFIX)) {
+      _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line,
+          pxin);
+      dreturn("%i", 1);
+      return 1;
+    }
+
+    if (D->fragment[me].px == NULL) {
+      *pxl = strlen(pxin);
+      *px = _GD_Malloc(D, *pxl + 1);
+      if (*px)
+        memcpy(*px, pxin, *pxl + 1); /* includes terminating NUL */
+    } else {
+      size_t pxinl = strlen(pxin);
+      *pxl = D->fragment[me].pxl + pxinl + 1;
+      *px = _GD_Malloc(D, *pxl);
+      if (*px) {
+        memcpy(*px, D->fragment[me].px, D->fragment[me].pxl);
+        memcpy(*px + D->fragment[me].pxl, pxin, pxinl + 1); /* incl. NUL */
+      }
+    }
+  } else if (D->fragment[me].px) {
+    *pxl = D->fragment[me].pxl;
+    *px = _GD_Malloc(D, *pxl + 1);
+    if (*px)
+      memcpy(*px, D->fragment[me].px, *pxl + 1); /* includes terminating NUL */
+  }
+
+  if (D->error) {
+    dreturn("%i", 1);
+    return 1;
+  }
+
+  dreturn("%i (\"%s\"/%" PRIuSIZE ", \"%s\"/%" PRIuSIZE ", \"%s\"/%" PRIuSIZE 
+      ", %i)", 0, *ns, *nsl, *px, *pxl, *sx, *sxl, *newns);
   return 0;
 }
 
 /* Include a format file fragment -- returns the new fragment index, or
  * -1 on error */
 int _GD_Include(DIRFILE *D, struct parser_state *p, const char *ename,
-    char **ref_name, int parent, const char *prefix_in, const char *suffix_in,
+    char **ref_name, int parent, const char *pxin, const char *sxin,
     int immediate)
 {
-  int i;
+  int i, newns = 0;
   int me = D->n_fragment;
   struct parser_state oldp = *p;
-  int dirfd = -1, pop_ns = 0, free_ns = 1;
+  int dirfd = -1, pop_ns = 0;
   char *temp_buf1 = NULL, *temp_buf2, *sname = NULL;
-  char *base = NULL, *prefix = NULL, *suffix = NULL, *ns = NULL;
-  size_t nsl = 0;
+  char *base = NULL, *px = NULL, *sx = NULL, *ns = NULL;
+  size_t pxl = 0, sxl = 0, nsl = 0;
   void *ptr = NULL;
   FILE* new_fp = NULL;
   time_t mtime = 0;
   struct stat statbuf;
 
   dtrace("%p, %p, \"%s\", %p, %i, \"%s\", \"%s\", %i", D, p, ename, ref_name,
-      parent, prefix_in, suffix_in, immediate);
+      parent, pxin, sxin, immediate);
 
   if (++D->recurse_level >= GD_MAX_RECURSE_LEVEL) {
     _GD_SetError(D, GD_E_RECURSE_LEVEL, GD_E_RECURSE_INCLUDE, p->file, p->line,
@@ -116,30 +212,20 @@ int _GD_Include(DIRFILE *D, struct parser_state *p, const char *ename,
     goto include_error;
   }
 
-  if (_GD_SetFieldAffixes(D, p, parent, prefix_in, suffix_in, &prefix, &suffix))
+  if (_GD_SetFieldAffixes(D, p, parent, pxin, sxin, &ns, &nsl, &px, &pxl, &sx,
+        &sxl, &newns))
   {
     goto include_error;
   }
 
-  if (ns) {
-    /* namespace change, remember old one so we can pop at the end */
+  /* when the rootspace changes, the current namespace gets reset to "", since
+   * it's relative to rootspace.  But, remember old current namespace so we can
+   * pop at the end
+   */
+  if (newns) {
     pop_ns = 1;
-    if (ns[0] == '\0') {
-      /* revert to NULLspace */
-      free(ns);
-      ns = p->ns = NULL;
-    } else {
-      /* new namespace */
-      p->ns = _GD_Strdup(D, ns);
-      if (p->ns == NULL)
-        goto include_error;
-      p->nsl = nsl;
-    }
-  } else {
-    /* no namespace change; inherit from parent */
-    free_ns = 0;
-    ns = D->fragment[parent].ns;
-    nsl = D->fragment[parent].nsl;
+    p->ns = NULL; /* Don't free: we'll need it back when we pop */
+    p->nsl = 0;
   }
 
   /* isolate filename */
@@ -247,8 +333,10 @@ int _GD_Include(DIRFILE *D, struct parser_state *p, const char *ename,
   D->fragment[me].ref_name = NULL;
   D->fragment[me].frame_offset = D->fragment[parent].frame_offset;
   D->fragment[me].protection = GD_PROTECT_NONE;
-  D->fragment[me].prefix = prefix;
-  D->fragment[me].suffix = suffix;
+  D->fragment[me].px = px;
+  D->fragment[me].pxl = px ? strlen(px) : 0;
+  D->fragment[me].sx = sx;
+  D->fragment[me].sxl = sx ? strlen(sx) : 0;
   D->fragment[me].ns = ns;
   D->fragment[me].nsl = nsl;
   D->fragment[me].mtime = mtime;
@@ -276,7 +364,7 @@ int _GD_Include(DIRFILE *D, struct parser_state *p, const char *ename,
   /* catch alloc errors */
   if (D->error) {
     D->n_fragment--;
-    temp_buf1 = prefix = suffix = base = NULL;
+    temp_buf1 = px = sx = base = NULL;
     fclose(new_fp);
     _GD_ReleaseDir(D, dirfd);
     goto include_error;
@@ -312,10 +400,9 @@ int _GD_Include(DIRFILE *D, struct parser_state *p, const char *ename,
   return me;
 
 include_error:
-  if (free_ns)
-    free(ns);
-  free(prefix);
-  free(suffix);
+  free(ns);
+  free(px);
+  free(sx);
   free(base);
   free(sname);
   free(temp_buf1);
@@ -324,48 +411,32 @@ include_error:
   return -1;
 }
 
-int gd_include_affix(DIRFILE* D, const char* file, int fragment_index,
-    const char *prefix, const char *suffix, unsigned long flags)
+static int _GD_IncludeAffix(DIRFILE* D, const char *funcname, const char* file,
+    int fragment_index, const char *px, const char *sx, unsigned long flags)
 {
   char* ref_name = NULL;
   int i, new_fragment;
+  /* Remember the last fragment, in case we have to undo everything */
+  int old_nfrag = D->n_fragment;
 
   struct parser_state p = {
-    "gd_include_affix()", 0, GD_DIRFILE_STANDARDS_VERSION, flags & GD_PEDANTIC,
+    funcname, 0, GD_DIRFILE_STANDARDS_VERSION, flags & GD_PEDANTIC,
     NULL, 0, flags
   };
 
-  dtrace("%p, \"%s\", %i, \"%s\", \"%s\", 0x%lX", D, file, fragment_index,
-      prefix, suffix, flags);
+  dtrace("%p, \"%s\", \"%s\", %i, \"%s\", \"%s\", 0x%lX", D, funcname, file,
+      fragment_index, px, sx, flags); 
 
-  _GD_ClearError(D);
+  /* Early checks */
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-  /* check access mode */
-  if ((D->flags & GD_ACCMODE) == GD_RDONLY) {
-    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  /* check for include index out of range */
-  if (fragment_index < 0 || fragment_index >= D->n_fragment) {
-    _GD_SetError(D, GD_E_BAD_INDEX, 0, NULL, fragment_index, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  /* check protection */
-  if (D->fragment[fragment_index].protection & GD_PROTECT_FORMAT) {
-    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+  if ((D->flags & GD_ACCMODE) == GD_RDONLY) 
+    GD_SET_RETURN_ERROR(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
+  else if (fragment_index < 0 || fragment_index >= D->n_fragment) 
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_INDEX, 0, NULL, fragment_index, NULL);
+  else if (D->fragment[fragment_index].protection & GD_PROTECT_FORMAT)
+    GD_SET_RETURN_ERROR(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
         D->fragment[fragment_index].cname);
-    dreturn("%i", -1);
-    return -1;
-  }
 
   if (~D->flags & GD_HAVE_VERSION)
     _GD_FindVersion(D);
@@ -379,32 +450,13 @@ int gd_include_affix(DIRFILE* D, const char* file, int fragment_index,
   if ((p.flags & (GD_ENCODING | GD_CREAT)) == GD_CREAT)
     p.flags |= D->fragment[fragment_index].encoding;
 
-  new_fragment = _GD_Include(D, &p, file, &ref_name, fragment_index, prefix,
-      suffix, 1);
-
-  if (!D->error) {
-    D->fragment[fragment_index].modified = 1;
-    D->flags &= ~GD_HAVE_VERSION;
-  }
-
-  /* If ref_name is non-NULL, the included fragment contained a REFERENCE
-   * directive.  If ref_name is NULL but D->fragment[new_fragment].ref_name is
-   * non-NULL, no REFERENCE directive was present, but the parser found a RAW
-   * field to serve as a reference field.
-   */
-  if (new_fragment >= 0 && D->fragment[new_fragment].ref_name != NULL)
-    /* If the parser chose a reference field, propagate it upward if requried */
-    for (i = fragment_index; i != -1; i = D->fragment[i].parent) {
-      if (D->fragment[i].ref_name == NULL) {
-        D->fragment[i].ref_name = strdup(D->fragment[new_fragment].ref_name);
-        D->fragment[i].modified = 1;
-      } else
-        break;
-    }
+  /* Perform the include */
+  new_fragment = _GD_Include(D, &p, file, &ref_name, fragment_index, px, sx, 1);
 
   /* Honour the reference directive, if not prohibited by the caller */
-  if (ref_name != NULL && ~flags & GD_IGNORE_REFS) {
-    gd_entry_t *E = _GD_FindField(D, ref_name, D->entry, D->n_entries, 1, NULL);
+  if (!D->error && ref_name != NULL && ~flags & GD_IGNORE_REFS) {
+    gd_entry_t *E = _GD_FindField(D, ref_name, strlen(ref_name), D->entry,
+        D->n_entries, 1, NULL);
 
     /* FIXME: These errors are problematic, since they'll cause the call to
      * fail, even though the new fragment has been integrated into the DIRFILE.
@@ -421,6 +473,99 @@ int gd_include_affix(DIRFILE* D, const char* file, int fragment_index,
   }
   free(ref_name);
 
+  /* If there was an error, we now have to:
+   *
+   * - delete all the entries that were added
+   * - delete all the fragemnts that were added
+   *
+   * This is the same thing that gd_uninclude() does, but it's easier here
+   * because we know none of the fields we delete are open, have been reference
+   * by the old fields, and all the fragment numbers are contiguous. */
+  if (D->error) {
+    unsigned int i, first = 0;
+    unsigned int next = 0;
+    int found_good = 0;
+
+    /* First delete all the new fields */
+    for (i = 0; i < D->n_entries; ++i) {
+      if (D->entry[i]->fragment_index >= old_nfrag) {
+        /* found an entry to delete */
+        _GD_FreeE(D, D->entry[i], 1);
+
+        if (found_good) { /* The last entry was good */
+          /* There is a run of entries spanning [first,i-1] that need to be
+           * moved to [next,next+i-first] */
+          memmove(D->entry + next, D->entry + first,
+              sizeof(D->entry[0]) * (i - first));
+
+          /* advance next to here */
+          next += i - first;
+
+          /* and change modes */
+          found_good = 0;
+        }
+      } else {
+        /* found a good entry */
+
+        if (!found_good) { /* The last entry was bad */
+          first = i; /* This is the start of the next good run */
+          found_good = 1; /* Change modes */
+        }
+      }
+    }
+
+    /* Handle the last bunch of good entries, if necessary */
+    if (found_good) {
+      memmove(D->entry + next, D->entry + first,
+          sizeof(D->entry[0]) * (i - first));
+      next += i - first;
+    }
+    
+    D->n_entries = next; /* Done.  Reset entry count. */
+
+    /* Now delete all the new fragments.  This is easier because we don't
+     * have to search for them.
+     */
+    _GD_FreeF(D, old_nfrag, D->n_fragment);
+    D->n_fragment = old_nfrag;
+
+    GD_RETURN_ERROR(D);
+  }
+
+  /* Successful include.  Mark the parent as dirty */
+  D->fragment[fragment_index].modified = 1;
+  D->flags &= ~GD_HAVE_VERSION;
+
+  /* If ref_name is non-NULL, the included fragment contained a REFERENCE
+   * directive.  If ref_name is NULL but D->fragment[new_fragment].ref_name is
+   * non-NULL, no REFERENCE directive was present, but the parser found a RAW
+   * field to serve as a reference field.
+   */
+  if (new_fragment >= 0 && D->fragment[new_fragment].ref_name != NULL)
+    /* If the parser chose a reference field, propagate it upward if requried */
+    for (i = fragment_index; i != -1; i = D->fragment[i].parent) {
+      if (D->fragment[i].ref_name == NULL) {
+        D->fragment[i].ref_name = strdup(D->fragment[new_fragment].ref_name);
+        D->fragment[i].modified = 1;
+      } else
+        break;
+    }
+
+  dreturn("%i", new_fragment);
+  return new_fragment;
+}
+
+int gd_include_affix(DIRFILE* D, const char* file, int fragment_index,
+    const char *prefix, const char *suffix, unsigned long flags)
+{
+  int new_fragment;
+
+  dtrace("%p, \"%s\", %i, \"%s\", \"%s\", 0x%lX", D, file, fragment_index,
+      prefix, suffix, flags);
+
+  new_fragment = _GD_IncludeAffix(D, "gd_include_affix()", file, fragment_index,
+      prefix, suffix, flags);
+
   dreturn("%i", new_fragment);
   return new_fragment;
 }
@@ -432,40 +577,39 @@ int gd_include(DIRFILE* D, const char* file, int fragment_index,
 
   dtrace("%p, \"%s\", %i, 0x%lX", D, file, fragment_index, flags);
 
-  new_fragment = gd_include_affix(D, file, fragment_index, NULL, NULL, flags);
+  new_fragment = _GD_IncludeAffix(D, "gd_include()", file, fragment_index, NULL,
+      NULL, flags);
 
   dreturn("%i", new_fragment);
   return new_fragment;
 }
 
-static int _GD_CollectFragments(DIRFILE* D, int** f, int fragment, int nf)
+int gd_include_ns(DIRFILE *D, const char *file, int fragment_index,
+    const char *nsin, unsigned long flags)
 {
-  int i;
-  int *new_f;
+  int new_fragment;
+  size_t len = 0;
+  char *ns = NULL;
 
-  dtrace("%p, %p, %i, %i", D, f, fragment, nf);
+  dtrace("%p, \"%s\", %i, \"%s\", 0x%lX", D, file, fragment_index, nsin, flags);
 
-  new_f = _GD_Realloc(D, *f, sizeof(*new_f) * ++nf);
-  if (new_f == NULL) {
-    dreturn("%i", -1);
-    return -1;
-  }
-  new_f[nf - 1] = fragment;
+  if (nsin) {
+    ns = _GD_NormaliseNamespace(D, nsin, &len);
+    if (ns == NULL)
+      GD_RETURN_ERROR(D);
+  } 
 
-  for (i = 0; i < D->n_fragment; ++i)
-    if (D->fragment[i].parent == fragment) {
-      nf = _GD_CollectFragments(D, &new_f, i, nf);
-      if (nf == -1)
-        break;
-    }
+  new_fragment = _GD_IncludeAffix(D, "gd_include_ns()", file, fragment_index,
+      ns, NULL, flags);
 
-  *f = new_f;
+  if (ns)
+    free(ns);
 
-  dreturn("%i", nf);
-  return nf;
+  dreturn("%i", new_fragment);
+  return new_fragment;
 }
 
-static int _GD_ContainsFragment(int* f, int nf, int fragment)
+int _GD_ContainsFragment(const int* f, int nf, int fragment)
 {
   int i;
 
@@ -489,42 +633,25 @@ int gd_uninclude(DIRFILE* D, int fragment_index, int del)
 
   dtrace("%p, %i, %i", D, fragment_index, del);
 
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if ((D->flags & GD_ACCMODE) == GD_RDONLY) {
-    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  if (fragment_index <= 0 || fragment_index >= D->n_fragment) {
-    _GD_SetError(D, GD_E_BAD_INDEX, 0, NULL, fragment_index, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  if ((D->flags & GD_ACCMODE) == GD_RDONLY) 
+    GD_SET_RETURN_ERROR(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
+  if (fragment_index <= 0 || fragment_index >= D->n_fragment)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_INDEX, 0, NULL, fragment_index, NULL);
 
   parent = D->fragment[fragment_index].parent;
 
-  if (D->fragment[parent].protection & GD_PROTECT_FORMAT) {
-    _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
+  if (D->fragment[parent].protection & GD_PROTECT_FORMAT)
+    GD_SET_RETURN_ERROR(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
         D->fragment[parent].cname);
-    dreturn("%i", -1);
-    return -1;
-  }
 
   /* find all affected fragments */
-  nf = _GD_CollectFragments(D, &f, fragment_index, 0);
+  nf = _GD_SubFragmentList(D, fragment_index, &f);
 
   if (D->error) {
     free(f);
-    dreturn("%i", -1);
-    return -1;
+    GD_RETURN_ERROR(D);
   }
 
   /* close affected raw fields */
@@ -542,8 +669,7 @@ int gd_uninclude(DIRFILE* D, int fragment_index, int del)
 
   if (D->error) {
     free(f);
-    dreturn("%i", -1);
-    return -1;
+    GD_RETURN_ERROR(D);
   }
 
   /* Nothing from now on may fail */
@@ -571,11 +697,7 @@ int gd_uninclude(DIRFILE* D, int fragment_index, int del)
 
   /* delete the fragments -- again, don't bother resizing D->fragment */
   for (j = 0; j < nf; ++j) {
-    _GD_ReleaseDir(D, D->fragment[f[j]].dirfd);
-    free(D->fragment[f[j]].cname);
-    free(D->fragment[f[j]].ename);
-    free(D->fragment[f[j]].bname);
-    free(D->fragment[f[j]].ref_name);
+    _GD_FreeF(D, f[j], f[j] + 1);
 
     memcpy(D->fragment + f[j], D->fragment + D->n_fragment - 1,
         sizeof(struct gd_fragment_t));
@@ -588,7 +710,6 @@ int gd_uninclude(DIRFILE* D, int fragment_index, int del)
   }
 
   /* Clear the cache of all fields */
-  /* FIXME: Should probably just clear affected fields */
   for (i = 0; i < D->n_entries; ++i) {
     D->entry[i]->flags &= ~GD_EN_CALC;
     for (j = 0; j < GD_MAX_LINCOM; ++j)
