@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2017 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -48,6 +48,9 @@
 #endif
 
 /* library headers */
+#ifdef HAVE_FEATURES_H
+#include <features.h>
+#endif
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
@@ -106,16 +109,19 @@
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
-
-#ifndef SIZEOF_SIZE_T
-#define SIZEOF_SIZE_T (sizeof size_t)
+#ifdef HAVE_REGEX_H
+#include <regex.h>
 #endif
 
+
 /* MSCVRT defines size_t but not ssize_t */
 #ifdef __MSVCRT__
-#if SIZEOF_SIZE_T == 8
+#undef SIZEOF_SIZE_T
+#ifdef _WIN64
+#define SIZEOF_SIZE_T 8
 typedef __int64 ssize_t;
 #else
+#define SIZEOF_SIZE_T 4
 typedef int ssize_t;
 #endif
 #endif
@@ -159,7 +165,7 @@ typedef gd_off64_t off64_t;
 /* the open() in the MSVCRT doesn't permit open()ing directories */
 #ifdef __MSVCRT__
 #define GD_NO_DIR_OPEN
-/* rename open() flags */
+/* rename open() 0 */
 #define O_RDWR _O_RDWR
 #define O_RDONLY _O_RDONLY
 #define O_CREAT _O_CREAT
@@ -313,6 +319,10 @@ double cimag(double complex z);
 #define GD_UINT_TYPE ((gd_type_t)(SIZEOF_UNSIGNED_INT))
 
 /* a few integer limits */
+#ifndef SIZEOF_SIZE_T
+#define SIZEOF_SIZE_T (sizeof(size_t))
+#endif
+
 #define GD_INT64_MAX ((int64_t)((uint64_t)-1>>1))
 #define GD_SSIZE_T_MAX ((ssize_t)((size_t)-1>>1))
 #define GD_SIZE_T_MAX ((size_t)-1)
@@ -415,6 +425,9 @@ extern "C" {
 }
 #endif
 
+#ifdef WIN32
+#define __func__ __FUNCTION__
+#endif
 
 #define dtracevoid() printf("%s %s()\n", gd_coladd(), __func__)
 #define dtrace(fmt, ...) printf("%s %s(" fmt ")\n", gd_coladd(), \
@@ -435,6 +448,34 @@ extern "C" {
 #define dwatch(...)
 #endif
 
+/* These doesn't return */
+#define GD_SET_RETURN_ERROR(D,e,s,f,l,t) do { \
+  _GD_SetError(D,e,s,f,l,t); \
+  dreturn("%i", e); \
+  return e; \
+} while(0)
+
+#define GD_RETURN_ERROR(D) do { \
+  dreturn("%i", D->error); \
+  return D->error; \
+} while(0)
+
+#define GD_RETURN_ERR_IF_INVALID(D) do { \
+  if (D->flags & GD_INVALID) \
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL); \
+  else \
+    _GD_ClearError(D); \
+} while(0)
+
+#define GD_RETURN_IF_INVALID(D,f,v) do { \
+  if (D->flags & GD_INVALID) { \
+    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL); \
+    dreturn(f, v); \
+    return v; \
+  } else \
+    _GD_ClearError(D); \
+} while(0)
+
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
@@ -493,6 +534,12 @@ char *basename(char *path);
 #define offsetof(t,m) ((size_t)(((char*)&((t*)0)->m) - (char*)0))
 #endif
 
+/* glibc 2.24 deprecates readdir_r */
+#if defined(__GLIBC__) && (__GLIBC__ > 2 || (__GLIBC__ == 2 \
+      && defined(__GLIBC_MINOR__) && __GLIBC_MINOR__ >= 24))
+#undef HAVE_READDIR_R
+#endif
+
 #ifdef HAVE_READDIR_R
 # define _GD_ReadDir readdir_r
 #else
@@ -806,6 +853,8 @@ ssize_t getdelim(char**, size_t*, int, FILE*);
 #define GD_E_FIELD_FORMAT      4
 #define GD_E_FIELD_STR         5
 
+#define GD_E_SUPPORT_REGEX     1
+
 #define GD_E_ENTRY_TYPE      1
 #define GD_E_ENTRY_METARAW   2
 #define GD_E_ENTRY_SPF       3
@@ -856,6 +905,8 @@ ssize_t getdelim(char**, size_t*, int, FILE*);
 #define GD_E_ARG_NODATA         4
 #define GD_E_ARG_NO_VERS        5
 #define GD_E_ARG_BAD_VERS       6
+#define GD_E_ARG_REGEX          7
+#define GD_E_ARG_PCRE           8
 
 #define GD_E_LONG_FLUSH         1
 
@@ -873,6 +924,7 @@ ssize_t getdelim(char**, size_t*, int, FILE*);
 #define GD_N_ENTRY_LISTS (GD_N_ENTYPES + 4)
 
 #define GD_LIST_VALID_STRING_VALUE 0x01
+#define GD_LIST_VALID_SARRAY_VALUE 0x02
 
 /* name types for ValidateField */
 #define GD_VF_NAME  0
@@ -880,22 +932,46 @@ ssize_t getdelim(char**, size_t*, int, FILE*);
 #define GD_VF_NS    2
 #define GD_VF_CODE  3
 
-/* MungeCode flags */
-#define GD_MC_RQ_PARTS 0x01
-#define GD_MC_CHECK_NS 0x02
-#define GD_MC_ERROR_OK 0x04
-#define GD_MC_NO_NS    0x08
-#define GD_MC_NO_ALLOC 0x10
+/* CodeOffsets flags (also used by StripCode, SlashDot, CheckCodeAffixes) */
+#define GD_CO_NSALL  0x001
+#define GD_CO_NSROOT 0x002
+#define GD_CO_ASSERT 0x004
+#define GD_CO_CHECK  0x008
+#define GD_CO_EARLY  0x010
+#define GD_CO_NAME   0x020
+#define GD_CO_ERROR  0x040
+#define GD_CO_REPR   0x080
+#define GD_CO_REPRZ  0x100
+
+/* Number of offsets returned by CodeOffsets */
+#define GD_N_CODEOFFSETS 10
+
+/* Internal rename flags */
+#define GD_REN_META 0x10 /* Updating a metafield */
 
 /* database metadata update data for a field rename */
-struct gd_rename_data_ {
-  int n_meta, n_code, old_dot, new_dot;
-  gd_entype_t type;
-  unsigned flags, dot_ind;
-  size_t old_len, new_len;
+struct gd_rename_update_ {
+  char *new_code; /* The newly minted code */
+  size_t new_len; /* The new code's length */
+  char **dst; /* A pointer to the old code */
+  size_t *dst_len; /* Non-NULL if we need to store new_len here */
+  int index; /* The affected metadata fragment */
+};
 
-  char **meta_name, **code_list, *old_code, *new_code;
-  gd_entry_t *E, **meta_entry;
+struct gd_rename_data_ {
+  gd_entype_t type; /* the type of the renamed field */
+  unsigned flags; /* GD_REN_ flags */
+  struct gd_flist_ *fl; /* The affected field_list struct */
+
+  int src_frag; /* Source fragment */
+  int dst_frag; /* Destination fragment for moves (for renames, this is -1) */
+
+  /* Old and new names */
+  size_t old_len, new_len;
+  const char *old_name, *new_name;
+
+  struct gd_rename_update_ *up; /* The list of updates */
+  unsigned n_up, up_size;
 };
 
 /* data file book-keeping record */
@@ -905,7 +981,7 @@ struct gd_raw_file_ {
   void* edata;
   int subenc;
   int error;
-  const DIRFILE *D;
+  DIRFILE *D;
   unsigned int mode;
   off64_t pos;
 };
@@ -921,10 +997,10 @@ struct gd_lut_ {
 
 /* field lists */
 struct gd_flist_ {
-  const char **alias_list;
   const char **entry_list[GD_N_ENTRY_LISTS];
   unsigned int entry_list_flags[GD_N_ENTRY_LISTS];
   const char **string_value_list;
+  const char ***sarray_value_list;
   void *const_value_list;
   gd_carray_t *carray_value_list;
   uint32_t value_list_validity;
@@ -933,6 +1009,10 @@ struct gd_flist_ {
 
 /* Unified entry struct */
 struct gd_private_entry_ {
+  size_t len; /* strlen(E->field) */
+
+  /* Input entries.  Aliases use entry[0] for the proximal target and
+   * entry[1] for the distal target */
   gd_entry_t* entry[GD_MAX_LINCOM];
   int repr[GD_MAX_LINCOM];
 
@@ -987,6 +1067,9 @@ struct gd_private_entry_ {
 /* the last record of the _GD_ef array is always the unknown encoding */
 #define GD_ENC_UNKNOWN (GD_N_SUBENCODINGS - 1)
 
+/* Internal gd_entry_t flags */
+#define GD_EN_EARLY 0x8000 /* field spec. contains non-namespace-related '.' */
+
 /* external module function provides flags */
 #define GD_EF_NAME    0x0001
 #define GD_EF_OPEN    0x0002
@@ -1083,8 +1166,6 @@ struct gd_fragment_t {
   char* sname; /* Subdirectory name (path relative to dirfile or absolute) */
   char *bname; /* basename (filename) */
   char* ename; /* External name (the one that appears in the format file) */
-  char *ns; /* unused */
-  size_t nsl; /* strlen(ns) */
   void *enc_data;
   int modified;
   int parent;
@@ -1097,8 +1178,12 @@ struct gd_fragment_t {
   off64_t frame_offset;
   uint32_t vers;
 
-  char *prefix;
-  char *suffix;
+  char *ns; /* root namespace */
+  size_t nsl; /* strlen(ns) */
+  char *px; /* prefix */
+  size_t pxl; /* strlen(px) */
+  char *sx; /* suffix */
+  size_t sxl; /* strlen(sx) */
 };
 
 /* directory metadata */
@@ -1153,11 +1238,9 @@ struct gd_dirfile_ {
 
   /* field counts */
   unsigned int n_entries;
-  unsigned int n_dot;
 
   /* field array */
   gd_entry_t** entry;
-  gd_entry_t** dot_list;
 
   /* the reference field */
   gd_entry_t* reference_field;
@@ -1177,6 +1260,7 @@ struct gd_dirfile_ {
   int n_fragment;
 
   /* field lists */
+  const char **regex_list;
   struct gd_flist_ fl;
 
   /* syntax error callback */
@@ -1184,6 +1268,11 @@ struct gd_dirfile_ {
   void* sehandler_extra;
 };
 
+/* The caller's preferred memory manager */
+extern void *(*_GD_CMalloc)(size_t);
+extern char *(*_GD_CStrdup)(const char*);
+extern void (*_GD_CFree)(void*);
+
 /* These are internal functions we need to expose so that modules can use them
  */
 #ifdef USE_MODULES
@@ -1225,7 +1314,6 @@ int _GD_MakeTempFile(const DIRFILE*, int, char*);
 
 /* forward declarations */
 void *_GD_Alloc(DIRFILE*, gd_type_t, size_t) __attribute_malloc__;
-int _GD_BadInput(DIRFILE *, const gd_entry_t *, int, gd_entype_t, int);
 
 #define _GD_BadWindop(op) \
   ( \
@@ -1234,13 +1322,17 @@ int _GD_BadInput(DIRFILE *, const gd_entry_t *, int, gd_entype_t, int);
    (op != GD_WINDOP_SET) && (op != GD_WINDOP_CLR) \
   )
 
+int _GD_BadType(int, gd_type_t);
+char *_GD_BuildCode(DIRFILE*, int, const char*, size_t, const char*, int,
+    size_t *restrict) __attribute_malloc__;
 int _GD_CalculateEntry(DIRFILE *restrict, gd_entry_t *restrict, int);
 char *_GD_CanonicalPath(const char *restrict, const char *restrict);
 int _GD_CheckByteSex(gd_type_t, unsigned, unsigned, int, int *restrict);
-gd_entry_t *_GD_CheckParent(DIRFILE *restrict D,
-    const struct parser_state *restrict, char **restrict name, int me);
-int _GD_CheckCodeAffixes(DIRFILE *D, const char *field_code, int fragment,
-    int set_error);
+gd_entry_t *_GD_CheckParent(DIRFILE *restrict,
+    const struct parser_state *restrict, char **restrict, size_t *restrict,
+    int);
+int _GD_CheckCodeAffixes(DIRFILE *restrict, const char *restrict, int,
+    unsigned);
 void _GD_CInvertData(DIRFILE *restrict, void *restrict, gd_type_t,
     GD_DCOMPLEXA(dividend), size_t);
 void _GD_CleanUpRename(struct gd_rename_data_*, int);
@@ -1251,6 +1343,9 @@ void _GD_CleanUpRename(struct gd_rename_data_*, int);
 void _GD_CLincomData(DIRFILE *restrict, int, void *restrict, gd_type_t,
     const GD_DCOMPLEXP_t restrict, const GD_DCOMPLEXP_t restrict,
     GD_DCOMPLEXV(m), GD_DCOMPLEXV(b), const unsigned int *restrict, size_t);
+int _GD_CodeOffsets(DIRFILE *restrict, int, const char *restrict, unsigned,
+    size_t offset[GD_N_CODEOFFSETS]);
+int _GD_ContainsFragment(const int *, int, int);
 void _GD_ConvertType(DIRFILE *restrict, const void *restrict, gd_type_t,
     void *restrict, gd_type_t, size_t) gd_nothrow;
 gd_type_t _GD_ConstType(DIRFILE *D, gd_type_t type);
@@ -1264,9 +1359,10 @@ off64_t _GD_DoSeek(DIRFILE *restrict, gd_entry_t *restrict,
 int _GD_EntryCmp(const void*, const void*);
 gd_entry_t *_GD_FindEntry(DIRFILE *restrict, const char *restrict);
 gd_entry_t *_GD_FindField(const DIRFILE *restrict, const char *restrict,
-    gd_entry_t *const *, unsigned int, int, unsigned int *restrict);
+    size_t, gd_entry_t *const *, unsigned int, int, unsigned int *restrict);
 gd_entry_t *_GD_FindFieldAndRepr(DIRFILE *restrict, const char *restrict,
-    char **restrict, int *restrict, unsigned int *restrict, int, int);
+    int *restrict, unsigned int *restrict, int);
+int _GD_FindInputs(DIRFILE *restrict, gd_entry_t *restrict, int) gd_nothrow;
 uint64_t _GD_FindVersion(DIRFILE *D);
 void _GD_FixEndianness(void*, size_t, gd_type_t, unsigned, unsigned);
 int _GD_FileSwapBytes(const DIRFILE *restrict, const gd_entry_t *restrict);
@@ -1274,13 +1370,13 @@ int _GD_FiniRawIO(DIRFILE*, const gd_entry_t*, int, int);
 void _GD_Flush(DIRFILE *restrict, gd_entry_t *restrict, int, int);
 void _GD_FlushMeta(DIRFILE* D, int fragment, int force);
 void _GD_FreeE(DIRFILE *restrict, gd_entry_t *restrict, int);
+void _GD_FreeF(DIRFILE *restrict, int, int);
 void _GD_FreeFL(struct gd_flist_ *);
-off64_t _GD_GetEOF(DIRFILE *restrict, const gd_entry_t *restrict,
+off64_t _GD_GetEOF(DIRFILE *restrict, gd_entry_t *restrict,
     const char *restrict, int *restrict);
 off64_t _GD_GetIOPos(DIRFILE *restrict, gd_entry_t *restrict, off64_t);
 char *_GD_GetLine(FILE *restrict, size_t *restrict, int *restrict);
-int _GD_GetRepr(DIRFILE *restrict, const char *restrict,
-    char **restrict, int);
+int _GD_GetRepr(const char *restrict, size_t *restrict);
 int _GD_GetScalar(DIRFILE *restrict, const char *restrict, int*, gd_type_t,
    void *restrict, gd_entry_t*);
 unsigned int _GD_GetSPF(DIRFILE*, gd_entry_t*);
@@ -1305,33 +1401,27 @@ void _GD_LincomData(DIRFILE *restrict, int n, void *restrict,
     const unsigned int *restrict, size_t);
 void _GD_LinterpData(DIRFILE *restrict, void *restrict, gd_type_t, int,
     const double *restrict, size_t, const struct gd_lut_ *restrict, size_t);
-int _GD_ListEntry(const gd_entry_t *E, int meta, int hidden, int noalias,
-    int special, gd_entype_t type);
+int _GD_ListEntry(const gd_entry_t*, int, int, int, int, int, gd_entype_t);
 char *_GD_MakeFullPath(DIRFILE *restrict, int, const char *restrict, int);
-void *_GD_Malloc(DIRFILE *D, size_t size);
+void *_GD_Malloc(DIRFILE *D, size_t size) __attribute_malloc__;
 int _GD_MissingFramework(int encoding, unsigned int funcs);
 int _GD_MogrifyFile(DIRFILE *restrict, gd_entry_t *restrict, unsigned long int,
     unsigned long int, off64_t, int, int, char *restrict);
-char *_GD_MungeCode(DIRFILE *restrict, const char *restrict, size_t,
-    const char *restrict, const char *restrict, const char *restrict,
-    const char *restrict, const char *restrict, char **restrict,
-    int *restrict, unsigned);
 gd_type_t _GD_NativeType(DIRFILE *restrict, gd_entry_t *restrict, int);
-unsigned int _GD_NEntries(DIRFILE*, struct gd_private_entry_*, int,
-    unsigned int);
-DIRFILE *_GD_Open(DIRFILE*, int, const char*, unsigned long,
-    gd_parser_callback_t, void*);
+char *_GD_NormaliseNamespace(DIRFILE *restrict, const char *restrict,
+    size_t *restrict) __attribute_malloc__;
 gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict,
-    const struct parser_state *restrict, int, char**,
+    const struct parser_state *restrict, int, char**, size_t,
     const gd_entry_t *restrict, int, int, int, char **, const char *);
 char *_GD_ParseFragment(FILE *restrict, DIRFILE*, struct parser_state *restrict,
     int, int);
 void _GD_PerformRename(DIRFILE *restrict, struct gd_rename_data_ *restrict);
 struct gd_rename_data_ *_GD_PrepareRename(DIRFILE *restrict, char *restrict,
-    gd_entry_t *restrict, int, unsigned, int, unsigned);
-void _GD_ReadLinterpFile(DIRFILE *restrict, gd_entry_t *restrict);
+    size_t, gd_entry_t *restrict, int, unsigned);
+int _GD_ReadLinterpFile(DIRFILE *restrict, gd_entry_t *restrict);
 void *_GD_Realloc(DIRFILE *restrict, void *restrict, size_t size);
 void _GD_ReleaseDir(DIRFILE *D, int dirfd);
+int _GD_SlashDot(const char*, size_t, unsigned, const char**, const char**);
 int _GD_Seek(DIRFILE *restrict, gd_entry_t *restrict, off64_t offset,
     unsigned int mode);
 void _GD_SetEncIOError(DIRFILE*, int suberror, const struct gd_raw_file_ *);
@@ -1344,13 +1434,19 @@ void _GD_SimpleParserInit(DIRFILE *restrict D, const char *restrict name,
 int _GD_ShutdownDirfile(DIRFILE*, int, int);
 int _GD_StrCmpNull(const char *restrict, const char *restrict);
 char *_GD_Strdup(DIRFILE *restrict, const char *restrict);
+char *_GD_StripCode(DIRFILE *restrict, int, const char *restrict, unsigned)
+  __attribute_malloc__;
+int _GD_SubFragmentList(DIRFILE *restrict, int, int **restrict);
 int _GD_Supports(DIRFILE *, const gd_entry_t*, unsigned int funcs);
 int _GD_Tokenise(DIRFILE *restrict, const struct parser_state *restrict,
     const char *restrict, char **, const char **, int, char **);
 int _GD_TokToNum(const char *restrict, int, int, double*, double*, uint64_t*,
     int64_t*);
 void _GD_UpdateAliases(DIRFILE*, int);
-int _GD_ValidateField(const char*, int, int, unsigned, int*);
+char *_GD_UpdateCode(DIRFILE *restrict, int, const char *restrict, int,
+    const char *restrict, size_t, const char *restrict, size_t,
+    const char *restrict, size_t) __attribute_malloc__;
+int _GD_ValidateField(const char*, size_t, int, int, unsigned);
 ssize_t _GD_WriteOut(const gd_entry_t*, const struct encoding_t*, const void*,
     gd_type_t, size_t, int);
 
