@@ -50,15 +50,17 @@ static unsigned int gd_max_(unsigned int A, unsigned int B)
  *  iQ - scalar index in Q
  *  lQ - literal value in Q
  *
- *  calculated: whether E was calculated (GD_EN_CALCULATED) before _GD_Change
- *              was called.
+ *  calculated: whether E was calculated (GD_EN_CALC) before _GD_Change was
+ *                          called.
  *
  *  alter_literal: boolean; true if lN and lQ differ and lN is not a special
  *                          "keep-as-is" value
  *
- * This function performs one of five actions, inicated by 0 through 4
+ *  early: whether E is from DSV <= 5 (GD_EN_EARLY)
+ *
+ * This function performs one of five actions, indicated by 0 through 4,
  * according to the values of sN, sQ, and alter_literal as given in 
- * the following table where:
+ * the following table, where:
  *
  * - sQ can be 'N' (NULL) or 'a' (a non-NULL field code from the
  *                                                     original entry)
@@ -90,13 +92,13 @@ static unsigned int gd_max_(unsigned int A, unsigned int B)
  */
 static int _GD_AlterScalar(DIRFILE* D, int alter_literal, gd_type_t type,
     void *lQ, const void *lN, char **sQ, int *iQ, const char *sN,
-    int iN, int calculated, int fragment_index)
+    int iN, int calculated, int early, int fragment_index)
 {
   int r = 0;
   int set_lQ = 0;
 
-  dtrace("%p, %i, 0x%X, %p, %p, %p, %p, %p, %i, %i, %i", D, alter_literal, type,
-      lQ, lN, sQ, iQ, sN, iN, calculated, fragment_index);
+  dtrace("%p, %i, 0x%X, %p, %p, %p, %p, %p, %i, %i, %i, %i", D, alter_literal,
+      type, lQ, lN, sQ, iQ, sN, iN, calculated, early, fragment_index);
 
   if (sN == NULL) {
     if (*sQ != NULL) {
@@ -148,9 +150,11 @@ static int _GD_AlterScalar(DIRFILE* D, int alter_literal, gd_type_t type,
      *    been asked to move the raw file, _GD_Change is going to need to
      *    recalculate the entry; no need to change lQ: it's ignored. */
 
-    if (_GD_CheckCodeAffixes(D, sN, fragment_index, 1))
+    if (_GD_CheckCodeAffixes(D, sN, fragment_index,
+          GD_CO_ERROR | (early ? GD_CO_EARLY : 0)))
+    {
       ; /* reject codes with bad affixes */
-    else if (iN == -1 && !(D->flags & GD_NOSTANDARD) && D->standards <= 7 &&
+    } else if (iN == -1 && !(D->flags & GD_NOSTANDARD) && D->standards <= 7 &&
         _GD_TokToNum(sN, D->standards, 1, NULL, NULL, NULL, NULL) != -1)
     {
       /* when using early Standards, reject ambiguous field codes */
@@ -265,12 +269,15 @@ static void _GD_SPFConvert(DIRFILE* D, void *A, unsigned int spfA, void *B,
 
 /* returns -1 on error, 1 if modified, 0 if no change */
 static int _GD_AlterInField(DIRFILE *D, int i, char **Q, char *const *N,
-    char *const *E, int fragment_index, int force)
+    char *const *E, int early, int fragment_index, int force)
 {
-  dtrace("%p, %i, %p, %p, %p, %i, %i", D, i, Q, N, E, fragment_index, force);
+  dtrace("%p, %i, %p, %p, %p, %i, %i, %i", D, i, Q, N, E, early, fragment_index,
+      force);
 
   if (force || (N[i] != NULL && strcmp(E[i], N[i]))) {
-    if (_GD_CheckCodeAffixes(D, N[i], fragment_index, 1)) {
+    if (_GD_CheckCodeAffixes(D, N[i], fragment_index,
+          GD_CO_ERROR | (early ? GD_CO_EARLY : 0)))
+    {
       dreturn("%i", -1);
       return -1;
     } else if ((Q[i] = _GD_Strdup(D, N[i])) == NULL) {
@@ -292,7 +299,7 @@ static int _GD_AlterInField(DIRFILE *D, int i, char **Q, char *const *N,
 static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
     int flags)
 {
-  int i, j;
+  int i, j, early, calc;
   int field_free = 0;
   int scalar_free = 0;
   int modified = 0;
@@ -307,20 +314,19 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
 
   if ((D->flags & GD_ACCMODE) != GD_RDWR)
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-  else if ((E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL))
-      == NULL)
-  {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-  } else if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT)
+  else if ((E = _GD_FindEntry(D, field_code)) == NULL)
+    ; /* Error already set */
+  else if (D->fragment[E->fragment_index].protection & GD_PROTECT_FORMAT)
     _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_FORMAT, NULL, 0,
         D->fragment[E->fragment_index].cname);
   else if (E->field_type != N->field_type)
     _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_MATCH, NULL, 0, field_code);
 
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (D->error)
+    GD_RETURN_ERROR(D);
+
+  early = E->flags & GD_EN_EARLY ? 1 : 0;
+  calc = E->flags & GD_EN_CALC ? 1 : 0;
 
   memcpy(&Qe, E->e, sizeof(struct gd_private_entry_));
   memcpy(&Q, E, sizeof(gd_entry_t));
@@ -338,8 +344,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
     case GD_RAW_ENTRY:
       j = _GD_AlterScalar(D, N->EN(raw,spf) && N->EN(raw,spf) != E->EN(raw,spf),
           GD_UINT_TYPE, &Q.EN(raw,spf), &N->EN(raw,spf), Q.scalar, Q.scalar_ind,
-          N->scalar[0], N->scalar_ind[0], E->flags & GD_EN_CALC,
-          E->fragment_index);
+          N->scalar[0], N->scalar_ind[0], calc, early, E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
@@ -354,17 +359,15 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       /* nothing to do */
       if (Q.EN(raw,spf) == E->EN(raw,spf) && Q.EN(raw,data_type) ==
           E->EN(raw,data_type) && Q.scalar[0] == E->scalar[0])
+      {
         break;
+      }
 
       modified = 1;
 
-      if (Q.EN(raw,data_type) & 0x40 ||
-          (Qe.u.raw.size = GD_SIZE(Q.EN(raw,data_type))) == 0)
-      {
-        _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, Q.EN(raw,data_type), NULL);
-        dreturn("%i", -1);
-        return -1;
-      }
+      if (_GD_BadType(D->standards, Q.EN(raw,data_type)))
+        GD_SET_RETURN_ERROR(D, GD_E_BAD_TYPE, 0, NULL, Q.EN(raw,data_type),
+            NULL);
 
       if (flags) {
         ssize_t nread, nwrote;
@@ -376,8 +379,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         if (!(Q.flags & GD_EN_CALC))
           if (gd_get_constant(D, Q.scalar[0], GD_UINT_TYPE, &Q.EN(raw,spf)))
             break;
-        
-        if (!(E->flags & GD_EN_CALC))
+
+        if (!calc)
           if (!_GD_CalculateEntry(D, E, 1))
             break;
 
@@ -515,7 +518,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       for (i = 0; i < Q.EN(lincom,n_fields); ++i) {
         if (flags & 0x1) {
           j = _GD_AlterInField(D, i, Q.in_fields, N->in_fields, E->in_fields,
-              E->fragment_index, E->EN(lincom,n_fields) <= i);
+              early, E->fragment_index, E->EN(lincom,n_fields) <= i);
           if (j < 0)
             break;
           else if (j) {
@@ -529,14 +532,13 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
             j = _GD_AlterScalar(D, !gd_ccmpc_(E->EN(lincom,cm)[i],
                   N->EN(lincom,cm)[i]), GD_COMPLEX128, Q.EN(lincom,cm) + i,
                 N->EN(lincom,cm) + i, Q.scalar + i, Q.scalar_ind + i,
-                N->scalar[i], N->scalar_ind[i], E->flags & GD_EN_CALC,
-                E->fragment_index);
+                N->scalar[i], N->scalar_ind[i], calc, early, E->fragment_index);
             Q.EN(lincom,m)[i] = creal(Q.EN(lincom,cm)[i]);
           } else {
             j = _GD_AlterScalar(D, E->EN(lincom,m)[i] != N->EN(lincom,m)[i],
                 GD_FLOAT64, Q.EN(lincom,m) + i, N->EN(lincom,m) + i,
                 Q.scalar + i, Q.scalar_ind + i, N->scalar[i], N->scalar_ind[i],
-                E->flags & GD_EN_CALC, E->fragment_index);
+                calc, early, E->fragment_index);
             gd_rs2cs_(Q.EN(lincom,cm)[i], Q.EN(lincom,m)[i]);
           }
 
@@ -556,7 +558,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
                   N->EN(lincom,cb)[i]), GD_COMPLEX128, Q.EN(lincom,cb) + i,
                 N->EN(lincom,cb) + i, Q.scalar + i + GD_MAX_LINCOM,
                 Q.scalar_ind + i + GD_MAX_LINCOM, N->scalar[i +  GD_MAX_LINCOM],
-                N->scalar_ind[i + GD_MAX_LINCOM], E->flags & GD_EN_CALC,
+                N->scalar_ind[i + GD_MAX_LINCOM], calc, early,
                 E->fragment_index);
             Q.EN(lincom,b)[i] = creal(Q.EN(lincom,cb)[i]);
           } else {
@@ -564,7 +566,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
                 GD_FLOAT64, Q.EN(lincom,b) + i, N->EN(lincom,b) + i,
                 Q.scalar + i + GD_MAX_LINCOM, Q.scalar_ind + i + GD_MAX_LINCOM,
                 N->scalar[i + GD_MAX_LINCOM], N->scalar_ind[i + GD_MAX_LINCOM],
-                E->flags & GD_EN_CALC, E->fragment_index);
+                calc, early, E->fragment_index);
             gd_rs2cs_(Q.EN(lincom,cb)[i], Q.EN(lincom,b)[i]);
           }
 
@@ -587,7 +589,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       break;
     case GD_LINTERP_ENTRY:
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -639,7 +641,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       j = _GD_AlterScalar(D, N->EN(bit,numbits) >= 1 && E->EN(bit,numbits) !=
           N->EN(bit,numbits), GD_INT_TYPE, &Q.EN(bit,numbits),
           &N->EN(bit,numbits), Q.scalar + 1, Q.scalar_ind + 1, N->scalar[1],
-          N->scalar_ind[1], E->flags & GD_EN_CALC, E->fragment_index);
+          N->scalar_ind[1], calc, early, E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
@@ -652,8 +654,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
 
       j = _GD_AlterScalar(D, N->EN(bit,bitnum) >= 0 && E->EN(bit,bitnum) !=
           N->EN(bit,bitnum), GD_INT_TYPE, &Q.EN(bit,bitnum), &N->EN(bit,bitnum),
-          Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0],
-          E->flags & GD_EN_CALC, E->fragment_index);
+          Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0], calc, early,
+          E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
@@ -665,7 +667,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         modified = 1;
 
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -676,8 +678,10 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       break;
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -686,7 +690,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       }
 
       j = _GD_AlterInField(D, 1, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -697,7 +701,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       break;
     case GD_RECIP_ENTRY:
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -710,8 +714,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         j = _GD_AlterScalar(D, cabs(N->EN(recip,cdividend)) != 0 &&
             !gd_ccmpc_(E->EN(recip,cdividend), N->EN(recip,cdividend)),
             GD_COMPLEX128, &Q.EN(recip,cdividend), &(N->EN(recip,cdividend)),
-            Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0],
-            E->flags & GD_EN_CALC, E->fragment_index);
+            Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0], calc, early,
+            E->fragment_index);
         Q.EN(recip,dividend) = creal(Q.EN(recip,cdividend));
         if (cimag(Q.EN(recip,cdividend)) != 0)
           Q.flags |= GD_EN_COMPSCAL;
@@ -719,8 +723,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         j = _GD_AlterScalar(D, N->EN(recip,dividend) != 0 &&
             E->EN(recip,dividend) != N->EN(recip,dividend), GD_FLOAT64,
             &Q.EN(recip,dividend), &(N->EN(recip,dividend)), Q.scalar,
-            Q.scalar_ind, N->scalar[0], N->scalar_ind[0],
-            E->flags & GD_EN_CALC, E->fragment_index);
+            Q.scalar_ind, N->scalar[0], N->scalar_ind[0], calc, early,
+            E->fragment_index);
         gd_rs2cs_(Q.EN(recip,cdividend), Q.EN(recip,dividend));
       }
 
@@ -740,20 +744,19 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
     case GD_PHASE_ENTRY:
       j = _GD_AlterScalar(D, E->EN(phase,shift) != N->EN(phase,shift), GD_INT64,
           &Q.EN(phase,shift), &N->EN(phase,shift), Q.scalar, Q.scalar_ind,
-          N->scalar[0], N->scalar_ind[0], E->flags & GD_EN_CALC,
-          E->fragment_index);
+          N->scalar[0], N->scalar_ind[0], calc, early, E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
       if (j & GD_AS_FREE_SCALAR)
-        scalar_free |= 1;
+        scalar_free = 1;
       if (j & GD_AS_NEED_RECALC)
         Q.flags &= ~GD_EN_CALC;
       if (j & GD_AS_MODIFIED)
         modified = 1;
 
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -764,7 +767,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       break;
     case GD_POLYNOM_ENTRY:
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -785,14 +788,13 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
             j = _GD_AlterScalar(D, !gd_ccmpc_(E->EN(polynom,ca)[i],
                   N->EN(polynom,ca)[i]), GD_COMPLEX128, Q.EN(polynom,ca) + i,
                 N->EN(polynom,ca) + i, Q.scalar + i, Q.scalar_ind + i,
-                N->scalar[i], N->scalar_ind[i], E->flags & GD_EN_CALC,
-                E->fragment_index);
+                N->scalar[i], N->scalar_ind[i], calc, early, E->fragment_index);
             Q.EN(polynom,a)[i] = creal(Q.EN(polynom,ca)[i]);
           } else {
             j = _GD_AlterScalar(D, E->EN(polynom,a)[i] != N->EN(polynom,a)[i],
                 GD_FLOAT64, Q.EN(polynom,a) + i, N->EN(polynom,a) + i,
                 Q.scalar + i, Q.scalar_ind + i, N->scalar[i], N->scalar_ind[i],
-                E->flags & GD_EN_CALC, E->fragment_index);
+                calc, early, E->fragment_index);
             gd_rs2cs_(Q.EN(polynom,ca)[i], Q.EN(polynom,a)[i]);
           }
 
@@ -826,20 +828,20 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
           j = _GD_AlterScalar(D, E->EN(window,threshold.i) !=
               N->EN(window,threshold.i), GD_INT64, &Q.EN(window,threshold.i),
               &N->EN(window,threshold.i), Q.scalar, Q.scalar_ind, N->scalar[0],
-              N->scalar_ind[0], E->flags & GD_EN_CALC, E->fragment_index);
+              N->scalar_ind[0], calc, early, E->fragment_index);
           break;
         case GD_WINDOP_SET:
         case GD_WINDOP_CLR:
           j = _GD_AlterScalar(D, E->EN(window,threshold.u) !=
               N->EN(window,threshold.u), GD_UINT64, &Q.EN(window,threshold.u),
               &N->EN(window,threshold.u), Q.scalar, Q.scalar_ind, N->scalar[0],
-              N->scalar_ind[0], E->flags & GD_EN_CALC, E->fragment_index);
+              N->scalar_ind[0], calc, early, E->fragment_index);
           break;
         default:
           j = _GD_AlterScalar(D, E->EN(window,threshold.r) !=
               N->EN(window,threshold.r), GD_FLOAT64, &Q.EN(window,threshold.r),
               &N->EN(window,threshold.r), Q.scalar, Q.scalar_ind, N->scalar[0],
-              N->scalar_ind[0], E->flags & GD_EN_CALC, E->fragment_index);
+              N->scalar_ind[0], calc, early, E->fragment_index);
           break;
       }
 
@@ -853,7 +855,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         modified = 1;
 
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -862,7 +864,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       }
 
       j = _GD_AlterInField(D, 1, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -875,8 +877,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       j = _GD_AlterScalar(D, N->EN(mplex,period) != -1 &&
           E->EN(mplex,period) != N->EN(mplex,period), GD_INT_TYPE,
           &Q.EN(mplex,period), &N->EN(mplex,period), Q.scalar + 1,
-          Q.scalar_ind + 1, N->scalar[1], N->scalar_ind[1],
-          E->flags & GD_EN_CALC, E->fragment_index);
+          Q.scalar_ind + 1, N->scalar[1], N->scalar_ind[1], calc, early,
+          E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
@@ -889,8 +891,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
 
       j = _GD_AlterScalar(D, E->EN(mplex,count_val) != N->EN(mplex,count_val),
           GD_INT_TYPE, &Q.EN(mplex,count_val), &N->EN(mplex,count_val),
-          Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0],
-          E->flags & GD_EN_CALC, E->fragment_index);
+          Q.scalar, Q.scalar_ind, N->scalar[0], N->scalar_ind[0], calc, early,
+          E->fragment_index);
 
       if (j & GD_AS_ERROR)
         break;
@@ -902,7 +904,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
         modified = 1;
 
       j = _GD_AlterInField(D, 0, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -911,7 +913,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       }
 
       j = _GD_AlterInField(D, 1, Q.in_fields, N->in_fields, E->in_fields,
-          E->fragment_index, 0);
+          early, E->fragment_index, 0);
       if (j < 0)
         break;
       else if (j) {
@@ -924,13 +926,9 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       Q.EN(scalar,const_type) = (N->EN(scalar,const_type) == GD_NULL) ?
         E->EN(scalar,const_type) : N->EN(scalar,const_type);
 
-      if (Q.EN(scalar,const_type) & 0x40 || GD_SIZE(Q.EN(scalar,const_type))
-          == 0)
-      {
-        _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, Q.EN(scalar,const_type), NULL);
-        dreturn("%i", -1);
-        return -1;
-      }
+      if (_GD_BadType(D->standards, Q.EN(scalar,const_type)))
+        GD_SET_RETURN_ERROR(D, GD_E_BAD_TYPE, 0, NULL, Q.EN(scalar,const_type),
+            NULL);
 
       type = _GD_ConstType(D, Q.EN(scalar,const_type));
       if (Q.EN(scalar,const_type) != E->EN(scalar,const_type))
@@ -941,10 +939,9 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       else {
         /* type convert */
         Qe.u.scalar.d = _GD_Malloc(D, GD_SIZE(type));
-        if (Qe.u.scalar.d == NULL) {
-          dreturn("%i", -1);
-          return -1;
-        }
+        if (Qe.u.scalar.d == NULL)
+          GD_RETURN_ERROR(D);
+
         if (type == GD_COMPLEX128) {
           *(double*)Qe.u.scalar.d = (E->EN(scalar,const_type) & GD_IEEE754) ?
             *(double*)E->e->u.scalar.d : (E->EN(scalar,const_type) & GD_SIGNED)
@@ -980,9 +977,7 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       {
         modified = 1;
 
-        if (Q.EN(scalar,const_type) & 0x40 ||
-            GD_SIZE(Q.EN(scalar,const_type)) == 0)
-        {
+        if (_GD_BadType(D->standards, Q.EN(scalar,const_type))) {
           _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, Q.EN(scalar,const_type),
               NULL);
           break;
@@ -1012,6 +1007,38 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       } else
         Qe.u.scalar.d = E->e->u.scalar.d;
       break;
+    case GD_SARRAY_ENTRY:
+      if (N->EN(scalar,array_len) == 0) {
+        Q.EN(scalar,array_len) = E->EN(scalar,array_len);
+        Qe.u.scalar.d = E->e->u.scalar.d;
+      } else {
+        Q.EN(scalar,array_len) = N->EN(scalar,array_len);
+
+        if (Q.EN(scalar,array_len) != E->EN(scalar,array_len)) {
+          modified = 1;
+
+          if (Q.EN(scalar,array_len) < E->EN(scalar,array_len)) {
+            /* Free dropped elements */
+            size_t i;
+            for (i = Q.EN(scalar,array_len); i < E->EN(scalar,array_len); ++i)
+              free(((char**)E->e->u.scalar.d)[i]);
+          }
+
+          Qe.u.scalar.d = _GD_Realloc(D, E->e->u.scalar.d,
+              sizeof(const char *) * Q.EN(scalar,array_len));
+          if (Qe.u.scalar.d == NULL)
+            break;
+
+          if (Q.EN(scalar,array_len) > E->EN(scalar,array_len)) {
+            /* Zero new elements */
+            size_t i;
+            for (i = E->EN(scalar,array_len); i < Q.EN(scalar,array_len); ++i)
+              ((char**)Qe.u.scalar.d)[i] = _GD_Strdup(D, "");
+          }
+        } else
+          Qe.u.scalar.d = E->e->u.scalar.d;
+      }
+      break;
     case GD_INDEX_ENTRY:
       /* INDEX may not be modified */
       _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_BAD, NULL, 0, field_code);
@@ -1021,10 +1048,8 @@ static int _GD_Change(DIRFILE *D, const char *field_code, const gd_entry_t *N,
       break;
   }
 
-  if (D->error) {
-    dreturn("%i", -1);
-    return -1;
-  }
+  if (D->error)
+    GD_RETURN_ERROR(D);
 
   if (modified) {
     for (i = 0; i < GD_MAX_LINCOM; ++i) {
@@ -1061,16 +1086,11 @@ int gd_alter_entry(DIRFILE* D, const char* field_code,
 
   dtrace("%p, \"%s\", %p, %i", D, field_code, entry, move);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   /* To ensure .e is NULLed */
   memcpy(&N, entry, sizeof(gd_entry_t));
+  N.e = NULL;
 
   /* for these field types, move is a set of bitflags; we set them all */
   if (N.field_type == GD_LINCOM_ENTRY || N.field_type == GD_POLYNOM_ENTRY)
@@ -1090,13 +1110,7 @@ int gd_alter_raw(DIRFILE *D, const char *field_code,
 
   dtrace("%p, \"%s\", %u, 0x%X, %i", D, field_code, spf, data_type, move);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_RAW_ENTRY;
@@ -1120,31 +1134,20 @@ int gd_alter_lincom(DIRFILE* D, const char* field_code, int n_fields,
   dtrace("%p, \"%s\", %i, %p, %p, %p", D, field_code, n_fields, in_fields, m,
       b);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_LINCOM_ENTRY;
-  if (n_fields > GD_MAX_LINCOM || n_fields < 0) {
-    _GD_SetError(D, GD_E_BAD_ENTRY, GD_E_ENTRY_NFIELDS, NULL, n_fields, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if (n_fields != 0)
+  if (n_fields > GD_MAX_LINCOM || n_fields < 0)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_ENTRY, GD_E_ENTRY_NFIELDS, NULL, n_fields,
+        NULL);
+  else if (n_fields != 0)
     N.EN(lincom,n_fields) = n_fields;
   else {
-    gd_entry_t *E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1,
-        NULL);
+    gd_entry_t *E = _GD_FindEntry(D, field_code);
 
-    if (E == NULL) {
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-      dreturn("%i", -1);
-      return -1;
-    }
+    if (E == NULL)
+      GD_RETURN_ERROR(D);
 
     N.EN(lincom,n_fields) = E->EN(lincom,n_fields);
   }
@@ -1187,32 +1190,21 @@ int gd_alter_clincom(DIRFILE* D, const char* field_code, int n_fields,
   dtrace("%p, \"%s\", %i, %p, %p, %p", D, field_code, n_fields, in_fields, cm,
       cb);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_LINCOM_ENTRY;
   N.flags = GD_EN_COMPSCAL;
-  if (n_fields > GD_MAX_LINCOM || n_fields < 0) {
-    _GD_SetError(D, GD_E_BAD_ENTRY, GD_E_ENTRY_NFIELDS, NULL, n_fields, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if (n_fields != 0)
+  if (n_fields > GD_MAX_LINCOM || n_fields < 0)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_ENTRY, GD_E_ENTRY_NFIELDS, NULL, n_fields,
+        NULL);
+  else if (n_fields != 0)
     N.EN(lincom,n_fields) = n_fields;
   else {
-    gd_entry_t *E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1,
-        NULL);
+    gd_entry_t *E = _GD_FindEntry(D, field_code);
 
-    if (E == NULL) {
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-      dreturn("%i", -1);
-      return -1;
-    }
+    if (E == NULL)
+      GD_RETURN_ERROR(D);
 
     N.EN(lincom,n_fields) = E->EN(lincom,n_fields);
   }
@@ -1253,11 +1245,7 @@ int gd_alter_linterp(DIRFILE* D, const char* field_code, const char* in_field,
   dtrace("%p, \"%s\", \"%s\", \"%s\", %i", D, field_code, in_field, table,
       move);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_LINTERP_ENTRY;
@@ -1279,11 +1267,7 @@ int gd_alter_bit(DIRFILE* D, const char* field_code, const char* in_field,
   dtrace("%p, \"%s\", \"%s\", %i, %i", D, field_code, in_field, bitnum,
       numbits);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_BIT_ENTRY;
@@ -1308,11 +1292,7 @@ int gd_alter_sbit(DIRFILE* D, const char* field_code, const char* in_field,
   dtrace("%p, \"%s\", \"%s\", %i, %i", D, field_code, in_field, bitnum,
       numbits);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_SBIT_ENTRY;
@@ -1336,11 +1316,7 @@ int gd_alter_recip(DIRFILE* D, const char* field_code, const char* in_field,
 
   dtrace("%p, \"%s\", \"%s\", %g", D, field_code, in_field, dividend);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_RECIP_ENTRY;
@@ -1380,11 +1356,7 @@ int gd_alter_crecip89(DIRFILE* D, const char* field_code, const char* in_field,
       (cdividend == NULL) ? 0 : cdividend[0],
       (cdividend == NULL) ? 0 : cdividend[1]);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_RECIP_ENTRY;
@@ -1414,11 +1386,7 @@ static int _GD_AlterYoke(DIRFILE* D, gd_entype_t t, const char* field_code,
   dtrace("%p, 0x%X, \"%s\", \"%s\", \"%s\"", D, t, field_code, in_field1,
       in_field2);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = t;
@@ -1443,6 +1411,18 @@ int gd_alter_multiply(DIRFILE* D, const char* field_code, const char* in_field1,
   return _GD_AlterYoke(D, GD_MULTIPLY_ENTRY, field_code, in_field1, in_field2);
 }
 
+int gd_alter_indir(DIRFILE* D, const char* field_code, const char* in_field1,
+    const char* in_field2) gd_nothrow
+{
+  return _GD_AlterYoke(D, GD_INDIR_ENTRY, field_code, in_field1, in_field2);
+}
+
+int gd_alter_sindir(DIRFILE* D, const char* field_code, const char* in_field1,
+    const char* in_field2) gd_nothrow
+{
+  return _GD_AlterYoke(D, GD_SINDIR_ENTRY, field_code, in_field1, in_field2);
+}
+
 int gd_alter_phase(DIRFILE* D, const char* field_code, const char* in_field,
     gd_shift_t shift) gd_nothrow
 {
@@ -1451,11 +1431,7 @@ int gd_alter_phase(DIRFILE* D, const char* field_code, const char* in_field,
 
   dtrace("%p, \"%s\", \"%s\", %" PRId64, D, field_code, in_field, shift);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_PHASE_ENTRY;
@@ -1477,11 +1453,7 @@ int gd_alter_const(DIRFILE* D, const char* field_code, gd_type_t const_type)
 
   dtrace("%p, \"%s\", 0x%X", D, field_code, const_type);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_CONST_ENTRY;
@@ -1501,11 +1473,7 @@ int gd_alter_carray(DIRFILE* D, const char* field_code, gd_type_t const_type,
 
   dtrace("%p, \"%s\", 0x%X, %" PRIuSIZE, D, field_code, const_type, array_len);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_CARRAY_ENTRY;
@@ -1518,6 +1486,25 @@ int gd_alter_carray(DIRFILE* D, const char* field_code, gd_type_t const_type,
   return ret;
 }
 
+int gd_alter_sarray(DIRFILE* D, const char* field_code, size_t array_len)
+  gd_nothrow
+{
+  gd_entry_t N;
+
+  dtrace("%p, \"%s\", %" PRIuSIZE, D, field_code, array_len);
+
+  GD_RETURN_ERR_IF_INVALID(D);
+
+  memset(&N, 0, sizeof(gd_entry_t));
+  N.field_type = GD_SARRAY_ENTRY;
+  N.EN(scalar,array_len) = array_len;
+
+  _GD_Change(D, field_code, &N, 0);
+
+  dreturn("%i", D->error);
+  return D->error;
+}
+
 int gd_alter_polynom(DIRFILE* D, const char* field_code, int poly_ord,
     const char* in_field, const double* a) gd_nothrow
 {
@@ -1527,31 +1514,20 @@ int gd_alter_polynom(DIRFILE* D, const char* field_code, int poly_ord,
 
   dtrace("%p, \"%s\", %i, \"%s\", %p", D, field_code, poly_ord, in_field, a);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_POLYNOM_ENTRY;
-  if (poly_ord > GD_MAX_POLYORD || poly_ord < 0) {
-    _GD_SetError(D, GD_E_BAD_ENTRY, GD_E_ENTRY_POLYORD, NULL, poly_ord, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if (poly_ord != 0)
+  if (poly_ord > GD_MAX_POLYORD || poly_ord < 0)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_ENTRY, GD_E_ENTRY_POLYORD, NULL, poly_ord,
+        NULL);
+  else if (poly_ord != 0)
     N.EN(polynom,poly_ord) = poly_ord;
   else {
-    gd_entry_t *E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1,
-        NULL);
+    gd_entry_t *E = _GD_FindEntry(D, field_code);
 
-    if (E == NULL) {
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-      dreturn("%i", -1);
-      return -1;
-    }
+    if (E == NULL)
+      GD_RETURN_ERROR(D);
 
     N.EN(polynom,poly_ord) = E->EN(polynom,poly_ord);
   }
@@ -1582,32 +1558,21 @@ int gd_alter_cpolynom(DIRFILE* D, const char* field_code, int poly_ord,
 
   dtrace("%p, \"%s\", %i, \"%s\", %p", D, field_code, poly_ord, in_field, ca);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_POLYNOM_ENTRY;
   N.flags = GD_EN_COMPSCAL;
-  if (poly_ord > GD_MAX_POLYORD || poly_ord < 0) {
-    _GD_SetError(D, GD_E_BAD_ENTRY, GD_E_ENTRY_POLYORD, NULL, poly_ord, NULL);
-    dreturn("%i", -1);
-    return -1;
-  } else if (poly_ord != 0)
+  if (poly_ord > GD_MAX_POLYORD || poly_ord < 0)
+    GD_SET_RETURN_ERROR(D, GD_E_BAD_ENTRY, GD_E_ENTRY_POLYORD, NULL, poly_ord,
+        NULL);
+  else if (poly_ord != 0)
     N.EN(polynom,poly_ord) = poly_ord;
   else {
-    gd_entry_t *E = _GD_FindField(D, field_code, D->entry, D->n_entries, 1,
-        NULL);
+    gd_entry_t *E = _GD_FindEntry(D, field_code);
 
-    if (E == NULL) {
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
-      dreturn("%i", -1);
-      return -1;
-    }
+    if (E == NULL)
+      GD_RETURN_ERROR(D);
 
     N.EN(polynom,poly_ord) = E->EN(polynom,poly_ord);
   }
@@ -1631,7 +1596,7 @@ int gd_alter_cpolynom(DIRFILE* D, const char* field_code, int poly_ord,
 
 int gd_alter_window(DIRFILE* D, const char *field_code, const char *in_field,
     const char *check_field, gd_windop_t windop, gd_triplet_t threshold)
-gd_nothrow
+  gd_nothrow
 {
   int ret;
   gd_entry_t N;
@@ -1640,11 +1605,7 @@ gd_nothrow
       field_code, in_field, check_field, windop, threshold.r, threshold.u,
       threshold.i);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_WINDOW_ENTRY;
@@ -1661,8 +1622,7 @@ gd_nothrow
 }
 
 int gd_alter_mplex(DIRFILE* D, const char *field_code, const char *in_field,
-    const char *count_field, int count_val, int period)
-gd_nothrow
+    const char *count_field, int count_val, int period) gd_nothrow
 {
   int ret;
   gd_entry_t N;
@@ -1670,11 +1630,7 @@ gd_nothrow
   dtrace("%p, \"%s\", \"%s\", \"%s\", %i, %i", D, field_code, in_field,
       count_field, count_val, period);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
   memset(&N, 0, sizeof(gd_entry_t));
   N.field_type = GD_MPLEX_ENTRY;
@@ -1695,6 +1651,7 @@ static int _GD_AlterSpec(DIRFILE* D, const char* line, const char* parent,
 {
   char *outstring = NULL, *new_code;
   const char *tok_pos;
+  size_t len0;
   char *in_cols[MAX_IN_COLS];
   int n_cols, ret;
   gd_entry_t *N = NULL;
@@ -1702,20 +1659,10 @@ static int _GD_AlterSpec(DIRFILE* D, const char* line, const char* parent,
 
   dtrace("%p, \"%s\", \"%s\", \"%s\", %i", D, line, parent, name, move);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
+  GD_RETURN_ERR_IF_INVALID(D);
 
-  /* check access mode */
-  if ((D->flags & GD_ACCMODE) == GD_RDONLY) {
-    _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  _GD_ClearError(D);
+  if ((D->flags & GD_ACCMODE) == GD_RDONLY)
+    GD_SET_RETURN_ERROR(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
 
   /* start parsing */
   _GD_SimpleParserInit(D, name, &p);
@@ -1728,54 +1675,57 @@ static int _GD_AlterSpec(DIRFILE* D, const char* line, const char* parent,
 
   if (D->error) {
     free(outstring);
-    dreturn("%i", -1); /* tokeniser threw an error */
-    return -1;
+    GD_RETURN_ERROR(D); /* tokeniser threw an error */
   }
 
-  if (parent) {
-    N = _GD_FindField(D, parent, D->entry, D->n_entries, 1, NULL);
-    if (N == NULL)
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, parent);
-  } else {
-    N = _GD_FindField(D, in_cols[0], D->entry, D->n_entries, 1, NULL);
-    if (N == NULL)
-      _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, in_cols[0]);
-  }
+  if (parent)
+    N = _GD_FindEntry(D, parent);
+  else
+    N = _GD_FindEntry(D, in_cols[0]);
 
   if (D->error) {
     free(outstring);
-    dreturn("%i", -1);
-    return -1;
+    GD_RETURN_ERROR(D);
   }
 
   /* the parser will modifiy in_cols[0] if it contains a metafield code */
+  len0 = strlen(in_cols[0]);
   if (parent) {
-    if ((new_code = _GD_Malloc(D, strlen(parent) + strlen(in_cols[0]) + 2)))
-      sprintf(new_code, "%s/%s", parent, in_cols[0]);
-  } else
-    new_code = _GD_Strdup(D, in_cols[0]);
+    new_code = _GD_Malloc(D, N->e->len + len0 + 2);
+
+    if (new_code) {
+      memcpy(new_code, N->field, N->e->len);
+      new_code[N->e->len] = '/';
+      memcpy(new_code + N->e->len + 1, in_cols[0], len0 + 1);
+    }
+    len0 += N->e->len + 1;
+  } else {
+    new_code = _GD_Malloc(D, len0 + 1);
+    if (new_code)
+      memcpy(new_code, in_cols[0], len0 + 1);
+  }
 
   if (D->error) { /* malloc error */
     free(outstring);
-    dreturn("%i", -1);
-    return -1;
+    GD_RETURN_ERROR(D);
   }
 
   /* Let the parser compose the entry */
-  N = _GD_ParseFieldSpec(D, &p, n_cols, in_cols, parent ? N : NULL,
+  N = _GD_ParseFieldSpec(D, &p, n_cols, in_cols, len0, parent ? N : NULL,
       N->fragment_index, 0, 0, &outstring, tok_pos);
 
-  /* The parser will have re-applied the prefix and suffix, undo that */
-  free(N->field);
-  N->field = new_code;
-
   free(outstring);
 
   if (D->error) {
-    dreturn("%i", -1); /* field spec parser threw an error */
-    return -1;
+    free(new_code);
+    GD_RETURN_ERROR(D); /* field spec parser threw an error */
   }
 
+  /* The parser will have re-applied the prefix and suffix, undo that */
+  free(N->field);
+  N->field = new_code;
+  N->e->len = len0;
+
   if (N->field_type == GD_LINCOM_ENTRY || N->field_type == GD_POLYNOM_ENTRY)
     move = 7;
 
