@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -26,7 +26,8 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
 {
   int ret, format_trunc = 0;
   DIR* dir;
-  struct dirent *lamb, *result;
+  struct dirent *result;
+  struct dirent unused;
   struct stat statbuf;
   size_t dirent_len = offsetof(struct dirent, d_name);
   size_t dirfile_len = strlen(dirfile);
@@ -71,34 +72,27 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
     return -1;
   }
 
-  if ((lamb = _GD_Malloc(D, dirent_len)) == NULL) {
-    closedir(dir);
-    dreturn("%i", -1);
-    return -1;
-  }
-
-  ret = _GD_ReadDir(dir, lamb, &result);
-  for (; result; ret = _GD_ReadDir(dir, lamb, &result)) {
+  ret = _GD_ReadDir(dir, &unused, &result);
+  for (; result; ret = _GD_ReadDir(dir, &unused, &result)) {
     char *name;
-    if (lamb->d_name[0] == '.' && lamb->d_name[1] == '\0')
+    if (result->d_name[0] == '.' && result->d_name[1] == '\0')
       continue; /* skip current dir */
-    else if (lamb->d_name[0] == '.' && lamb->d_name[1] == '.' &&
-        lamb->d_name[2] == '\0')
+    else if (result->d_name[0] == '.' && result->d_name[1] == '.' &&
+        result->d_name[2] == '\0')
     {
       continue; /* skip parent dir */
     }
 
-    name = _GD_Malloc(D, dirfile_len + strlen(lamb->d_name) + 2);
+    name = _GD_Malloc(D, dirfile_len + strlen(result->d_name) + 2);
     if (name == NULL) {
-      free(lamb);
       closedir(dir);
       dreturn("%i", -1);
       return -1;
     }
-    sprintf(name, "%s%c%s", dirfile, GD_DIRSEP, lamb->d_name);
+    sprintf(name, "%s%c%s", dirfile, GD_DIRSEP, result->d_name);
     if (
 #if defined(HAVE_FSTATAT) && !defined(GD_NO_DIR_OPEN)
-        fstatat(dirfd, lamb->d_name, &statbuf, AT_SYMLINK_NOFOLLOW)
+        fstatat(dirfd, result->d_name, &statbuf, AT_SYMLINK_NOFOLLOW)
 #elif HAVE_LSTAT
         lstat(name, &statbuf)
 #else
@@ -128,19 +122,18 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
 #ifdef S_IFLNK
       case S_IFLNK:
 #endif
-        if (root && strcmp(lamb->d_name, "format") == 0) {
+        if (root && strcmp(result->d_name, "format") == 0) {
           /* don't delete the format file; we'll truncate it later */
           format_trunc = 1;
         } else if (
 #ifdef HAVE_UNLINKAT
-            unlinkat(dirfd, lamb->d_name, 0)
+            unlinkat(dirfd, result->d_name, 0)
 #else
             unlink(name)
 #endif
             )
         {
           _GD_SetError(D, GD_E_IO, GD_E_IO_UNLINK, name, 0, NULL);
-          free(lamb);
           free(name);
           closedir(dir);
           dreturn("%i", -1);
@@ -156,14 +149,13 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
 #else
           if ((
 #ifdef HAVE_OPENAT
-                subdirfd = openat(dirfd, lamb->d_name, O_RDONLY)
+                subdirfd = openat(dirfd, result->d_name, O_RDONLY)
 #else
                 subdirfd = open(name, O_RDONLY)
 #endif
               ) < 0)
           {
             _GD_SetError(D, GD_E_IO, 0, name, 0, NULL);
-            free(lamb);
             closedir(dir);
             free(name);
             dreturn("%i", -1);
@@ -176,13 +168,12 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
           /* delete */
           if (
 #ifdef HAVE_UNLINKAT
-              unlinkat(dirfd, lamb->d_name, AT_REMOVEDIR)
+              unlinkat(dirfd, result->d_name, AT_REMOVEDIR)
 #else
               rmdir(name)
 #endif
              ) {
             _GD_SetError(D, GD_E_IO, GD_E_IO_UNLINK, name, 0, NULL);
-            free(lamb);
             free(name);
             closedir(dir);
             dreturn("%i", -1);
@@ -192,7 +183,6 @@ static int _GD_TruncDir(DIRFILE *D, int dirfd, const char *dirfile, int root)
     }
     free(name);
   }
-  free(lamb);
 
   closedir(dir);
 
@@ -248,7 +238,7 @@ static FILE *_GD_CreateDirfile(DIRFILE *restrict D, int dirfd, int dir_error,
   /* Error reading the format file, and we weren't asked to create it; do
    * nothing else. */
   if (format_error == EACCES || (format_error && !(D->flags & GD_CREAT))) {
-    char *format_file = (char *)malloc(strlen(dirfile) + 8);
+    char *format_file = malloc(strlen(dirfile) + 8);
     sprintf(format_file, "%s%cformat", dirfile, GD_DIRSEP);
     _GD_SetError2(D, GD_E_IO, GD_E_IO_OPEN, format_file, 0, NULL, format_error);
     free(format_file);
@@ -332,7 +322,7 @@ static FILE *_GD_CreateDirfile(DIRFILE *restrict D, int dirfd, int dir_error,
     if ((fd = gd_OpenAt(D, dirfd, "format", O_RDWR | O_CREAT | O_BINARY |
             (format_trunc ? O_TRUNC : O_EXCL), 0666)) < 0)
     {
-      char *format_file = (char *)malloc(strlen(dirfile) + 8);
+      char *format_file = malloc(strlen(dirfile) + 8);
       sprintf(format_file, "%s/format", dirfile);
       _GD_SetError(D, GD_E_CREAT, GD_E_CREAT_FORMAT, format_file, 0, NULL);
       free(dirfile);
@@ -352,7 +342,7 @@ static FILE *_GD_CreateDirfile(DIRFILE *restrict D, int dirfd, int dir_error,
 
   /* associate a stream with the format file */
   if ((fp = fdopen(fd, "rb")) == NULL) {
-    char *format_file = (char *)malloc(strlen(dirfile) + 8);
+    char *format_file = malloc(strlen(dirfile) + 8);
     sprintf(format_file, "%s/format", dirfile);
     _GD_SetError(D, GD_E_CREAT, GD_E_CREAT_FORMAT, format_file, 0, NULL);
     free(dirfile);
@@ -367,7 +357,7 @@ static FILE *_GD_CreateDirfile(DIRFILE *restrict D, int dirfd, int dir_error,
   }
 
   /* open succeeds */
-  D->dir = (struct gd_dir_t *)malloc(sizeof(struct gd_dir_t));
+  D->dir = malloc(sizeof(struct gd_dir_t));
   D->dir[0].fd = dirfd;
   D->dir[0].rc = 1;
   D->dir[0].path = dirfile;
@@ -398,7 +388,7 @@ DIRFILE* gd_invalid_dirfile(void) gd_nothrow
 
   dtracevoid();
 
-  D = (DIRFILE *)malloc(sizeof(DIRFILE));
+  D = malloc(sizeof(DIRFILE));
   if (D) {
     memset(D, 0, sizeof(DIRFILE));
     D->flags = GD_INVALID;
@@ -410,7 +400,7 @@ DIRFILE* gd_invalid_dirfile(void) gd_nothrow
 
 /* _GD_Open: open (or, perhaps, create) and parse the specified dirfile
 */
-DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
+static DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
     unsigned long flags, gd_parser_callback_t sehandler, void *extra)
 {
   FILE *fp;
@@ -454,7 +444,7 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
   _GD_InitialiseFramework();
 
   if (D == NULL)
-    D = (DIRFILE *)malloc(sizeof(DIRFILE));
+    D = malloc(sizeof(DIRFILE));
   if (D == NULL) {
     free(dirfile);
 #ifndef GD_NO_DIR_OPEN
@@ -468,6 +458,7 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
 
   /* user specified flags (used if we're forced to re-open) */
   D->open_flags = flags;
+
   /* clear GD_PERMISSIVE if it was specified along with GD_PEDANTIC */
   if (flags & GD_PERMISSIVE && flags & GD_PEDANTIC)
     flags &= ~GD_PERMISSIVE;
@@ -519,6 +510,7 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
   }
   memset(D->entry[0]->e, 0, sizeof(struct gd_private_entry_));
   D->entry[0]->flags = GD_EN_CALC;
+  D->entry[0]->e->len = 5;
 
   /* open the format file (or create it) */
   if ((fp = _GD_CreateDirfile(D, dirfd, dirfd_error, dirfile, &mtime)) == NULL)
@@ -543,7 +535,7 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
     return D;
   }
 
-  /* Parse the file.  This will take care of any necessary inclusions */
+  /* Build the root fragment metadata */
   D->n_fragment = 1;
 
   D->fragment = _GD_Malloc(D, sizeof(*D->fragment));
@@ -567,8 +559,7 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
     return D;
   }
   D->fragment[0].sname = NULL;
-  /* The root format file needs no external name */
-  D->fragment[0].ename = NULL;
+  D->fragment[0].ename = NULL; /* The root format file needs no external name */
   D->fragment[0].enc_data = NULL;
   D->fragment[0].modified = 0;
   D->fragment[0].parent = -1;
@@ -587,9 +578,8 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
   D->fragment[0].protection = GD_PROTECT_NONE;
   D->fragment[0].vers = (flags & GD_PEDANTIC) ? GD_DIRFILE_STANDARDS_VERSION :
     0;
-  D->fragment[0].suffix = D->fragment[0].prefix = NULL;
-  D->fragment[0].ns = NULL;
-  D->fragment[0].nsl = 0;
+  D->fragment[0].px = D->fragment[0].sx = D->fragment[0].ns = NULL;
+  D->fragment[0].pxl = D->fragment[0].sxl = D->fragment[0].nsl = 0;
 
   /* parser proto-state */
   p.line = 0;
@@ -599,8 +589,11 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
   p.flags = D->flags;
   p.ns = NULL;
   p.nsl = 0;
+
+  /* Parser invocation */
   ref_name = _GD_ParseFragment(fp, D, &p, 0, 1);
   fclose(fp);
+  free(p.ns);
 
   if (D->error != GD_E_OK) {
     dreturn("%p", D);
@@ -616,7 +609,8 @@ DIRFILE *_GD_Open(DIRFILE *D, int dirfd, const char *filedir,
 
   /* Find the reference field */
   if (ref_name != NULL) {
-    E = _GD_FindField(D, ref_name, D->entry, D->n_entries, 1, NULL);
+    E = _GD_FindField(D, ref_name, strlen(ref_name), D->entry, D->n_entries, 1,
+        NULL);
     if (E == NULL)
       _GD_SetError(D, GD_E_BAD_REFERENCE, GD_E_REFERENCE_CODE, NULL, 0,
           ref_name);
@@ -674,5 +668,93 @@ DIRFILE* gd_open(const char* filedir, unsigned long flags)
   dreturn("%p", D);
   return D;
 }
+
+/* returns non-zero if the metadata has changed on disk since the dirfile was
+ * opened and, optionally, re-opens the dirfile.
+ */
+int gd_desync(DIRFILE *D, unsigned int flags)
+{
+  int changed = 0, i;
+  struct stat statbuf;
+
+  dtrace("%p, 0x%x", D, flags);
+
+  GD_RETURN_ERR_IF_INVALID(D);
+
+  /* if we can't open directories, we're stuck with the full path method */
+#ifdef GD_NO_DIR_OPEN
+  flags |= GD_DESYNC_PATHCHECK;
+#endif
+
+  for (i = 0; i < D->n_fragment; ++i) {
+    if (flags & GD_DESYNC_PATHCHECK) {
+      /* stat the file via it's path relative to the original filedir */
+      char *buffer;
+      if (D->fragment[i].sname) {
+        buffer = _GD_Malloc(D, strlen(D->name) + strlen(D->fragment[i].bname) +
+            strlen(D->fragment[i].sname) + 3);
+        if (buffer == NULL)
+          GD_RETURN_ERROR(D);
+        sprintf(buffer, "%s%c%s%c%s", D->name, GD_DIRSEP, D->fragment[i].sname,
+            GD_DIRSEP, D->fragment[i].bname);
+      } else {
+        buffer = _GD_Malloc(D, strlen(D->name) + strlen(D->fragment[i].bname) +
+            2);
+        if (buffer == NULL)
+          GD_RETURN_ERROR(D);
+        sprintf(buffer, "%s%c%s", D->name, GD_DIRSEP, D->fragment[i].bname);
+      }
+      if (stat(buffer, &statbuf)) {
+        _GD_SetError(D, GD_E_IO, 0, buffer, 0, NULL);
+        free(buffer);
+        GD_RETURN_ERROR(D);
+      }
+      free(buffer);
+    } else
+      /* stat the file based on it's name and our cached dirfd */
+      if (gd_StatAt(D, D->fragment[i].dirfd, D->fragment[i].bname, &statbuf, 0))
+        GD_SET_RETURN_ERROR(D, GD_E_IO, 0, D->fragment[i].cname, 0, NULL);
+
+    if (statbuf.st_mtime != D->fragment[i].mtime) {
+      changed = 1;
+      break;
+    }
+  }
+
+  if (changed && flags & GD_DESYNC_REOPEN) {
+    /* reopening is easy: just delete everything and start again.  In the
+     * non-PATHCHECK case, we also have to cache the dirfd to the root directory
+     */
+
+    /* remember how we were called */
+    char *name = D->name;
+    gd_parser_callback_t sehandler = D->sehandler;
+    void *extra = D->sehandler_extra;
+    unsigned long int flags = D->open_flags;
+    int dirfd = -1;
+
+    if (!(flags & GD_DESYNC_PATHCHECK)) {
+      dirfd = dup(D->fragment[0].dirfd);
+      if (dirfd == -1)
+        GD_SET_RETURN_ERROR(D, GD_E_IO, GD_E_OPEN, D->name, 0, NULL);
+    }
+
+    D->name = NULL; /* so FreeD doesn't delete it */
+    if (_GD_ShutdownDirfile(D, 0, 1)) {
+      D->name = name;
+      if (dirfd >= 0)
+        close(dirfd);
+      GD_RETURN_ERROR(D);
+    }
+    _GD_Open(D, dirfd, name, flags, sehandler, extra);
+    free(name);
+
+    if (D->error)
+      changed = D->error;
+  }
+
+  dreturn("%i", changed);
+  return changed;
+}
 /* vim: ts=2 sw=2 et tw=80
 */
