@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 C. Barth Netterfield
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -50,21 +50,21 @@ static gd_type_t _GD_RawType(const char* type, int standards, int pedantic)
     else if (strcmp(type, "UINT64") == 0)
       t = GD_UINT64;
   } else if (type[0] == 'F') {
-    if (strcmp(type, "FLOAT32") == 0)
+    if (strcmp(type, "FLOAT64") == 0)
+      t = GD_FLOAT64;
+    else if (strcmp(type, "FLOAT32") == 0)
       t = GD_FLOAT32;
     else if (strcmp(type, "FLOAT") == 0)
       t = GD_FLOAT32;
-    else if (strcmp(type, "FLOAT64") == 0)
-      t = GD_FLOAT64;
   } else if (strcmp(type, "DOUBLE") == 0)
     t = GD_FLOAT64;
   else if (pedantic && standards < 7)
     t = GD_UNKNOWN;
 
-  else if (strcmp(type, "COMPLEX64") == 0)
-    t = GD_COMPLEX64;
   else if (strcmp(type, "COMPLEX128") == 0)
     t = GD_COMPLEX128;
+  else if (strcmp(type, "COMPLEX64") == 0)
+    t = GD_COMPLEX64;
 
   dreturn("0x%X", t);
   return t;
@@ -77,31 +77,31 @@ static gd_windop_t _GD_WindOp(const char *op)
 
   switch (op[0]) {
     case 'E':
-      if (op[1] == 'Q')
+      if (op[1] == 'Q' && op[2] == 0)
         o = GD_WINDOP_EQ;
       break;
     case 'L':
-      if (op[1] == 'T')
+      if (op[1] == 'T' && op[2] == 0)
         o = GD_WINDOP_LT;
-      else if (op[1] == 'E')
+      else if (op[1] == 'E' && op[2] == 0)
         o = GD_WINDOP_LE;
       break;
     case 'G':
-      if (op[1] == 'T')
+      if (op[1] == 'T' && op[2] == 0)
         o = GD_WINDOP_GT;
-      else if (op[1] == 'E')
+      else if (op[1] == 'E' && op[2] == 0)
         o = GD_WINDOP_GE;
       break;
     case 'N':
-      if (op[1] == 'E')
+      if (op[1] == 'E' && op[2] == 0)
         o = GD_WINDOP_NE;
       break;
     case 'S':
-      if (op[1] == 'E' && op[2] == 'T')
+      if (op[1] == 'E' && op[2] == 'T' && op[3] == 0)
         o = GD_WINDOP_SET;
       break;
     case 'C':
-      if (op[1] == 'L' && op[2] == 'R')
+      if (op[1] == 'L' && op[2] == 'R' && op[3] == 0)
         o = GD_WINDOP_CLR;
       break;
   }
@@ -216,7 +216,7 @@ int _GD_TokToNum(const char *restrict token, int standards, int pedantic,
         *im = ui;
     }
   } else if (u) { /* unsigned int -- reject negative values */
-    if (rt == GD_UINT64) 
+    if (rt == GD_UINT64)
       *u = ur;
     else if (rt == GD_INT64) {
       if (ir < 0) {
@@ -249,12 +249,12 @@ int _GD_TokToNum(const char *restrict token, int standards, int pedantic,
 /* Compose a subfield code from a parent code and a subfield name -- this
  * used to be done in _GD_MungeCode */
 static char *_GD_SubfieldCode(DIRFILE *D, const gd_entry_t *P, const char *code,
-    char **nso, int *offset)
+    size_t *offset)
 {
   size_t len, len_par;
   char *new_code;
 
-  dtrace("%p, %p, \"%s\", %p, %p", D, P, code, nso, offset);
+  dtrace("%p, %p, \"%s\", %p", D, P, code, offset);
 
   if (code == NULL) {
     dreturn("%p", NULL);
@@ -276,54 +276,31 @@ static char *_GD_SubfieldCode(DIRFILE *D, const gd_entry_t *P, const char *code,
   if (offset)
     *offset = len_par;
 
-  /* find the namespace, if necessary */
-  if (nso) {
-    size_t i;
-    for (i = strlen(new_code); i > 0; --i)
-      if (new_code[i - 1] == ',')
-        break;
-
-    *nso = new_code + i;
-  }
-
-  dreturn("\"%s\" (%i, %p)", new_code, offset ? *offset : -1,
-      nso ? *nso : NULL);
+  dreturn("\"%s\" (%" PRIuSIZE ")", new_code, offset ? *offset : 0);
   return new_code;
 }
 
 /* Create a field code using the prefix and suffix of the given fragment and the
  * current namespace.
  *
- * Returns a newly malloc'd code, or NULL on error.  nso points to the portion
- * of the code immediately after the namespace prefix.  Offset is the offset
- * from the start of the code to the field name (ie. the character following
- * / for a metafield. */
+ * Returns a newly malloc'd code, or NULL on error.  For subfields, offset is
+ * the offset to the character following / for a metafield.  For non-subfields,
+ * it's the length of the namespace. */
 static char *_GD_CodeFromFrag(DIRFILE *restrict D,
-    const struct parser_state *restrict p, const gd_entry_t *restrict P, int me,
-    const char *code, char **nso, int *offset)
+    const struct parser_state *restrict p, const gd_entry_t *restrict P,
+    int me, const char *code, size_t *offset)
 {
   char *new_code;
 
   dtrace("%p, %p, %p, %i, \"%s\", %p", D, p, P, me, code, offset);
 
   if (P)
-    new_code = _GD_SubfieldCode(D, P, code, nso, offset);
-  else {
-    const char *ns = p->ns;
-    size_t nsl = p->nsl;
+    new_code = _GD_SubfieldCode(D, P, code, offset);
+  else
+    new_code = _GD_BuildCode(D, me, p->ns, p->nsl, code,
+        p->pedantic && p->standards < 10, offset);
 
-    /* set fragment root space, if necessary */
-    if (!ns) {
-      ns = D->fragment[me].ns;
-      nsl = D->fragment[me].nsl;
-    }
-
-    new_code = _GD_MungeCode(D, ns, nsl, NULL, NULL, D->fragment[me].prefix,
-        D->fragment[me].suffix, code, nso, offset,
-        GD_MC_RQ_PARTS | GD_MC_ERROR_OK | GD_MC_NO_NS);
-  }
-
-  dreturn("\"%s\"", new_code);
+  dreturn("\"%s\" (%" PRIuSIZE ")", new_code, offset ? *offset : 0);
   return new_code;
 }
 
@@ -333,43 +310,42 @@ static char *_GD_InputCode(DIRFILE *D, const struct parser_state *restrict p,
     int me, const char *token)
 {
   char *code;
-  const char *ns = p->ns;
-  size_t nsl = p->nsl;
-  unsigned flags = GD_MC_RQ_PARTS | GD_MC_ERROR_OK;
 
   dtrace("%p, %p, %i, \"%s\"", D, p, me, token);
 
-  if (token[0] == '.') { /* absolute name */
-    token++;
-    ns = D->fragment[me].ns;
-    nsl = D->fragment[me].nsl;
-  } else if (ns == NULL) {
-    ns = D->fragment[me].ns;
-    nsl = D->fragment[me].nsl;
-  }
-
-  code = _GD_MungeCode(D, ns, nsl, NULL, NULL, D->fragment[me].prefix,
-      D->fragment[me].suffix, token, NULL, NULL, flags);
+  code = _GD_BuildCode(D, me, p->ns, p->nsl, token,
+      p->pedantic && p->standards <= 5, NULL);
 
   dreturn("\"%s\"", code);
   return code;
 }
-  
+
 /* Create E->field; frees the entry and returns non-zero on error. */
 static int _GD_SetField(DIRFILE *restrict D,
     const struct parser_state *restrict p, gd_entry_t *restrict E,
-    const gd_entry_t *restrict P, int me, const char *restrict name, int no_dot)
+    const gd_entry_t *restrict P, int me, const char *restrict name)
 {
-  int offset;
-  int is_dot = 0;
+  size_t offset;
 
-  dtrace("%p, %p{%s,%i}, %p, %p, %i, \"%s\", %i", D, p, p->file, p->line, E, P, me, name, no_dot);
+  dtrace("%p, %p{%s,%i}, %p, %p, %i, \"%s\"", D, p, p->file, p->line, E, P, me,
+      name);
 
-  E->field = _GD_CodeFromFrag(D, p, P, me, name, NULL, &offset);
-  if (E->field && _GD_ValidateField(E->field + offset, p->standards,
-        p->pedantic, GD_VF_NAME, (p->ns || no_dot) ? NULL : &is_dot))
-  {
-    _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line, name);
+  E->field = _GD_CodeFromFrag(D, p, P, me, name, &offset);
+
+  /* The _GD_ValidateField call differs based on whether we're dealing with
+   * a subfield or not because the meaning of offset is different */
+  if (E->field) {
+    int bad_code;
+    if (P)
+      bad_code = _GD_ValidateField(E->field + offset, 0, p->standards,
+          p->pedantic, GD_VF_NAME);
+    else
+      bad_code = _GD_ValidateField(E->field, offset, p->standards, p->pedantic,
+          GD_VF_NAME);
+
+    if (bad_code)
+      _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line,
+          name);
   }
 
   if (D->error) {
@@ -377,10 +353,8 @@ static int _GD_SetField(DIRFILE *restrict D,
     dreturn("%i", 1);
     return 1;
   }
- 
-  if (is_dot)
-    E->flags |= GD_EN_DOTTED;
 
+  E->e->len = strlen(E->field);
   dreturn("%i", 0);
   return 0;
 }
@@ -404,12 +378,8 @@ static char *_GD_SetScalar(DIRFILE *restrict D,
 
     i = _GD_TokToNum(token, p->standards, p->pedantic, &re, &im, NULL, NULL);
 
-    if (i == -2) { /* malformed number */
-      _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_LITERAL, p->file, p->line,
-          token);
-      dreturn("%p", NULL);
-      return NULL;
-    } else if (i == -1) { /* assume it's a field name */
+    /* _GD_TokToNum can't return -2 for complex return type */
+    if (i == -1) { /* assume it's a field name */
       ptr = _GD_InputCode(D, p, me, token);
       if (D->error) {
         dreturn("%p", NULL);
@@ -538,7 +508,7 @@ carray_check:
         break;
     }
   }
-  
+
   dreturn("\"%s\" (%i)", ptr, *index);
   return ptr;
 }
@@ -588,16 +558,19 @@ static gd_entry_t *_GD_ParseRaw(DIRFILE *restrict D,
   E->e->u.raw.file[0].subenc = GD_ENC_UNKNOWN; /* don't know the encoding
                                                     subscheme yet */
 
-  if (_GD_SetField(D, p, E, NULL, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, NULL, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
 
+  if (p->pedantic && p->standards <= 5)
+    E->flags |= GD_EN_EARLY;
+
   E->e->u.raw.filebase = _GD_Strdup(D, in_cols[0]);
   E->EN(raw,data_type) = _GD_RawType(in_cols[2], p->standards, p->pedantic);
   E->e->u.raw.size = GD_SIZE(E->EN(raw,data_type));
 
-  if (E->e->u.raw.size == 0 || E->EN(raw,data_type) & 0x40)
+  if (E->e->u.raw.size == 0)
     _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_TYPE, p->file, p->line,
         in_cols[2]);
   else if ((E->scalar[0] = _GD_SetScalar(D, p, in_cols[3], &E->EN(raw,spf),
@@ -655,7 +628,7 @@ static gd_entry_t *_GD_ParseLincom(DIRFILE *restrict D,
 
   E->field_type = GD_LINCOM_ENTRY;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -749,7 +722,7 @@ static gd_entry_t *_GD_ParseLinterp(DIRFILE *restrict D,
   E->flags |= GD_EN_CALC;
   E->EN(linterp,table) = NULL;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -769,7 +742,7 @@ static gd_entry_t *_GD_ParseLinterp(DIRFILE *restrict D,
 }
 
 /* _GD_ParseYoke: parse a field specified by two input fields only (MULTIPLY,
- * DIVIDE)
+ * DIVIDE, INDIR, SINDIR)
 */
 static gd_entry_t *_GD_ParseYoke(DIRFILE *restrict D, gd_entype_t type,
     const struct parser_state *restrict p,
@@ -808,7 +781,7 @@ static gd_entry_t *_GD_ParseYoke(DIRFILE *restrict D, gd_entype_t type,
   E->e->entry[0] = E->e->entry[1] = NULL;
   E->flags |= GD_EN_CALC;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -862,7 +835,7 @@ static gd_entry_t *_GD_ParseRecip(DIRFILE *restrict D,
   E->in_fields[0] = NULL;
   E->e->entry[0] = NULL;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -917,7 +890,7 @@ static gd_entry_t *_GD_ParseWindow(DIRFILE *restrict D,
 
   E->field_type = GD_WINDOW_ENTRY;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -995,7 +968,7 @@ static gd_entry_t *_GD_ParseMplex(DIRFILE *restrict D,
 
   E->field_type = GD_MPLEX_ENTRY;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1063,7 +1036,7 @@ static gd_entry_t *_GD_ParseBit(DIRFILE *restrict D, int is_signed,
   E->e->entry[0] = NULL;
   E->flags |= GD_EN_CALC;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1137,7 +1110,7 @@ static gd_entry_t *_GD_ParsePhase(DIRFILE *restrict D,
   E->in_fields[0] = NULL;
   E->e->entry[0] = NULL;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1194,7 +1167,7 @@ static gd_entry_t *_GD_ParsePolynom(DIRFILE *restrict D,
   memset(E->e, 0, sizeof(struct gd_private_entry_));
 
   E->field_type = GD_POLYNOM_ENTRY;
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1254,6 +1227,7 @@ gd_type_t _GD_ConstType(DIRFILE *D, gd_type_t type)
       return GD_COMPLEX128;
     case GD_NULL:
     case GD_UNKNOWN:
+    case GD_STRING:
       _GD_InternalError(D);
   }
 
@@ -1300,7 +1274,7 @@ static gd_entry_t *_GD_ParseConst(DIRFILE *restrict D,
   E->field_type = GD_CONST_ENTRY;
   E->flags |= GD_EN_CALC;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1345,7 +1319,7 @@ static gd_entry_t *_GD_ParseConst(DIRFILE *restrict D,
 
 /* _GD_ParseArray: parse [CS]ARRAY entry in formats file.
 */
-static gd_entry_t *_GD_ParseArray(DIRFILE *restrict D,
+static gd_entry_t *_GD_ParseArray(DIRFILE *restrict D, int string,
     const struct parser_state *restrict p, char *in_cols[MAX_IN_COLS],
     int n_cols, const gd_entry_t *restrict parent, int me, char **outstring,
     const char *tok_pos)
@@ -1357,10 +1331,11 @@ static gd_entry_t *_GD_ParseArray(DIRFILE *restrict D,
   void *data = NULL;
   gd_entry_t *E;
 
-  dtrace("%p, %p, %p, %i, %p, %i, %p, %p", D, p, in_cols, n_cols, parent, me,
-      outstring, tok_pos);
+  dtrace("%p, %i, %p, %p, %i, %p, %i, %p, %p", D, string, p, in_cols, n_cols,
+      parent, me, outstring, tok_pos);
 
-  if (n_cols < 4) {
+  /* CARRAYs have a data_type token which SARRAYs lack */
+  if (n_cols < 4 - string) {
     _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_N_TOK, p->file, p->line, NULL);
     dreturn("%p", NULL);
     return NULL;
@@ -1382,26 +1357,31 @@ static gd_entry_t *_GD_ParseArray(DIRFILE *restrict D,
   }
   memset(E->e, 0, sizeof(struct gd_private_entry_));
 
-  E->field_type = GD_CARRAY_ENTRY;
+  E->field_type = string ? GD_SARRAY_ENTRY : GD_CARRAY_ENTRY;
   E->flags |= GD_EN_CALC;
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
 
-  E->EN(scalar,const_type) = _GD_RawType(in_cols[2], p->standards,
-      p->pedantic);
-  t = _GD_ConstType(D, E->EN(scalar,const_type));
-  first = 3;
-  s = GD_SIZE(t);
+  if (string) {
+    s = sizeof(const char *);
+    first = 2;
+  } else {
+    E->EN(scalar,const_type) = _GD_RawType(in_cols[2], p->standards,
+        p->pedantic);
+    t = _GD_ConstType(D, E->EN(scalar,const_type));
+    first = 3;
+    s = GD_SIZE(t);
 
-  if (GD_SIZE(E->EN(scalar,const_type)) == 0 || E->EN(raw,data_type) & 0x40) {
-    _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_TYPE, p->file, p->line,
-        in_cols[2]);
-    _GD_FreeE(D, E, 1);
-    dreturn("%p", NULL);
-    return NULL;
+    if (GD_SIZE(E->EN(scalar,const_type)) == 0 || E->EN(raw,data_type) & 0x40) {
+      _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_TYPE, p->file, p->line,
+          in_cols[2]);
+      _GD_FreeE(D, E, 1);
+      dreturn("%p", NULL);
+      return NULL;
+    }
   }
 
   /* spool in the data */
@@ -1424,17 +1404,32 @@ static gd_entry_t *_GD_ParseArray(DIRFILE *restrict D,
       if (n == GD_SIZE_T_MAX)
         break;
 
-      ptr = _GD_SetScalar(D, p, in_cols[c], (char *)data + s * n++, t, me,
-          &offset, NULL);
+      if (string) {
+        ((char**)data)[n++] = _GD_Strdup(D, in_cols[c]);
 
-      if (ptr) {
-        free(ptr);
-        free(data);
-        _GD_FreeE(D, E, 1);
-        _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_LITERAL, p->file, p->line,
-            in_cols[c]);
-        dreturn("%p", NULL);
-        return NULL;
+        if (D->error) {
+          size_t i;
+
+          for (i = 0; i < n; ++i)
+            free(((char**)data)[i]);
+          free(data);
+          _GD_FreeE(D, E, 1);
+          dreturn("%p", NULL);
+          return NULL;
+        }
+      } else {
+        ptr = _GD_SetScalar(D, p, in_cols[c], (char *)data + s * n++, t, me,
+            &offset, NULL);
+
+        if (ptr) {
+          free(ptr);
+          free(data);
+          _GD_FreeE(D, E, 1);
+          _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_LITERAL, p->file, p->line,
+              in_cols[c]);
+          dreturn("%p", NULL);
+          return NULL;
+        }
       }
     }
 
@@ -1506,7 +1501,7 @@ static gd_entry_t *_GD_ParseString(DIRFILE *restrict D,
     return NULL;
   }
 
-  if (_GD_SetField(D, p, E, parent, me, in_cols[0], 0)) {
+  if (_GD_SetField(D, p, E, parent, me, in_cols[0])) {
     dreturn("%p", NULL);
     return NULL;
   }
@@ -1550,27 +1545,37 @@ static int _GD_UTF8Encode(DIRFILE *restrict D, const char *restrict format_file,
 /* _GD_CheckParent: look for a slashed field name and, if found, see if the
  * parent exists in the current fragment.  Returns parent entry on success,
  * and points *name to the metaname part.  me == -1 implies we're not in the
- * parser, but called from _GD_Add.
+ * parser, but called from _GD_Add.  In this case, it never sets D->error.
  */
 gd_entry_t *_GD_CheckParent(DIRFILE *restrict D,
-    const struct parser_state *p, char **restrict name, int me)
+    const struct parser_state *p, char **restrict name, size_t *len, int me)
 {
+  size_t plen = 0;
   char *cptr, *munged_code;
   gd_entry_t *P = NULL;
 
-  dtrace("%p, %p, \"%s\", %i", D, p, *name, me);
+  dtrace("%p, %p, \"%s\", %p, %i", D, p, *name, len, me);
 
   for (cptr = *name + 1; *cptr != '\0'; ++cptr)
     if (*cptr == '/') {
       *cptr = '\0';
-      if (me == -1)
-        munged_code = strdup(*name);
-      else
-        munged_code = _GD_CodeFromFrag(D, p, NULL, me, *name, NULL, NULL);
+      if (me == -1) {
+        plen = (cptr - *name);
+        munged_code = malloc(plen + 1);
+        if (munged_code)
+          memcpy(munged_code, *name, plen + 1);
+      } else {
+        munged_code = _GD_CodeFromFrag(D, p, NULL, me, *name, NULL);
+        if (munged_code)
+          plen = strlen(munged_code);
+      }
+
       if (munged_code) {
-        P = _GD_FindField(D, munged_code, D->entry, D->n_entries, 0, NULL);
+        P = _GD_FindField(D, munged_code, plen, D->entry, D->n_entries, 0,
+            NULL);
         free(munged_code);
       }
+
       if (P == NULL) {
         if (me == -1) {
           *cptr = '/'; /* undo field munging; _GD_Add will conclude this is
@@ -1596,11 +1601,12 @@ gd_entry_t *_GD_CheckParent(DIRFILE *restrict D,
       }
 
       /* point name to the metafield name */
+      *len -= (cptr - *name) + 1;
       *name = cptr + 1;
       break;
     }
 
-  dreturn("%p", P);
+  dreturn("%p (\"%s\", %" PRIuSIZE ")", P, *name, *len);
   return P;
 }
 
@@ -1608,24 +1614,21 @@ gd_entry_t *_GD_CheckParent(DIRFILE *restrict D,
  * specification */
 gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
     const struct parser_state *restrict p, int n_cols, char **in_cols,
-    const gd_entry_t *restrict P, int me, int creat, int insert,
+    size_t len0, const gd_entry_t *restrict P, int me, int creat, int insert,
     char **outstring, const char *tok_pos)
 {
   gd_entry_t* E = NULL;
   void *ptr;
 
-  dtrace("%p, %p, %i, %p, %p, %i, %i, %i, %p, %p", D, p, n_cols, in_cols, P, me,
-      creat, insert, outstring, tok_pos);
+  dtrace("%p, %p, %i, %p, %" PRIuSIZE ", %p, %i, %i, %i, %p, %p", D, p, n_cols,
+      in_cols, len0, P, me, creat, insert, outstring, tok_pos);
 
   /* Check for barth-style metafield definition */
   if (P == NULL && GD_PVERS_GE(*p, 7)) {
-    P = _GD_CheckParent(D, p, in_cols + 0, me);
+    P = _GD_CheckParent(D, p, in_cols + 0, &len0, me);
     if (P) {
-      if (n_cols < 2)
-        _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_N_TOK, p->file, p->line, NULL);
-      else
-        E = _GD_ParseFieldSpec(D, p, n_cols, in_cols, P, me, creat, insert,
-            outstring, tok_pos);
+      E = _GD_ParseFieldSpec(D, p, n_cols, in_cols, len0, P, me, creat, insert,
+          outstring, tok_pos);
       dreturn("%p", (!insert) ? E : NULL);
       return (!insert) ? E : NULL;
     }
@@ -1638,10 +1641,7 @@ gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
   }
   D->entry = (gd_entry_t **)ptr;
 
-  if (n_cols < 2)
-    _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_N_TOK, p->file, p->line,
-        NULL);
-  else if (P == NULL && (strcmp(in_cols[0], "INDEX") == 0 || (p->pedantic &&
+  if (P == NULL && (strcmp(in_cols[0], "INDEX") == 0 || (p->pedantic &&
           p->standards < 6 && strcmp(in_cols[0], "FILEFRAM") == 0)))
     /* reserved field name */
     _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_RES_NAME, p->file, p->line,
@@ -1656,7 +1656,7 @@ gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
         break;
       case 'C':
         if (strcmp(in_cols[1], "CARRAY") == 0 && GD_PVERS_GE(*p, 8))
-          E = _GD_ParseArray(D, p, in_cols, n_cols, P, me, outstring,
+          E = _GD_ParseArray(D, 0, p, in_cols, n_cols, P, me, outstring,
               tok_pos);
         else if (strcmp(in_cols[1], "CONST") == 0 && GD_PVERS_GE(*p, 6))
           E = _GD_ParseConst(D, p, in_cols, n_cols, P, me);
@@ -1669,6 +1669,12 @@ gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
         else
           goto NO_MATCH;
         break;
+      case 'I':
+        if (strcmp(in_cols[1], "INDIR") == 0 && GD_PVERS_GE(*p, 10))
+          E = _GD_ParseYoke(D, GD_INDIR_ENTRY, p, in_cols, n_cols, P, me);
+        else
+          goto NO_MATCH;
+        break;
       case 'L':
         if (strcmp(in_cols[1], "LINCOM") == 0)
           E = _GD_ParseLincom(D, p, in_cols, n_cols, P, me);
@@ -1703,16 +1709,14 @@ gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
             if (D->fragment[me].protection != GD_PROTECT_NONE)
               _GD_SetError(D, GD_E_PROTECTED, GD_E_PROTECTED_DATA, NULL, 0,
                   D->fragment[me].cname);
-            /* If the encoding scheme is unknown, we can't add the field */
-            if (D->fragment[me].encoding == GD_AUTO_ENCODED)
-              _GD_SetError(D, GD_E_UNKNOWN_ENCODING, GD_E_UNENC_UNDET, NULL, 0,
-                  NULL);
-            else if (D->fragment[me].encoding == GD_ENC_UNSUPPORTED)
-              /* If the encoding scheme is unsupported, can't add the field */
-              _GD_SetError(D, GD_E_UNSUPPORTED, 0, NULL, 0, NULL);
             else
               _GD_InitRawIO(D, E, NULL, -1, NULL, 0,
                   GD_FILE_WRITE | GD_FILE_TOUCH, _GD_FileSwapBytes(D, E));
+
+            if (D->error) {
+              _GD_FreeE(D, E, 1);
+              E = NULL;
+            }
           }
 
           /* Is this the first raw field ever defined? */
@@ -1727,6 +1731,11 @@ gd_entry_t *_GD_ParseFieldSpec(DIRFILE *restrict D,
       case 'S':
         if (strcmp(in_cols[1], "SBIT") == 0 && GD_PVERS_GE(*p, 7))
           E = _GD_ParseBit(D, 1, p, in_cols, n_cols, P, me);
+        else if (strcmp(in_cols[1], "SINDIR") == 0 && GD_PVERS_GE(*p, 2))
+          E = _GD_ParseYoke(D, GD_SINDIR_ENTRY, p, in_cols, n_cols, P, me);
+        else if (strcmp(in_cols[1], "SARRAY") == 0 && GD_PVERS_GE(*p, 10))
+          E = _GD_ParseArray(D, 1, p, in_cols, n_cols, P, me, outstring,
+              tok_pos);
         else if (strcmp(in_cols[1], "STRING") == 0 && GD_PVERS_GE(*p, 6))
           E = _GD_ParseString(D, p, in_cols, n_cols, P, me);
         else
@@ -1751,7 +1760,7 @@ NO_MATCH:
     unsigned int u;
 
     /* Check for duplicate */
-    if (_GD_FindField(D, E->field, D->entry, D->n_entries, 0, &u)) {
+    if (_GD_FindField(D, E->field, E->e->len, D->entry, D->n_entries, 0, &u)) {
       if (~p->flags & GD_IGNORE_DUPS)
         _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_DUPLICATE, p->file, p->line,
             E->field);
@@ -1760,16 +1769,6 @@ NO_MATCH:
       return NULL;
     }
 
-    if (E->flags & GD_EN_DOTTED) {
-      ptr = _GD_Realloc(D, D->dot_list, (D->n_dot + 1) * sizeof(gd_entry_t*));
-      if (ptr == NULL) {
-        _GD_FreeE(D, E, 1);
-        dreturn ("%p", NULL);
-        return NULL;
-      }
-      D->dot_list = (gd_entry_t **)ptr;
-    }
-
     /* Initialse the meta counts */
     if (P != NULL) {
       E->e->n_meta = -1;
@@ -1792,12 +1791,6 @@ NO_MATCH:
     /* the Format file fragment index */
     E->fragment_index = me;
 
-    /* update the dot list if necessary */
-    if (E->flags & GD_EN_DOTTED) {
-      D->dot_list[D->n_dot++] = E;
-      qsort(D->dot_list, D->n_dot, sizeof(gd_entry_t*), _GD_EntryCmp);
-    }
-
     /* sort */
     _GD_InsertSort(D, E, u);
     D->n_entries++;
@@ -2027,11 +2020,12 @@ int _GD_Tokenise(DIRFILE *restrict D, const struct parser_state *restrict p,
 }
 
 /* _GD_ParseAlias: set up an alias
-*/
+ */
 static void _GD_ParseAlias(DIRFILE *restrict D,
     const struct parser_state *restrict p, char **restrict name,
     const char *restrict target, int me)
 {
+  size_t len = strlen(*name);
   gd_entry_t **new_meta_list = NULL;
   gd_entry_t *E, *P = NULL;
   unsigned int u;
@@ -2039,7 +2033,7 @@ static void _GD_ParseAlias(DIRFILE *restrict D,
 
   dtrace("%p, %p, \"%s\", \"%s\", %i", D, p, *name, target, me);
 
-  P = _GD_CheckParent(D, p, name, me);
+  P = _GD_CheckParent(D, p, name, &len, me);
   if (D->error) {
     dreturnvoid();
     return;
@@ -2078,13 +2072,13 @@ static void _GD_ParseAlias(DIRFILE *restrict D,
     return;
   }
 
-  if (_GD_SetField(D, p, E, P, me, *name, 1)) {
+  if (_GD_SetField(D, p, E, P, me, *name)) {
     dreturnvoid();
     return;
   }
 
   /* Check for duplicate */
-  if (_GD_FindField(D, E->field, D->entry, D->n_entries, 0, &u)) {
+  if (_GD_FindField(D, E->field, E->e->len, D->entry, D->n_entries, 0, &u)) {
     if (!(p->flags & GD_IGNORE_DUPS))
       _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_DUPLICATE, p->file, p->line,
           E->field);
@@ -2122,6 +2116,54 @@ static void _GD_ParseAlias(DIRFILE *restrict D,
   dreturnvoid();
 }
 
+static void _GD_ParseNamespace(DIRFILE *D, struct parser_state *restrict p,
+    const char *ns)
+{
+  char *ptr;
+
+  dtrace("%p, %p, \"%s\"", D, p, ns);
+
+  /* Ignore a leading '.' -- namespaces given in a /NAMESPACE directive are
+   * always relative to the rootspace
+   */
+  if (ns[0] == '.')
+    ns++;
+
+  if (ns[0] == '\0') {
+    /* Revert to rootspace */
+    free(p->ns);
+    p->ns = NULL;
+    p->nsl = 0;
+    dreturnvoid();
+    return;
+  }
+
+  /* check new namespace name */
+  if (_GD_ValidateField(ns, 0, p->standards, p->pedantic, GD_VF_NS)) {
+    _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_BAD_NAME, p->file, p->line, ns);
+    dreturnvoid();
+    return;
+  }
+
+  ptr = _GD_Strdup(D, ns);
+  if (ptr == NULL) { /* alloc error */
+    dreturnvoid();
+    return;
+  }
+
+  free(p->ns);
+  p->ns = ptr;
+  p->nsl = strlen(ptr);
+
+  /* strip trailing dot */
+  if (p->ns[p->nsl - 1] == '.') {
+    p->ns[p->nsl - 1] = 0;
+    p->nsl--;
+  }
+
+  dreturn("(nil) [\"%s\", %" PRIuSIZE "]", p->ns, p->nsl);
+}
+
 /* _GD_ParseDirective: Actually parse a single format file line.
  *       Returns 1 if a match was made.
  */
@@ -2231,10 +2273,12 @@ static int _GD_ParseDirective(DIRFILE *D, struct parser_state *restrict p,
     case 'H':
       if (strcmp(ptr, "HIDDEN") == 0 && GD_PVERS_GE(*p, 9)) {
         matched = 1;
-        munged_code = _GD_CodeFromFrag(D, p, NULL, me, in_cols[1], NULL, NULL);
-        if (munged_code)
-          E = _GD_FindField(D, munged_code, D->entry, D->n_entries, 0, NULL);
-        free(munged_code);
+        munged_code = _GD_CodeFromFrag(D, p, NULL, me, in_cols[1], NULL);
+        if (munged_code) {
+          E = _GD_FindField(D, munged_code, strlen(munged_code), D->entry,
+              D->n_entries, 0, NULL);
+          free(munged_code);
+        }
 
         if (E == NULL)
           _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_NO_FIELD, p->file, p->line,
@@ -2277,9 +2321,10 @@ static int _GD_ParseDirective(DIRFILE *D, struct parser_state *restrict p,
     case 'M':
       if (strcmp(ptr, "META") == 0 && GD_PVERS_GE(*p, 6)) {
         matched = 1;
-        munged_code = _GD_CodeFromFrag(D, p, NULL, me, in_cols[1], NULL, NULL);
+        munged_code = _GD_CodeFromFrag(D, p, NULL, me, in_cols[1], NULL);
         if (munged_code) {
-          E = _GD_FindField(D, munged_code, D->entry, D->n_entries, 0, NULL);
+          E = _GD_FindField(D, munged_code, strlen(munged_code), D->entry,
+              D->n_entries, 0, NULL);
           free(munged_code);
         }
 
@@ -2299,8 +2344,14 @@ static int _GD_ParseDirective(DIRFILE *D, struct parser_state *restrict p,
           _GD_SetError(D, GD_E_FORMAT, GD_E_FORMAT_N_TOK, p->file, p->line,
               NULL);
         else
-          _GD_ParseFieldSpec(D, p, n_cols - 2, in_cols + 2, E, me, 0, 1,
-              outstring, tok_pos);
+          _GD_ParseFieldSpec(D, p, n_cols - 2, in_cols + 2, strlen(in_cols[2]),
+              E, me, 0, 1, outstring, tok_pos);
+      }
+      break;
+    case 'N':
+      if (strcmp(ptr, "NAMESPACE") == 0 && GD_PVERS_GE(*p, 10)) {
+        matched = 1;
+        _GD_ParseNamespace(D, p, in_cols[1]);
       }
       break;
     case 'P':
@@ -2343,40 +2394,43 @@ static int _GD_ParseDirective(DIRFILE *D, struct parser_state *restrict p,
 }
 
 /* Resolve and record an alias, taking care of loops */
-static gd_entry_t *_GD_ResolveAlias(DIRFILE *restrict D, gd_entry_t *restrict E)
+static gd_entry_t *_GD_ResolveAlias(DIRFILE *restrict D, const gd_entry_t *base,
+    gd_entry_t *E)
 {
   gd_entry_t *T = NULL;
 
-  dtrace("%p, %p", D, E);
-
-  if (++D->recurse_level >= GD_MAX_RECURSE_LEVEL) {
-    _GD_SetError(D, GD_E_RECURSE_LEVEL, GD_E_RECURSE_CODE, NULL, 0, E->field);
-    D->recurse_level--;
-    dreturn("%p", NULL);
-    return NULL;
-  }
+  dtrace("%p, %p, %p(%s)", D, base, E, E->field);
 
   /* Find the target */
-  T = _GD_FindField(D, E->in_fields[0], D->entry, D->n_entries, 0, NULL);
+  T = _GD_FindField(D, E->in_fields[0], strlen(E->in_fields[0]), D->entry,
+      D->n_entries, 0, NULL);
 
-  /* Aliases store the ulitmate target in entry[0] and the direct link
-   * in entry[1].
-   */
-  E->e->entry[0] = E->e->entry[1] = T;
+  /* Aliases store the ulitmate target in entry[0] and the direct link in
+   * entry[1] */
+  E->e->entry[1] = T;
   if (T) {
     if (T->field_type == GD_ALIAS_ENTRY) {
       if (T->e->entry[0])
         T = T->e->entry[0];
+      else if (base == T) /* loop */
+        T = NULL;
       else
-        T = _GD_ResolveAlias(D, T);
+        T = _GD_ResolveAlias(D, base, T);
     }
 
-    E->e->entry[0] = T;
   }
 
-  D->recurse_level--;
-  dreturn("%p", D->error ? NULL : E->e->entry[0]);
-  return D->error ? NULL : E->e->entry[0];
+  E->e->entry[0] = T;
+
+  if (D->error) {
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  dreturn("%p (\"%s\" > \"%s\" > \"%s\")", E->e->entry[0], E->field,
+      E->e->entry[1] ? E->e->entry[1]->field : "",
+      E->e->entry[0] ? E->e->entry[0]->field : "");
+  return E->e->entry[0];
 }
 
 void _GD_UpdateAliases(DIRFILE *D, int reset)
@@ -2394,7 +2448,7 @@ void _GD_UpdateAliases(DIRFILE *D, int reset)
     if (D->entry[u]->field_type == GD_ALIAS_ENTRY &&
         D->entry[u]->e->entry[1] == NULL)
     {
-      _GD_ResolveAlias(D, D->entry[u]);
+      _GD_ResolveAlias(D, D->entry[u], D->entry[u]);
     }
 
   dreturnvoid();
@@ -2448,8 +2502,8 @@ char *_GD_ParseFragment(FILE *restrict fp, DIRFILE *D, struct parser_state *p,
           &outstring, tok_pos);
 
     if (D->error == GD_E_OK && !match)
-      first_raw = _GD_ParseFieldSpec(D, p, n_cols, in_cols, NULL, me, 0, 1,
-          &outstring, tok_pos);
+      first_raw = _GD_ParseFieldSpec(D, p, n_cols, in_cols, strlen(in_cols[0]),
+          NULL, me, 0, 1, &outstring, tok_pos);
 
     if (D->error == GD_E_FORMAT) {
       /* call the callback for this error */
@@ -2489,10 +2543,25 @@ char *_GD_ParseFragment(FILE *restrict fp, DIRFILE *D, struct parser_state *p,
           _GD_ClearError(D);
           rescan = 1; /* rescan the modified instring */
           if (pdata.line != instring) {
-            /* a new line was malloc'd by the caller, delete our old one. */
+            /* a new line was malloc'd by the caller, delete our old one
+             * then copy it from the caller's heap to libc's to avoid
+             * problems later. */
             free(instring);
-            instring = pdata.line;
-            n = pdata.buflen;
+            if (_GD_CFree != free) {
+              instring = strdup(pdata.line);
+              _GD_CFree(pdata.line);
+              if (instring == NULL) {
+                _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
+                break;
+              }
+            } else
+              instring = pdata.line;
+
+            /* The callback isn't required to update pdata.buflen when it
+             * updates pdata.line, so just assume the buffer is as large as
+             * the string it contains
+             */
+            n = strlen(instring) + 1;
           }
           break;
         default:
@@ -2570,13 +2639,7 @@ char *gd_strtok(DIRFILE *D, const char *string) gd_nothrow
 
   dtrace("%p, \"%s\"", D, string);
 
-  if (D->flags & GD_INVALID) {
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%p", NULL);
-    return NULL;
-  }
-
-  _GD_ClearError(D);
+  GD_RETURN_IF_INVALID(D, "%p", NULL);
 
   if (string) {
     free(D->tok_base);
@@ -2600,11 +2663,20 @@ char *gd_strtok(DIRFILE *D, const char *string) gd_nothrow
     free(D->tok_base);
     D->tok_pos = D->tok_base = NULL;
     free(outstring);
-    outstring = NULL;
+    dreturn("%p", NULL);
+    return NULL;
+  }
+
+  /* move to caller's heap -- in_col and outstring always end up pointing to
+   * the same thing, so if we don't have to do this, there's nothing else we
+   * need to do */
+  if (_GD_CMalloc != malloc) {
+    outstring = _GD_CStrdup(in_col);
+    free(in_col);
+    if (outstring == NULL)
+      _GD_SetError(D, GD_E_ALLOC, 0, NULL, 0, NULL);
   }
 
-  /* in_col invariably points to outstring, so just let the caller worry about
-   * cleaning up.  Ha! */
   dreturn("\"%s\"", outstring);
   return outstring;
 }
