@@ -1,6 +1,6 @@
 /* Copyright (C) 2003-2005 C. Barth Netterfield
  * Copyright (C) 2003-2005 Theodore Kisner
- * Copyright (C) 2005-2015 D. V. Wiebe
+ * Copyright (C) 2005-2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -82,7 +82,7 @@ static size_t _GD_DoRawOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   }
 
   if (_GD_DoSeek(D, E, _GD_ef + E->e->u.raw.file[0].subenc, s0, GD_FILE_WRITE)
-      == -1)
+      < 0)
   {
     free(databuffer);
     dreturn("%i", 0);
@@ -115,17 +115,18 @@ static size_t _GD_DoLinterpOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-  
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
 
+  if (E->e->u.linterp.table_len < 0)
+    if (_GD_ReadLinterpFile(D, E)) {
+      dreturn("%i", 0);
+      return 0;
+    }
+
   /* if the table is complex valued, we can't invert it */
   if (E->e->u.linterp.complex_table) {
     _GD_SetError(D, GD_E_DOMAIN, GD_E_DOMAIN_COMPLEX, NULL, 0, NULL);
@@ -133,14 +134,6 @@ static size_t _GD_DoLinterpOut(DIRFILE *restrict D, gd_entry_t *restrict E,
     return 0;
   }
 
-  if (E->e->u.linterp.table_len < 0) {
-    _GD_ReadLinterpFile(D, E);
-    if (D->error != GD_E_OK) {
-      dreturn("%i", 0);
-      return 0;
-    }
-  }
-
   /* Check whether the LUT is monotonic */
   if (E->e->u.linterp.table_monotonic == -1) {
     E->e->u.linterp.table_monotonic = 1;
@@ -230,11 +223,6 @@ static size_t _GD_DoLincomOut(DIRFILE *restrict D, gd_entry_t *restrict E,
     return 0;
   }
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
@@ -306,11 +294,6 @@ static size_t _GD_DoBitOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
@@ -367,11 +350,6 @@ static size_t _GD_DoPhaseOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
@@ -396,11 +374,6 @@ static size_t _GD_DoRecipOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
@@ -456,11 +429,6 @@ static size_t _GD_DoPolynomOut(DIRFILE *restrict D, gd_entry_t *restrict E,
     return 0;
   }
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
@@ -562,8 +530,7 @@ static void _GD_MplexOutData(DIRFILE *restrict D, void *restrict A,
     case GD_FLOAT64:    MPLEX(  double); break;
     case GD_COMPLEX64:  MPLEXC(  float); break;
     case GD_COMPLEX128: MPLEXC( double); break;
-    default:            _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, type, NULL);
-                        break;
+    default:       _GD_InternalError(D); break;
   }
 
   dreturnvoid();
@@ -582,22 +549,12 @@ static size_t _GD_DoMplexOut(DIRFILE *restrict D, gd_entry_t *restrict E,
   dtrace("%p, %p, %" PRId64 ", %" PRIuSIZE ", 0x%X, %p", D, E,
       (int64_t)first_samp, num_samp, data_type, data_in);
 
-  if (_GD_BadInput(D, E, 0, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   if (E->e->repr[0] != GD_REPR_NONE) { /* can't write to representaions */
     _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_REPR, NULL, 0, E->in_fields[0]);
     dreturn("%i", 0);
     return 0;
   }
 
-  if (_GD_BadInput(D, E, 1, GD_NO_ENTRY, 1)) {
-    dreturn("%i", 0);
-    return 0;
-  }
-
   /* read the data to be modified */
   spf1 = _GD_GetSPF(D, E->e->entry[0]);
   spf2 = _GD_GetSPF(D, E->e->entry[1]);
@@ -695,14 +652,15 @@ size_t _GD_DoFieldOut(DIRFILE *restrict D, gd_entry_t *restrict E,
     return 0;
   }
 
-  if (!(E->flags & GD_EN_CALC)) {
+  if (!(E->flags & GD_EN_CALC))
     _GD_CalculateEntry(D, E, 1);
 
-    if (D->error) {
-      D->recurse_level--;
-      dreturn("%i", 0);
-      return 0;
-    }
+  _GD_FindInputs(D, E, 1);
+
+  if (D->error) {
+    D->recurse_level--;
+    dreturn("%i", 0);
+    return 0;
   }
 
   /* Avoid craziness */
@@ -756,10 +714,13 @@ size_t _GD_DoFieldOut(DIRFILE *restrict D, gd_entry_t *restrict E,
     case GD_MULTIPLY_ENTRY:
     case GD_DIVIDE_ENTRY:
     case GD_WINDOW_ENTRY:
+    case GD_INDIR_ENTRY:
+    case GD_SINDIR_ENTRY:
     case GD_INDEX_ENTRY:
       _GD_SetError(D, GD_E_BAD_FIELD_TYPE, GD_E_FIELD_PUT, NULL, 0, E->field);
       break;
     case GD_STRING_ENTRY:
+    case GD_SARRAY_ENTRY:
     case GD_ALIAS_ENTRY:
     case GD_NO_ENTRY:
       _GD_InternalError(D);
@@ -784,11 +745,7 @@ size_t gd_putdata64(DIRFILE* D, const char *field_code, off64_t first_frame,
       ", 0x%X, %p", D, field_code, (int64_t)first_frame, (int64_t)first_samp,
       num_frames, num_samp, data_type, data_in);
 
-  if (D->flags & GD_INVALID) {/* don't crash */
-    _GD_SetError(D, GD_E_BAD_DIRFILE, 0, NULL, 0, NULL);
-    dreturn("%i", 0);
-    return 0;
-  }
+  GD_RETURN_IF_INVALID(D, "%i", 0);
 
   if ((D->flags & GD_ACCMODE) != GD_RDWR) {
     _GD_SetError(D, GD_E_ACCMODE, 0, NULL, 0, NULL);
@@ -796,18 +753,17 @@ size_t gd_putdata64(DIRFILE* D, const char *field_code, off64_t first_frame,
     return 0;
   }
 
-  _GD_ClearError(D);
-
-  entry = _GD_FindField(D, field_code, D->entry, D->n_entries, 1, NULL);
+  entry = _GD_FindEntry(D, field_code);
 
   if (entry == NULL) {
-    _GD_SetError(D, GD_E_BAD_CODE, GD_E_CODE_MISSING, NULL, 0, field_code);
     dreturn("%i", 0);
     return 0;
   }
 
   if (entry->field_type & GD_SCALAR_ENTRY_BIT)
     _GD_SetError(D, GD_E_DIMENSION, GD_E_DIM_CALLER, NULL, 0, field_code);
+  else if (_GD_BadType(GD_DIRFILE_STANDARDS_VERSION, data_type))
+    _GD_SetError(D, GD_E_BAD_TYPE, 0, NULL, data_type, NULL);
 
   if (D->error) {
     dreturn("%i", 0);
@@ -819,7 +775,7 @@ size_t gd_putdata64(DIRFILE* D, const char *field_code, off64_t first_frame,
     first_frame = 0;
   }
 
-  if (num_frames > 0 || first_frame > 0) {
+  if (num_frames || first_frame) {
     /* get the samples per frame */
     spf = _GD_GetSPF(D, entry);
 
