@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 D. V. Wiebe
+/* Copyright (C) 2013, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -20,33 +20,20 @@
  */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-
 int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
   const char *data = "dirfile/data";
-  const char *format_data = "data RAW UINT16 2\n/FRAMEOFFSET 35\n";
-  int fd, error, r = 0;
+  int error, r = 0;
   off_t n;
   DIRFILE *D;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
-
-  fd = open(data, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, 0666);
-  write(fd, data, strlen(data));
-  close(fd);
+  MAKEFORMATFILE(format, "data RAW UINT16 2\n/FRAMEOFFSET 35\n");
+  MAKEDATAFILE(data, char, data[i], (int)strlen(data));
 
   D = gd_open(filedir, GD_RDONLY | GD_VERBOSE);
   n = gd_bof(D, "INDEX");
