@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 D. V. Wiebe
+/* Copyright (C) 2015, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -24,21 +24,19 @@ int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data =
-    "data1 CONST UINT8 1\n"
-    "data2 CONST UINT8 2\n"
-    "data3 CONST UINT8 3\n"
-    "data4 RAW UINT8 1\n";
-  int fd, error, r = 0;
+  int error, r = 0;
   const uint8_t *field_list;
   DIRFILE *D;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format,
+    "data1 CONST UINT8 1\n"
+    "data2 CONST UINT8 2\n"
+    "data3 CONST UINT8 3\n"
+    "data4 RAW UINT8 1\n"
+  );
 
   D = gd_open(filedir, GD_RDONLY);
   field_list = gd_constants(D, GD_NULL);
