@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2011, 2013 D. V. Wiebe
+/* Copyright (C) 2008-2011, 2013, 2016, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -18,42 +18,31 @@
  * along with GetData; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* Attempt to delete a field */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-#include <stdio.h>
-
 int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data = "data RAW UINT8 8\nlincom LINCOM 1 data 1 0\n";
-  int fd, ret, error, r = 0;
+  int ret, error, r = 0;
   DIRFILE *D;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format, "data RAW UINT8 8\nlincom LINCOM 1 data 1 0\n");
 
   D = gd_open(filedir, GD_RDWR);
   ret = gd_delete(D, "data", 0);
   error = gd_error(D);
+
+  CHECKI(error, GD_E_DELETE);
+  CHECKI(ret, GD_E_DELETE);
+
   gd_discard(D);
 
   unlink(format);
   rmdir(filedir);
 
-  CHECKI(error, GD_E_DELETE);
-  CHECKI(ret, -1);
-
   return r;
 }
