@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2011, 2013 D. V. Wiebe
+/* Copyright (C) 2008-2011, 2013, 2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -18,16 +18,8 @@
  * along with GetData; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* Try to read entry from an invalid DIRFILE */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-
 int main(void)
 {
   DIRFILE *D = gd_open("not a dirfile", GD_RDONLY);
@@ -39,7 +31,7 @@ int main(void)
   gd_discard(D);
 
   CHECKI(error, GD_E_BAD_DIRFILE);
-  CHECKI(n, -1);
+  CHECKI(n, GD_E_BAD_DIRFILE);
 
   return r;
 }
