@@ -1,4 +1,4 @@
-/* Copyright (C) 2014, 2015 D. V. Wiebe
+/* Copyright (C) 2014, 2015, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -20,29 +20,18 @@
  */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-#include <math.h>
-
 int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data = "string STRING String\\ data\n";
-  int fd, n, error, r = 0;
+  int n, error, r = 0;
   char s[1000];
   DIRFILE *D;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format, "string STRING String\\ data\n");
 
   D = gd_open(filedir, GD_RDONLY | GD_VERBOSE);
   n = gd_get_string(D, "string", 1000, s);
