@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2011 D. V. Wiebe
+/* Copyright (C) 2008-2011, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -22,13 +22,6 @@
  * succeed cleanly */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-
 int main(void)
 {
 #ifndef GD_LEGACY_API
@@ -36,16 +29,13 @@ int main(void)
 #else
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data = "data RAW UINT8 11\n";
-  int fd, error, r = 0;
+  int error, r = 0;
   unsigned int spf;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format, "data RAW UINT8 11\n");
 
   spf = GetSamplesPerFrame(filedir, "data", &error);
 
