@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 D. V. Wiebe
+/* Copyright (C) 2013, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -24,19 +24,17 @@ int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data =
-    "polynom POLYNOM data 1 0\n"
-    "data RAW COMPLEX128 11\n";
-  int fd, error, r = 0;
+  int error, r = 0;
   DIRFILE *D;
   gd_type_t type;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format,
+    "polynom POLYNOM data 1 0\n"
+    "data RAW COMPLEX128 11\n"
+  );
 
   D = gd_open(filedir, GD_RDONLY | GD_VERBOSE);
 
