@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2011, 2013 D. V. Wiebe
+/* Copyright (C) 2008-2011, 2013, 2016 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -18,47 +18,32 @@
  * along with GetData; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* Attempt to write LINCOM 1 */
 #include "test.h"
 
-#include <inttypes.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-
 int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data = "lincom LINCOM 1 data 0.5 3.0\n";
-  int8_t c[8];
-  int fd, i, n, error, r = 0;
+  int8_t c = 0;
+  int n, error, r = 0;
   DIRFILE *D;
 
-  memset(c, 0, 8);
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  for (i = 0; i < 8; ++i)
-    c[i] = (int8_t)(40 + i);
-
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format, "lincom LINCOM data 1 0");
 
   D = gd_open(filedir, GD_RDWR | GD_UNENCODED);
-  n = gd_putdata(D, "lincom", 5, 0, 1, 0, GD_INT8, c);
+  n = gd_putdata(D, "lincom", 0, 0, 0, 1, GD_INT8, &c);
+  CHECKI(n,0);
+
   error = gd_error(D);
+  CHECKI(error,GD_E_BAD_CODE);
 
   gd_discard(D);
 
   unlink(format);
   rmdir(filedir);
 
-  CHECKI(n,0);
-  CHECKI(error,GD_E_BAD_CODE);
   return r;
 }
