@@ -1,4 +1,4 @@
-/* Copyright (C) 2010-2011, 2013 D. V. Wiebe
+/* Copyright (C) 2010-2011, 2013, 2017 D.V. Wiebe
  *
  ***************************************************************************
  *
@@ -20,31 +20,22 @@
  */
 #include "test.h"
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <errno.h>
-
 int main(void)
 {
   const char *filedir = "dirfile";
   const char *format = "dirfile/format";
-  const char *format_data =
-    "in1 RAW UINT8 11\n"
-    "in2 RAW UINT8 13\n"
-    "div DIVIDE in1 in2\n";
-  int fd, r = 0;
+  int r = 0;
   DIRFILE *D;
   unsigned int spf;
 
   rmdirfile();
-  mkdir(filedir, 0777);
+  mkdir(filedir, 0700);
 
-  fd = open(format, O_CREAT | O_EXCL | O_WRONLY, 0666);
-  write(fd, format_data, strlen(format_data));
-  close(fd);
+  MAKEFORMATFILE(format,
+    "in1 RAW UINT8 11\n"
+    "in2 RAW UINT8 13\n"
+    "div DIVIDE in1 in2\n"
+  );
 
   D = gd_open(filedir, GD_RDONLY | GD_VERBOSE);
   spf = gd_spf(D, "div");
